@@ -5,7 +5,7 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from datetime import date
+import datetime
 import subprocess
 import getpass
 import socket
@@ -32,7 +32,9 @@ def save_properties(props, filename):
         s = str(value).replace("'", "\\'")
         f.write("%s='%s'\n" % (name, s))
     f.close()
-    print("updated %s with %s" % (filename, props))
+    print("updated %s with:" % filename)
+    for k in sorted(props.keys()):
+        print("* %s = %s" % (str(k).ljust(20), props[k]))
 
 def get_properties(filename):
     props = dict()
@@ -70,29 +72,80 @@ def get_cpuinfo():
         pass
     return "unknown"
 
-def get_compiler_info():
+def get_first_line_output(commands):
+    for cmd, valid_exit_code in commands:
+        try:
+            proc = subprocess.Popen(cmd, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
+            stdout, _ = proc.communicate()
+            if proc.returncode!=valid_exit_code:
+                print("'%s' failed with return code %s" % (cmd, proc.returncode))
+                continue
+            if not stdout:
+                print("could not get version information")
+                continue
+            out = stdout.decode('utf-8')
+            return out.splitlines()[0]
+        except:
+            pass
+    return  ""
+
+def get_compiler_version():
+    #FIXME: we assume we'll use GCC if it is on the path...
+    test_options = [("gcc --version", 0)]
     if sys.platform.startswith("win"):
-        cmd = "cl"
-    else:
-        cmd = "gcc --version"
-    proc = subprocess.Popen(cmd, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
-    stdout, _ = proc.communicate()
-    #print("get_compiler_info() %s returned %s" % (cmd, proc.returncode))
-    #print("get_compiler_info() stdout(%s)=%s" % (cmd, stdout))
-    #print("get_compiler_info() stderr(%s)=%s" % (cmd, stderr))
-    if proc.returncode!=0:
-        print("'%s' failed with return code %s" % (cmd, proc.returncode))
-        return  ""
-    if not stdout:
-        print("could not get GCC version information")
-        return  ""
-    out = stdout.decode('utf-8')
-    return out.splitlines()[0]
+        test_options.append(("cl", 0))
+        test_options.append((os.path.join(os.environ.get("VCINSTALLDIR", ""), "bin", "cl"), 0))
+    return get_first_line_output(test_options)
+
+def get_linker_version():
+    #FIXME: we assume we'll use GCC if it is on the path...
+    test_options = [("ld --version", 0)]
+    if sys.platform.startswith("win"):
+        test_options.append(("link", 1100))
+        test_options.append((os.path.join(os.environ.get("VCINSTALLDIR", ""), "bin", "link"), 1100))
+    return get_first_line_output(test_options)
+
 
 def set_prop(props, key, value):
     if value!="unknown" or props.get(key) is None:
         props[key] = value
 
+def get_platform_name():
+    #better version info than standard python platform:
+    if sys.platform.startswith("sun"):
+        #couldn't find a better way to distinguish opensolaris from solaris...
+        f = open("/etc/release")
+        data = f.read()
+        f.close()
+        if data and str(data).lower().find("opensolaris"):
+            return "OpenSolaris"
+        return "Solaris"
+    if sys.platform.find("darwin")>=0:
+        try:
+            #ie: Mac OS X 10.5.8
+            return "Mac OS X %s" % platform.mac_ver()[0]
+        except:
+            return "Mac OS X"
+    if sys.platform.find("openbsd")>=0:
+        return "OpenBSD"
+    if sys.platform.startswith("win"):
+        try:
+            import wmi            #@UnresolvedImport
+            c = wmi.WMI()
+            for os in c.Win32_OperatingSystem():
+                eq = os.Caption.find("=")
+                if eq>0:
+                    return (os.Caption[:eq]).strip()
+                return os.Caption
+        except:
+            pass
+        return "Microsoft Windows"
+    if sys.platform.find("bsd")>=0:
+        return "BSD"
+    if sys.platform.find("linux")>=0 and hasattr(platform, "linux_distribution"):
+        return "Linux %s" % (" ".join(platform.linux_distribution()))
+    return sys.platform
+
 
 def record_build_info(is_build=True):
     BUILD_INFO_FILE = "./xpra/build_info.py"
@@ -101,15 +154,19 @@ def record_build_info(is_build=True):
     if is_build:
         set_prop(props, "BUILT_BY", getpass.getuser())
         set_prop(props, "BUILT_ON", socket.gethostname())
-        set_prop(props, "BUILD_DATE", date.today().isoformat())
+        set_prop(props, "BUILD_DATE", datetime.date.today().isoformat())
+        set_prop(props, "BUILD_TIME", datetime.datetime.now().strftime("%H:%M"))
         set_prop(props, "BUILD_CPU", get_cpuinfo())
         set_prop(props, "BUILD_BIT", platform.architecture()[0])
+        set_prop(props, "BUILD_OS", get_platform_name())
         try:
             from Cython import __version__ as cython_version
         except:
             cython_version = "unknown"
+        set_prop(props, "PYTHON_VERSION", sys.version_info[:3])
         set_prop(props, "CYTHON_VERSION", cython_version)
-        set_prop(props, "COMPILER_INFO", get_compiler_info())
+        set_prop(props, "COMPILER_VERSION", get_compiler_version())
+        set_prop(props, "LINKER_VERSION", get_linker_version())
         set_prop(props, "RELEASE_BUILD", not bool(os.environ.get("BETA", "")))
     save_properties(props, BUILD_INFO_FILE)
 
@@ -199,10 +256,13 @@ def get_svn_props():
     props["LOCAL_MODIFICATIONS"] = changes
     return props
 
+SRC_INFO_FILE = "./xpra/src_info.py"
 def record_src_info():
-    SRC_INFO_FILE = "./xpra/src_info.py"
     update_properties(get_svn_props(), SRC_INFO_FILE)
 
+def has_src_info():
+    return os.path.exists(SRC_INFO_FILE) and os.path.isfile(SRC_INFO_FILE)
+
 def main():
     record_build_info(True)
 
