@@ -1,14 +1,14 @@
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2012 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
 # Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import pygtk
 pygtk.require('2.0')
 import gtk
 
-from wimpiggy.log import Logger
+from xpra.log import Logger
 log = Logger()
 
 from tests.xpra.gl.gl_simple_backing import GLTestBacking
