@@ -17,7 +17,8 @@ log = Logger("client")
 
 from xpra.net.protocol import Protocol, use_lz4, use_rencode, get_network_caps
 from xpra.scripts.config import ENCRYPTION_CIPHERS
-from xpra.version_util import version_compat_check, add_version_info, get_platform_info
+from xpra.scripts.main import SOCKET_TIMEOUT
+from xpra.version_util import version_compat_check, get_version_info, get_platform_info, local_version
 from xpra.platform.features import GOT_PASSWORD_PROMPT_SUGGESTION
 from xpra.platform.info import get_name
 from xpra.os_util import get_hex_uuid, get_machine_id, get_user_uuid, load_binary_file, SIGNAMES, strtobytes, bytestostr
@@ -39,8 +40,17 @@ EXIT_UNSUPPORTED = 12
 EXIT_REMOTE_ERROR = 13
 
 
-DEFAULT_TIMEOUT = 20*1000
+DEFAULT_TIMEOUT = (10 + SOCKET_TIMEOUT)*1000        #in millis
 ALLOW_UNENCRYPTED_PASSWORDS = os.environ.get("XPRA_ALLOW_UNENCRYPTED_PASSWORDS", "0")=="1"
+DETECT_LEAKS = os.environ.get("XPRA_DETECT_LEAKS", "0")=="1"
+
+
+if sys.version < '3':
+    def b(x):
+        return x
+else:
+    def b(x):
+        return bytes(x, 'UTF-8')
 
 
 class XpraClientBase(object):
@@ -105,6 +115,19 @@ class XpraClientBase(object):
         self.speed = opts.speed
         self.min_speed = opts.min_speed
 
+        if DETECT_LEAKS:
+            from xpra.util import detect_leaks
+            detailed = []
+            #example: warning, uses ugly direct import:
+            #try:
+            #    from xpra.x11.bindings.ximage import XShmImageWrapper       #@UnresolvedImport
+            #    detailed.append(XShmImageWrapper)
+            #except:
+            #    pass
+            print_leaks = detect_leaks(log, detailed)
+            self.timeout_add(10*1000, print_leaks)
+
+
     def timeout_add(self, *args):
         raise Exception("override me!")
 
@@ -181,7 +204,7 @@ class XpraClientBase(object):
             hello["challenge_response"] = challenge_response
             if client_salt:
                 hello["challenge_client_salt"] = client_salt
-        log.debug("send_hello(%s) packet=%s", binascii.hexlify(challenge_response or ""), hello)
+        log.debug("send_hello(%s) packet=%s", binascii.hexlify(b(challenge_response or "")), hello)
         self.send("hello", hello)
         self.timeout_add(DEFAULT_TIMEOUT, self.verify_connected)
 
@@ -194,6 +217,7 @@ class XpraClientBase(object):
     def make_hello_base(self):
         capabilities = get_network_caps()
         capabilities.update({
+                "version"               : local_version,
                 "encoding.generic"      : True,
                 "namespace"             : True,
                 "hostname"              : socket.gethostname(),
@@ -206,8 +230,8 @@ class XpraClientBase(object):
                 })
         if self.display:
             capabilities["display"] = self.display
-        capabilities.update(get_platform_info())
-        add_version_info(capabilities)
+        capabilities.update(get_platform_info("platform"))
+        capabilities.update(get_version_info("build"))
         mid = get_machine_id()
         if mid:
             capabilities["machine_id"] = mid
@@ -407,11 +431,11 @@ class XpraClientBase(object):
             self.warn_and_quit(EXIT_NO_AUTHENTICATION, "the server did not request our password")
             return
         try:
-            self.server_capabilities = packet[1]
+            self.server_capabilities = typedict(packet[1])
             log("processing hello from server: %s", self.server_capabilities)
-            c = typedict(self.server_capabilities)
-            self.parse_server_capabilities(c)
+            self.parse_server_capabilities(self.server_capabilities)
         except Exception, e:
+            log.info("error in hello packet", exc_info=True)
             self.warn_and_quit(EXIT_FAILURE, "error processing hello packet from server: %s" % e)
 
     def capsget(self, capabilities, key, default):
@@ -424,12 +448,20 @@ class XpraClientBase(object):
         self._remote_machine_id = c.strget("machine_id")
         self._remote_uuid = c.strget("uuid")
         self._remote_version = c.strget("version")
+        self._remote_version = c.strget("build.version", self._remote_version)
         self._remote_revision = c.strget("revision")
         self._remote_revision = c.strget("build.revision", self._remote_revision)
         self._remote_platform = c.strget("platform")
         self._remote_platform_release = c.strget("platform.release")
         self._remote_platform_platform = c.strget("platform.platform")
-        self._remote_platform_linux_distribution = c.get("platform.linux_distribution")
+        #linux distribution is a tuple of different types, ie: ('Linux Fedora' , 20, 'Heisenbug')
+        pld = c.listget("platform.linux_distribution")
+        if pld and len(pld)==3:
+            def san(v):
+                if type(v)==int:
+                    return v
+                return bytestostr(v)
+            self._remote_platform_linux_distribution = [san(x) for x in pld]
         verr = version_compat_check(self._remote_version)
         if verr is not None:
             self.warn_and_quit(EXIT_INCOMPATIBLE_VERSION, "incompatible remote version '%s': %s" % (self._remote_version, verr))
@@ -474,6 +506,8 @@ class XpraClientBase(object):
         try:
             handler = None
             packet_type = packet[0]
+            if packet_type!=int:
+                packet_type = bytestostr(packet_type)
             handler = self._packet_handlers.get(packet_type)
             if handler:
                 handler(packet)
