@@ -5,19 +5,17 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
 import re
 import sys
 
 from xpra.client.client_widget_base import ClientWidgetBase
+from xpra.util import typedict
 from xpra.log import Logger
 log = Logger("window")
 plog = Logger("paint")
 focuslog = Logger("focus")
 mouselog = Logger("mouse")
 
-#pretend to draw the windows, but don't actually do anything
-USE_FAKE_BACKING = os.environ.get("XPRA_USE_FAKE_BACKING", "0")=="1"
 if sys.version < '3':
     import codecs
     def u(x):
@@ -29,15 +27,14 @@ else:
 
 class ClientWindowBase(ClientWidgetBase):
 
-    def __init__(self, client, group_leader, wid, x, y, w, h, metadata, override_redirect, client_properties, auto_refresh_delay, border):
-        log("%s%s", type(self), (client, group_leader, wid, x, y, w, h, metadata, override_redirect, client_properties, auto_refresh_delay))
-        ClientWidgetBase.__init__(self, client, wid)
+    def __init__(self, client, group_leader, wid, x, y, w, h, metadata, override_redirect, client_properties, border):
+        log("%s%s", type(self), (client, group_leader, wid, x, y, w, h, metadata, override_redirect, client_properties))
+        ClientWidgetBase.__init__(self, client, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
         self._pos = (x, y)
         self._size = (w, h)
         self._client_properties = client_properties
-        self._auto_refresh_delay = auto_refresh_delay
         self.border = border
         self.button_state = {}
 
@@ -49,23 +46,23 @@ class ClientWindowBase(ClientWidgetBase):
 
     def init_window(self, metadata):
         self._backing = None
-        self._metadata = {}
-        self._refresh_timer = None
-        self._refresh_min_pixels = -1
-        self._refresh_ignore_sequence = -1
+        self._metadata = typedict()
         # used for only sending focus events *after* the window is mapped:
         self._been_mapped = False
         self._override_redirect_windows = []
         self.update_metadata(metadata)
 
+    def new_backing(self, w, h):
+        backing_class = self.get_backing_class()
+        assert backing_class is not None
+        self._backing = self.make_new_backing(backing_class, w, h)
+        self._backing.border = self.border
+
     def destroy(self):
         #ensure we clear reference to other windows:
         self.group_leader = None
         self._override_redirect_windows = []
         self._metadata = {}
-        if self._refresh_timer:
-            self.source_remove(self._refresh_timer)
-            self._refresh_timer = None
         if self._backing:
             self._backing.close()
             self._backing = None
@@ -73,6 +70,9 @@ class ClientWindowBase(ClientWidgetBase):
 
     def setup_window(self):
         self.new_backing(*self._size)
+        #tell the server about the encoding capabilities of this backing instance:
+        #"rgb_formats", "full_csc_modes", "csc_modes":
+        self._client_properties.update(self._backing.get_encoding_properties())
 
 
     def send(self, *args):
@@ -106,7 +106,7 @@ class ClientWindowBase(ClientWidgetBase):
 
     def update_metadata(self, metadata):
         #normalize window-type:
-        window_type = metadata.get("window-type")
+        window_type = metadata.strlistget("window-type")
         if window_type is not None:
             #normalize the window type for servers that don't do "generic_window_types"
             window_type = [x.replace("_NET_WM_WINDOW_TYPE_", "").replace("_NET_WM_TYPE_", "") for x in window_type]
@@ -114,7 +114,7 @@ class ClientWindowBase(ClientWidgetBase):
 
         self._metadata.update(metadata)
         if not self.is_realized():
-            self.set_wmclass(*self._metadata.get("class-instance",
+            self.set_wmclass(*self._metadata.strlistget("class-instance",
                                                  ("xpra", "Xpra")))
         self.set_metadata(metadata)
 
@@ -130,7 +130,7 @@ class ClientWindowBase(ClientWidgetBase):
                         atvar = match.group(0)          #ie: '@title@'
                         var = atvar[1:len(atvar)-1]     #ie: 'title'
                         default_value = default_values.get(var, u("<unknown %s>") % var)
-                        value = self._metadata.get(var, default_value)
+                        value = self._metadata.strget(var, default_value)
                         if sys.version<'3':
                             value = value.decode("utf-8")
                         return value
@@ -142,11 +142,11 @@ class ClientWindowBase(ClientWidgetBase):
             self.set_title(utf8_title)
 
         if "icon-title" in metadata:
-            icon_title = metadata["icon-title"]
+            icon_title = metadata.strget("icon-title")
             self.set_icon_name(icon_title)
 
         if "size-constraints" in metadata:
-            size_metadata = metadata["size-constraints"]
+            size_metadata = typedict(metadata.dictget("size-constraints"))
             hints = {}
             for (a, h1, h2) in [
                 ("maximum-size", "max_width", "max_height"),
@@ -154,7 +154,7 @@ class ClientWindowBase(ClientWidgetBase):
                 ("base-size", "base_width", "base_height"),
                 ("increment", "width_inc", "height_inc"),
                 ]:
-                v = size_metadata.get(a)
+                v = size_metadata.intlistget(a)
                 if v:
                     v1, v2 = v
                     hints[h1], hints[h2] = int(v1), int(v2)
@@ -162,7 +162,7 @@ class ClientWindowBase(ClientWidgetBase):
                 ("minimum-aspect-ratio", "min_aspect"),
                 ("maximum-aspect-ratio", "max_aspect"),
                 ]:
-                v = size_metadata.get(a)
+                v = size_metadata.intlistget(a)
                 if v:
                     v1, v2 = v
                     hints[h] = float(v1)/float(v2)
@@ -174,43 +174,55 @@ class ClientWindowBase(ClientWidgetBase):
             #gravity = size_metadata.get("gravity")
 
         if "icon" in metadata:
-            width, height, coding, data = metadata["icon"]
+            width, height, coding, data = metadata.listget("icon")
             self.update_icon(width, height, coding, data)
 
         if "transient-for" in metadata:
-            wid = metadata["transient-for"]
+            wid = metadata.get("transient-for", -1)
             self.apply_transient_for(wid)
 
         if "modal" in metadata:
-            modal = bool(metadata["modal"])
+            modal = metadata.boolget("modal")
             self.set_modal(modal)
 
         #apply window-type hint if window has not been mapped yet:
         if "window-type" in metadata and not self.is_mapped():
-            window_types = metadata["window-type"]
+            window_types = metadata.strlistget("window-type")
             self.set_window_type(window_types)
 
         if "role" in metadata:
-            role = metadata["role"]
+            role = metadata.strget("role")
             self.set_role(role)
 
         if "xid" in metadata:
-            xid = metadata["xid"]
+            xid = metadata.strget("xid")
             self.set_xid(xid)
 
+        if "opacity" in metadata:
+            opacity = metadata.intget("opacity")
+            if opacity<0:
+                opacity = 1
+            else:
+                opacity = min(1, opacity/float(0xffffffff))
+            #requires gtk>=2.12!
+            if hasattr(self, "set_opacity"):
+                self.set_opacity(opacity)
+
         if "has-alpha" in metadata:
-            self._has_alpha = bool(metadata["has-alpha"])
-            self.set_alpha()
+            new_alpha = metadata.boolget("has-alpha")
+            if new_alpha!=self._has_alpha:
+                log.warn("window %s changed its alpha flag from %s to %s (unsupported)", self._id, self._has_alpha, new_alpha)
+                self._has_alpha = new_alpha
 
         if "maximized" in metadata:
-            maximized = bool(metadata["maximized"])
+            maximized = metadata.boolget("maximized")
             if maximized:
                 self.maximize()
             else:
                 self.unmaximize()
 
         if "fullscreen" in metadata:
-            fullscreen = bool(metadata["fullscreen"])
+            fullscreen = metadata.boolget("fullscreen")
             self.set_fullscreen(fullscreen)
 
 
@@ -229,9 +241,6 @@ class ClientWindowBase(ClientWidgetBase):
     def set_fullscreen(self, fullscreen):
         pass
 
-    def set_alpha(self):
-        pass
-
     def set_xid(self, xid):
         pass
 
@@ -253,26 +262,8 @@ class ClientWindowBase(ClientWidgetBase):
         assert self._backing, "window %s has no backing!" % self._id
         def after_draw_refresh(success):
             plog("after_draw_refresh(%s) %sx%s at %sx%s encoding=%s, options=%s", success, width, height, x, y, coding, options)
-            if self._backing is None:
-                return
-            if success and self._backing.draw_needs_refresh:
+            if success and self._backing and self._backing.draw_needs_refresh:
                 self.queue_draw(x, y, width, height)
-            #clear the auto refresh if enough pixels were sent (arbitrary limit..)
-            if success and self._refresh_timer and width*height>=self._refresh_min_pixels:
-                self.source_remove(self._refresh_timer)
-                self._refresh_timer = None
-            #if we need to set a refresh timer, do it:
-            is_hq = options.get("quality", 0)>=95
-            is_lossy = coding in ("jpeg", "vp8", "h264")
-            if self._refresh_timer is None and self._auto_refresh_delay>0 and is_lossy and not is_hq:
-                #make sure our own refresh does not make us fire again
-                #FIXME: this should be per-window!
-                if self._refresh_ignore_sequence<packet_sequence:
-                    #NOTE: for x264 and vpx, we always get full frames (whole window refresh)
-                    #this is not the case with jpeg but since jpeg does not switch the encoding on the fly, we're ok
-                    self._refresh_min_pixels = width*height
-                    self._refresh_ignore_sequence = packet_sequence+1
-                    self._refresh_timer = self.timeout_add(int(1000 * self._auto_refresh_delay), self.refresh_window)
         callbacks.append(after_draw_refresh)
         self._backing.draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
 
@@ -296,7 +287,7 @@ class ClientWindowBase(ClientWidgetBase):
 
 
     def _unfocus(self):
-        focuslog("_unfocus() wid=%s", self._id)
+        focuslog("_unfocus() wid=%s, focused=%s", self._id, self._client._focused)
         if self._client._focused==self._id:
             self._client.update_focus(self._id, False)
 
@@ -350,5 +341,8 @@ class ClientWindowBase(ClientWidgetBase):
     def do_scroll_event(self, event):
         if self._client.readonly:
             return
-        self._button_action(self.SCROLL_MAP[event.direction], event, True)
-        self._button_action(self.SCROLL_MAP[event.direction], event, False)
+        button_mapping = self.SCROLL_MAP.get(event.direction, -1)
+        mouselog("do_scroll_event direction=%s, button_mapping=%s", event.direction, button_mapping)
+        if button_mapping>=0:
+            self._button_action(button_mapping, event, True)
+            self._button_action(button_mapping, event, False)
