@@ -14,21 +14,53 @@ required_extensions = ["GL_ARB_texture_rectangle", "GL_ARB_vertex_program"]
 
 
 BLACKLIST = {"vendor" : ["nouveau", "Humper", "VMware, Inc."]}
+if sys.platform.startswith("darwin"):
+    #crashes were reported with the Intel driver on OSX
+    BLACKLIST["vendor"].append("Intel Inc.")
 
-DEFAULT_HAS_ALPHA = not sys.platform.startswith("win") and not sys.platform.startswith("darwin")
-HAS_ALPHA = os.environ.get("XPRA_ALPHA", DEFAULT_HAS_ALPHA) in (True, "1")
+
+DEFAULT_ALPHA = not sys.platform.startswith("win") and not sys.platform.startswith("darwin")
+GL_ALPHA_SUPPORTED = os.environ.get("XPRA_ALPHA", DEFAULT_ALPHA) in (True, "1")
 DEFAULT_DOUBLE_BUFFERED = 0
 if sys.platform.startswith("win"):
     #needed on win32?
     DEFAULT_DOUBLE_BUFFERED = 1
 DOUBLE_BUFFERED = os.environ.get("XPRA_OPENGL_DOUBLE_BUFFERED", str(DEFAULT_DOUBLE_BUFFERED))=="1"
 
+def get_visual_name(visual):
+    import gtk.gdk
+    if not visual:
+        return ""
+    return {
+           gtk.gdk.VISUAL_STATIC_GRAY   : "STATIC_GRAY",
+           gtk.gdk.VISUAL_GRAYSCALE     : "GRAYSCALE",
+           gtk.gdk.VISUAL_STATIC_COLOR  : "STATIC_COLOR",
+           gtk.gdk.VISUAL_PSEUDO_COLOR  : "PSEUDO_COLOR",
+           gtk.gdk.VISUAL_TRUE_COLOR    : "TRUE_COLOR",
+           gtk.gdk.VISUAL_DIRECT_COLOR  : "DIRECT_COLOR"}.get(visual.type, "unknown")
 
-def get_DISPLAY_MODE():
+def get_visual_byte_order(visual):
+    import gtk.gdk
+    if not visual:
+        return ""
+    return {
+            gtk.gdk.LSB_FIRST   : "LSB",
+            gtk.gdk.MSB_FIRST   : "MSB"}.get(visual.byte_order, "unknown")
+
+def visual_to_str(visual):
+    if not visual:
+        return ""
+    d = {"type"         : get_visual_name(visual),
+         "byte_order"   : get_visual_byte_order(visual)}
+    for k in ("bits_per_rgb", "depth"):
+        d[k] = getattr(visual, k)
+    return str(d)
+
+def get_DISPLAY_MODE(want_alpha=GL_ALPHA_SUPPORTED):
     import gtk.gdkgl
     #gtk.gdkgl.MODE_DEPTH
     mode = 0
-    if HAS_ALPHA:
+    if want_alpha:
         mode = mode | gtk.gdkgl.MODE_RGBA | gtk.gdkgl.MODE_ALPHA
     else:
         mode = mode | gtk.gdkgl.MODE_RGB
@@ -58,9 +90,6 @@ def raise_error(msg):
     raise ImportError(msg)
 gl_check_error = raise_error
 
-if sys.version > '3':
-    unicode = str           #@ReservedAssignment
-
 
 def check_functions(*functions):
     missing = []
@@ -97,6 +126,8 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         fhlogger = redirect_log('OpenGL.formathandler')
         elogger = redirect_log('OpenGL.extensions')
         alogger = redirect_log('OpenGL.acceleratesupport')
+        arlogger = redirect_log('OpenGL.arrays')
+        clogger = redirect_log('OpenGL.converters')
 
         import OpenGL
         props["pyopengl"] = OpenGL.__version__
@@ -175,6 +206,16 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             glMultiTexCoord2i, \
             glVertex2i, glEnd)
 
+        glEnablei = None
+        try:
+            from OpenGL.GL import glEnablei
+        except:
+            pass
+        if not bool(glEnablei):
+            log.warn("OpenGL glEnablei is not available, disabling transparency")
+            global GL_ALPHA_SUPPORTED
+            GL_ALPHA_SUPPORTED = False
+
         #check for framebuffer functions we need:
         from OpenGL.GL.ARB.framebuffer_object import GL_FRAMEBUFFER, \
             GL_COLOR_ATTACHMENT0, glGenFramebuffers, glBindFramebuffer, glFramebufferTexture2D
@@ -206,9 +247,16 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
         check_functions(glGenProgramsARB, glDeleteProgramsARB, glBindProgramARB, glProgramStringARB)
 
         try:
-            from OpenGL.GL import GL_MAX_RECTANGLE_TEXTURE_SIZE, GL_MAX_TEXTURE_SIZE
+            from OpenGL.GL import GL_MAX_TEXTURE_SIZE
             texture_size = glGetInteger(GL_MAX_TEXTURE_SIZE)
-            rect_texture_size = glGetInteger(GL_MAX_RECTANGLE_TEXTURE_SIZE)
+            #this one may be missing?
+            rect_texture_size = texture_size
+            try:
+                from OpenGL.GL import GL_MAX_RECTANGLE_TEXTURE_SIZE
+                rect_texture_size = glGetInteger(GL_MAX_RECTANGLE_TEXTURE_SIZE)
+            except ImportError, e:
+                log("OpenGL: %s", e)
+                log("using GL_MAX_TEXTURE_SIZE=%s as default", texture_size)
         except Exception, e:
             emsg = str(e)
             if hasattr(e, "description"):
@@ -227,7 +275,9 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             msg = x.getMessage().replace("No OpenGL_accelerate module loaded: ", "")
             if msg=="No module named OpenGL_accelerate":
                 msg = "missing accelerate module"
-            log.info("PyOpenGL warning: %s" % msg)
+            if msg!="OpenGL_accelerate module loaded":
+                msg = "PyOpenGL warning: %s" % msg
+            log.info(msg)
 
         #format handler messages:
         STRIP_LOG_MESSAGE = "Unable to load registered array format handler "
@@ -254,15 +304,31 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             if not p:
                 log.info(msg)
 
+        missing_accelerators = []
+        STRIP_AR_HEAD = "Unable to load"
+        STRIP_AR_TAIL = "from OpenGL_accelerate"
+        for x in arlogger.handlers[0].records+clogger.handlers[0].records:
+            msg = x.getMessage()
+            if msg.startswith(STRIP_AR_HEAD) and msg.endswith(STRIP_AR_TAIL):
+                m = msg[len(STRIP_AR_HEAD):-len(STRIP_AR_TAIL)].strip()
+                m = m.replace("accelerators", "").replace("accelerator", "").strip()
+                missing_accelerators.append(m)
+                continue
+            log.info(msg)
+        if missing_accelerators:
+            log.info("OpenGL accelerate missing: %s", ", ".join(missing_accelerators))
+
         def restore_logger(logger):
             logger.handlers = logger.saved_handlers
             logger.propagate = logger.saved_propagate
         restore_logger(fhlogger)
         restore_logger(elogger)
         restore_logger(alogger)
+        restore_logger(arlogger)
+        restore_logger(clogger)
         gldrawable.gl_end()
 
-def check_support(min_texture_size=0, force_enable=False):
+def check_support(min_texture_size=0, force_enable=False, check_colormap=False):
     try:
         from xpra.platform.paths import get_icon_dir
         opengl_icon = os.path.join(get_icon_dir(), "opengl.png")
@@ -280,6 +346,8 @@ def check_support(min_texture_size=0, force_enable=False):
     assert gtk.gdkgl is not None and gtk.gtkgl is not None
     log("pygdkglext version=%s", gtk.gdkgl.pygdkglext_version)
     props["pygdkglext_version"] = gtk.gdkgl.pygdkglext_version
+    log("gtkglext_version=%s", gtk.gdkgl.gdkglext_version)
+    props["gdkglext_version"] = gtk.gdkgl.gdkglext_version
     log("pygdkglext OpenGL version=%s", gtk.gdkgl.query_version())
     props["gdkgl_version"] = gtk.gdkgl.query_version()
     display_mode = get_DISPLAY_MODE()
@@ -309,7 +377,7 @@ def check_support(min_texture_size=0, force_enable=False):
         w.set_decorated(False)
         vbox = gtk.VBox()
         width, height = 32, 32
-        if opengl_icon and os.path.exists(opengl_icon):
+        if opengl_icon and os.path.exists(opengl_icon) and hasattr(gtk, "image_new_from_pixbuf"):
             pixbuf = gtk.gdk.pixbuf_new_from_file(opengl_icon)
             image = gtk.image_new_from_pixbuf(pixbuf)
             vbox.add(image)
@@ -329,6 +397,16 @@ def check_support(min_texture_size=0, force_enable=False):
         glcontext = glarea.get_gl_context()
 
         gl_props = check_GL_support(gldrawable, glcontext, min_texture_size, force_enable)
+
+        if check_colormap:
+            rgb_visual = w.get_screen().get_rgb_visual()
+            rgba_visual = w.get_screen().get_rgba_visual()
+            gl_props["rgb_visual"] = visual_to_str(rgb_visual)
+            gl_props["rgba_visual"] = visual_to_str(rgba_visual)
+            #i = 0
+            #for v in w.get_screen().list_visuals():
+            #    gl_props["visual[%s]" % i] = visual_to_str(v)
+            #    i += 1
     finally:
         if w:
             w.destroy()
@@ -339,9 +417,11 @@ def check_support(min_texture_size=0, force_enable=False):
 
 def main():
     from xpra.platform import init,clean
+    from xpra.util import pver
     try:
         init("OpenGL-Check")
-        if "-v" in sys.argv or "--verbose" in sys.argv:
+        verbose = "-v" in sys.argv or "--verbose" in sys.argv
+        if verbose:
             log.enable_debug()
         #replace ImportError with a log message:
         global gl_check_error
@@ -350,7 +430,7 @@ def main():
             log.error("ERROR: %s", msg)
             errors.append(msg)
         gl_check_error = log_error
-        props = check_support(True)
+        props = check_support(0, True, verbose)
         log.info("")
         if len(errors)>0:
             log.info("OpenGL errors:")
@@ -358,10 +438,11 @@ def main():
                 log.info("  %s", e)
         log.info("")
         log.info("OpenGL properties:")
-        for k,v in props.items():
+        for k in sorted(props.keys()):
+            v = props[k]
             #skip not human readable:
             if k not in ("extensions", "glconfig"):
-                log.info("  %s : %s", str(k).ljust(24), v)
+                log.info("* %s : %s", str(k).ljust(24), pver(v))
         return len(errors)
     finally:
         clean()
