@@ -27,9 +27,11 @@ from xpra.client.gtk2.tray_menu import GTK2TrayMenu
 from xpra.gtk_common.cursor_names import cursor_names
 from xpra.client.window_border import WindowBorder
 from xpra.log import Logger
+
 log = Logger("gtk", "client")
 cursorlog = Logger("gtk", "client", "cursor")
 clipboardlog = Logger("gtk", "client", "clipboard")
+grablog = Logger("gtk", "client", "grab")
 
 from xpra.client.gtk2.border_client_window import BorderClientWindow
 from xpra.client.gtk2.client_window import ClientWindow
@@ -47,9 +49,6 @@ missing_cursor_names = set()
 
 class XpraClient(GTKXpraClient):
 
-    WINDOW_TOPLEVEL = gdk.WINDOW_TOPLEVEL
-    INPUT_ONLY = gdk.INPUT_ONLY
-
     def __init__(self):
         GTKXpraClient.__init__(self)
         self.border = None
@@ -60,7 +59,6 @@ class XpraClient(GTKXpraClient):
         #avoid ugly "not implemented" warning on win32
         self.supports_group_leader = not sys.platform.startswith("win")
 
-        self.window_with_grab = None
         self._ref_to_group_leader = {}
         self._group_leader_wids = {}
 
@@ -93,7 +91,7 @@ class XpraClient(GTKXpraClient):
             color = gtk.gdk.color_parse(color_str)
         except Exception, e:
             log.warn("invalid color specified: %s (%s)", color_str, e)
-            return
+            color = gtk.gdk.Color("red")
         alpha = 0.6
         size = 4
         if len(parts)==2:
@@ -161,19 +159,6 @@ class XpraClient(GTKXpraClient):
         gtk.gdk.notify_startup_complete()
 
 
-    def do_get_pixbuf(self, icon_filename):
-        if not hasattr(gdk, "pixbuf_new_from_file"):
-            return None
-        return  gdk.pixbuf_new_from_file(icon_filename)
-
-    def do_get_image(self, pixbuf, size=None):
-        if not hasattr(gtk, "image_new_from_pixbuf"):
-            return None
-        if size:
-            pixbuf = pixbuf.scale_simple(size, size, gdk.INTERP_BILINEAR)
-        return  gtk.image_new_from_pixbuf(pixbuf)
-
-
     def get_tray_menu_helper_classes(self):
         tmhc = GTKXpraClient.get_tray_menu_helper_classes(self)
         tmhc.append(GTK2TrayMenu)
@@ -288,13 +273,6 @@ class XpraClient(GTKXpraClient):
 
     def process_ui_capabilities(self, capabilities):
         GTKXpraClient.process_ui_capabilities(self, capabilities)
-        if self.server_randr:
-            display = gdk.display_get_default()
-            i=0
-            while i<display.get_n_screens():
-                screen = display.get_screen(i)
-                screen.connect("size-changed", self.screen_size_changed)
-                i += 1
         global UI_watcher
         UI_watcher.start()
         #if server supports it, enable UI thread monitoring workaround when needed:
@@ -307,45 +285,8 @@ class XpraClient(GTKXpraClient):
             UI_watcher.add_fail_callback(UI_failed)
 
 
-    def get_screen_sizes(self):
-        display = gdk.display_get_default()
-        i=0
-        screen_sizes = []
-        while i<display.get_n_screens():
-            screen = display.get_screen(i)
-            j = 0
-            monitors = []
-            while j<screen.get_n_monitors():
-                geom = screen.get_monitor_geometry(j)
-                plug_name = ""
-                if hasattr(screen, "get_monitor_plug_name"):
-                    plug_name = screen.get_monitor_plug_name(j) or ""
-                wmm = -1
-                if hasattr(screen, "get_monitor_width_mm"):
-                    wmm = screen.get_monitor_width_mm(j)
-                hmm = -1
-                if hasattr(screen, "get_monitor_height_mm"):
-                    hmm = screen.get_monitor_height_mm(j)
-                monitor = plug_name, geom.x, geom.y, geom.width, geom.height, wmm, hmm
-                monitors.append(monitor)
-                j += 1
-            work_x, work_y = 0, 0
-            work_width, work_height = screen.get_width(), screen.get_height()
-            if not sys.platform.startswith("win"):
-                try:
-                    p = gtk.gdk.atom_intern('_NET_WORKAREA')
-                    root = screen.get_root_window()
-                    work_x, work_y, work_width, work_height = root.property_get(p)[2][:4]
-                except:
-                    pass
-            item = (screen.make_display_name(), screen.get_width(), screen.get_height(),
-                        screen.get_width_mm(), screen.get_height_mm(),
-                        monitors,
-                        work_x, work_y, work_width, work_height)
-            screen_sizes.append(item)
-            i += 1
-        return screen_sizes
-
+    def get_root_window(self):
+        return gdk.get_default_root_window()
 
     def get_root_size(self):
         return gdk.get_default_root_window().get_size()
@@ -421,24 +362,15 @@ class XpraClient(GTKXpraClient):
             window.present()
 
 
-    def _process_pointer_grab(self, packet):
-        wid = packet[1]
-        window = self._id_to_window.get(wid)
-        if window:
-            log("grabbing %s", window)
-            mask = gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.POINTER_MOTION_MASK  | gtk.gdk.POINTER_MOTION_HINT_MASK | gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK
-            gtk.gdk.pointer_grab(window.gdk_window(), owner_events=True, event_mask=mask)
-            #also grab the keyboard so the user won't Alt-Tab away:
-            gtk.gdk.keyboard_grab(window.gdk_window(), owner_events=False)
-            self.window_with_grab = wid
+    def window_grab(self, window):
+        mask = gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.POINTER_MOTION_MASK  | gtk.gdk.POINTER_MOTION_HINT_MASK | gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK
+        gtk.gdk.pointer_grab(window.gdk_window(), owner_events=True, event_mask=mask)
+        #also grab the keyboard so the user won't Alt-Tab away:
+        gtk.gdk.keyboard_grab(window.gdk_window(), owner_events=False)
 
-    def _process_pointer_ungrab(self, packet):
-        wid = packet[1]
-        window = self._id_to_window.get(wid)
-        log("ungrabbing %s", window)
+    def window_ungrab(self):
         gtk.gdk.pointer_ungrab()
         gtk.gdk.keyboard_ungrab()
-        self.window_with_grab = None
 
 
     def init_opengl(self, enable_opengl):
@@ -482,16 +414,16 @@ class XpraClient(GTKXpraClient):
     def get_group_leader(self, metadata, override_redirect):
         if not self.supports_group_leader:
             return None
-        wid = metadata.get("transient-for")
+        wid = metadata.intget("transient-for", -1)
         if wid>0:
             client_window = self._id_to_window.get(wid)
             if client_window:
                 gdk_window = client_window.gdk_window()
                 if gdk_window:
                     return gdk_window
-        pid = metadata.get("pid", -1)
-        leader_xid = metadata.get("group-leader-xid")
-        leader_wid = metadata.get("group-leader-wid")
+        pid = metadata.intget("pid", -1)
+        leader_xid = metadata.intget("group-leader-xid", -1)
+        leader_wid = metadata.intget("group-leader-wid", -1)
         group_leader_window = self._id_to_window.get(leader_wid)
         if group_leader_window:
             #leader is another managed window
@@ -514,7 +446,7 @@ class XpraClient(GTKXpraClient):
             return group_leader_window
         #we need to create one:
         title = "%s group leader for %s" % (self.session_name or "Xpra", pid)
-        group_leader_window = gdk.Window(None, 1, 1, self.WINDOW_TOPLEVEL, 0, self.INPUT_ONLY, title)
+        group_leader_window = gdk.Window(None, 1, 1, gdk.WINDOW_TOPLEVEL, 0, gdk.INPUT_ONLY, title)
         self._ref_to_group_leader[refkey] = group_leader_window
         #spec says window should point to itself
         group_leader_window.set_group(group_leader_window)
@@ -524,12 +456,6 @@ class XpraClient(GTKXpraClient):
 
     def destroy_window(self, wid, window):
         #override so we can cleanup the group-leader if needed,
-        #and lose the grab
-        if self.window_with_grab==wid:
-            log("destroying window %s which has grab, ungrabbing!", wid)
-            gtk.gdk.pointer_ungrab()
-            gtk.gdk.keyboard_ungrab()
-            self.window_with_grab = None
         GTKXpraClient.destroy_window(self, wid, window)
         group_leader = window.group_leader
         if group_leader is None or len(self._group_leader_wids)==0:
@@ -557,9 +483,7 @@ class XpraClient(GTKXpraClient):
 
 
     def get_client_window_classes(self, metadata, override_redirect):
-        #only enable GL for normal windows:
-        window_types = metadata.get("window-type", ())
-        log("get_client_window_class(%s, %s) GLClientWindowClass=%s, opengl_enabled=%s, mmap_enabled=%s, window_types=%s, encoding=%s", metadata, override_redirect, self.GLClientWindowClass, self.opengl_enabled, self.mmap_enabled, window_types, self.encoding)
+        log("get_client_window_class(%s, %s) GLClientWindowClass=%s, opengl_enabled=%s, mmap_enabled=%s, encoding=%s", metadata, override_redirect, self.GLClientWindowClass, self.opengl_enabled, self.mmap_enabled, self.encoding)
         if self.GLClientWindowClass is None or not self.opengl_enabled:
             return [self.ClientWindowClass]
         return [self.GLClientWindowClass, self.ClientWindowClass]
@@ -582,7 +506,6 @@ class XpraClient(GTKXpraClient):
             x, y = window._pos
             w, h = window._size
             client_properties = window._client_properties
-            auto_refresh_delay = window._auto_refresh_delay
             metadata = window._metadata
             override_redirect = window._override_redirect
             backing = window._backing
@@ -615,7 +538,7 @@ class XpraClient(GTKXpraClient):
                 except:
                     pass
                 #create the new window, which should honour the new state of the opengl_enabled flag:
-                window = self.make_new_window(wid, x, y, w, h, metadata, override_redirect, client_properties, auto_refresh_delay)
+                window = self.make_new_window(wid, x, y, w, h, metadata, override_redirect, client_properties)
                 if video_decoder or csc_decoder:
                     backing = window._backing
                     backing._video_decoder = video_decoder
