@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -13,7 +13,6 @@ from xpra.log import Logger
 log = Logger("window")
 
 from xpra.client.gtk_base.gtk_client_window_base import GTKClientWindowBase, HAS_X11_BINDINGS
-from xpra.client.gtk2.window_backing import HAS_ALPHA
 
 
 GTK2_WINDOW_EVENT_MASK = gdk.STRUCTURE_MASK | gdk.KEY_PRESS_MASK | gdk.KEY_RELEASE_MASK \
@@ -43,6 +42,11 @@ GTK2_NAME_TO_HINT = {
                 "COMBO"         : gdk.WINDOW_TYPE_HINT_COMBO,
                 "DND"           : gdk.WINDOW_TYPE_HINT_DND
                 }
+GTK2_BUTTON_MASK = {gtk.gdk.BUTTON1_MASK : 1,
+                    gtk.gdk.BUTTON2_MASK : 2,
+                    gtk.gdk.BUTTON3_MASK : 3,
+                    gtk.gdk.BUTTON4_MASK : 4,
+                    gtk.gdk.BUTTON5_MASK : 5}
 # Map scroll directions back to mouse buttons.  Mapping is taken from
 # gdk/x11/gdkevents-x11.c.
 GTK2_SCROLL_MAP = {
@@ -63,13 +67,12 @@ class GTK2WindowBase(GTKClientWindowBase):
     OR_TYPE_HINTS       = GTK2_OR_TYPE_HINTS
     NAME_TO_HINT        = GTK2_NAME_TO_HINT
     SCROLL_MAP          = GTK2_SCROLL_MAP
+    BUTTON_MASK         = GTK2_BUTTON_MASK
 
     WINDOW_STATE_FULLSCREEN = gdk.WINDOW_STATE_FULLSCREEN
     WINDOW_STATE_MAXIMIZED  = gdk.WINDOW_STATE_MAXIMIZED
     WINDOW_STATE_ICONIFIED  = gdk.WINDOW_STATE_ICONIFIED
 
-    #must be overriden by subclasses
-    BACKING_CLASS = None
 
     def init_window(self, metadata):
         if self._override_redirect:
@@ -81,12 +84,15 @@ class GTK2WindowBase(GTKClientWindowBase):
         # see: https://bugs.kde.org/show_bug.cgi?id=274485
         self.set_data("_kde_no_window_grab", 1)
 
+
     def setup_window(self):
         #preserve screen:
         if not self._override_redirect:
             display = gtk.gdk.display_get_default()
-            screen_num = self._client_properties.get("screen")
-            if screen_num is not None and screen_num>=0 and screen_num<display.get_n_screens():
+            screen_num = self._client_properties.get("screen", -1)
+            n = display.get_n_screens()
+            log("setup_window() screen=%s, nscreens=%s", screen_num, n)
+            if screen_num>=0 and screen_num<n:
                 screen = display.get_screen(screen_num)
                 if screen:
                     self.set_screen(screen)
@@ -94,10 +100,10 @@ class GTK2WindowBase(GTKClientWindowBase):
 
     def set_alpha(self):
         #by default, only RGB (no transparency):
+        #rgb_formats = list(BACKING_CLASS.RGB_MODES)
         self._client_properties["encodings.rgb_formats"] = ["RGB", "RGBX"]
-        if not HAS_ALPHA:
+        if not self._has_alpha:
             self._client_properties["encoding.transparency"] = False
-            self._has_alpha = False
             return
         if self._has_alpha and not self.is_realized():
             screen = self.get_screen()
