@@ -13,13 +13,15 @@ import sys
 import time
 import datetime
 
-from xpra.os_util import os_info
+from xpra.os_util import os_info, bytestostr
 from xpra.gtk_common.graph import make_graph_pixmap
 from xpra.deque import maxdeque
 from xpra.simple_stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec
 from xpra.scripts.config import python_platform
 from xpra.log import Logger
-from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, set_tooltip_text, TableBuilder, imagebutton
+from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, set_tooltip_text, \
+                        TableBuilder, imagebutton, scaled_image, get_preferred_size, get_gtk_version_info, \
+                        RELIEF_NONE, RELIEF_NORMAL, EXPAND, FILL, WIN_POS_CENTER
 from xpra.net.protocol import get_network_caps
 log = Logger("info")
 
@@ -84,10 +86,7 @@ class SessionInfo(gtk.Window):
         self.set_decorated(True)
         if window_icon_pixbuf:
             self.set_icon(window_icon_pixbuf)
-        if is_gtk3():
-            self.set_position(gtk.WindowPosition.CENTER)
-        else:
-            self.set_position(gtk.WIN_POS_CENTER)
+        self.set_position(WIN_POS_CENTER)
 
         #tables on the left in a vbox with buttons at the top:
         self.tab_box = gtk.VBox(False, 0)
@@ -99,9 +98,9 @@ class SessionInfo(gtk.Window):
         #Package Table:
         tb, _ = self.table_tab("package.png", "Software", self.populate_package)
         #title row:
-        tb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Client"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Server"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+        tb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Client"), 1, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Server"), 2, xoptions=EXPAND|FILL, xpadding=0)
         tb.inc()
 
         def make_os_str(*args):
@@ -128,7 +127,7 @@ class SessionInfo(gtk.Window):
             return version or "unknown"
         def server_info(*prop_names):
             for x in prop_names:
-                v = scaps.get(x)
+                v = scaps.capsget(x)
                 if v is not None:
                     return v
                 if self.client.server_last_info:
@@ -141,18 +140,16 @@ class SessionInfo(gtk.Window):
         tb.new_row("Revision", label(cl_rev), label(make_version_str(self.client._remote_revision)))
         tb.new_row("Local Changes", label(cl_ch), label(server_version_info("build.local_modifications", "local_modifications")))
         tb.new_row("Build date", label(cl_date), label(server_info("build_date", "build.date")))
-        def client_version_info(prop_name):
-            info = "unknown"
-            if hasattr(gtk, prop_name):
-                info = make_version_str(getattr(gtk, prop_name))
-            return info
-        if is_gtk3():
-            tb.new_row("PyGobject", label(gobject._version))
-            tb.new_row("Client GDK", label(gdk._version))
-            tb.new_row("GTK", label(gtk._version), label(server_version_info("gtk_version")))
-        else:
-            tb.new_row("PyGTK", label(client_version_info("pygtk_version")), label(server_version_info("pygtk.version", "pygtk_version")))
-            tb.new_row("GTK", label(client_version_info("gtk_version")), label(server_version_info("gtk.version", "gtk_version")))
+        gtk_version_info = get_gtk_version_info()
+        def client_vinfo(prop, fallback="unknown"):
+            k = "%s.version" % prop
+            return label(gtk_version_info.get(k, fallback))
+        tb.new_row("Gobject",   client_vinfo("gobject"),    label(server_version_info("gobject.version", "pygtk_version")))
+        tb.new_row("PyGTK",     client_vinfo("pygtk", ""),  label(server_version_info("pygtk.version", "pygtk_version")))
+        tb.new_row("GTK",       client_vinfo("gtk"),        label(server_version_info("gtk.version", "gtk_version")))
+        tb.new_row("GDK",       client_vinfo("gdk"),        label(server_version_info("gdk.version", "gdk_version")))
+        tb.new_row("Cairo",     client_vinfo("cairo"),      label(server_version_info("cairo.version", "cairo_version")))
+        tb.new_row("Pango",     client_vinfo("pango"),      label(server_version_info("pango.version", "cairo_version")))
         tb.new_row("Python", label(python_platform.python_version()), label(server_version_info("server.python.version", "python_version", "python.version")))
 
         cl_gst_v, cl_pygst_v = "", ""
@@ -217,9 +214,9 @@ class SessionInfo(gtk.Window):
         table = tb.get_table()
         vbox.pack_start(table, expand=True, fill=True, padding=20)
         #bottom table headings:
-        tb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Client"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Server"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+        tb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Client"), 1, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Server"), 2, xoptions=EXPAND|FILL, xpadding=0)
         tb.inc()
         #bottom table contents:
         self.client_encodings_label = label()
@@ -247,8 +244,9 @@ class SessionInfo(gtk.Window):
         tb.new_row("Server Endpoint", label(self.connection.target))
         if self.client.server_display:
             tb.new_row("Server Display", label(self.client.server_display))
-        if "hostname" in scaps:
-            tb.new_row("Server Hostname", label(scaps.get("hostname")))
+        hostname = scaps.strget("hostname")
+        if hostname:
+            tb.new_row("Server Hostname", label(hostname))
         if self.client.server_platform:
             tb.new_row("Server Platform", label(self.client.server_platform))
         self.server_load_label = label()
@@ -283,12 +281,12 @@ class SessionInfo(gtk.Window):
         # Details:
         tb, stats_box = self.table_tab("browse.png", "Statistics", self.populate_statistics)
         tb.widget_xalign = 1.0
-        tb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Latest"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Minimum"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Average"), 3, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("90 percentile"), 4, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-        tb.attach(title_box("Maximum"), 5, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+        tb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Latest"), 1, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Minimum"), 2, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Average"), 3, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("90 percentile"), 4, xoptions=EXPAND|FILL, xpadding=0)
+        tb.attach(title_box("Maximum"), 5, xoptions=EXPAND|FILL, xpadding=0)
         tb.inc()
 
         def maths_labels():
@@ -325,15 +323,15 @@ class SessionInfo(gtk.Window):
             wtb = TableBuilder()
             stats_box.add(wtb.get_table())
             #title row:
-            wtb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-            wtb.attach(title_box("Regular"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-            wtb.attach(title_box("Transient"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
-            wtb.attach(title_box("Trays"), 3, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            wtb.attach(title_box(""), 0, xoptions=EXPAND|FILL, xpadding=0)
+            wtb.attach(title_box("Regular"), 1, xoptions=EXPAND|FILL, xpadding=0)
+            wtb.attach(title_box("Transient"), 2, xoptions=EXPAND|FILL, xpadding=0)
+            wtb.attach(title_box("Trays"), 3, xoptions=EXPAND|FILL, xpadding=0)
             if self.client.client_supports_opengl:
-                wtb.attach(title_box("OpenGL"), 4, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+                wtb.attach(title_box("OpenGL"), 4, xoptions=EXPAND|FILL, xpadding=0)
             wtb.inc()
 
-            wtb.attach(label("Windows:"), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            wtb.attach(label("Windows:"), 0, xoptions=EXPAND|FILL, xpadding=0)
             self.windows_managed_label = label()
             wtb.attach(self.windows_managed_label, 1)
             self.transient_managed_label = label()
@@ -350,13 +348,16 @@ class SessionInfo(gtk.Window):
             self.encoder_info_box = gtk.HBox(spacing=4)
             etb.new_row("Window Encoders", self.encoder_info_box)
 
-        self.graph_box = gtk.VBox(False, 10)
-        self.add_tab("statistics.png", "Graphs", self.populate_graphs, self.graph_box)
-        bandwidth_label = "Bandwidth used"
-        if SHOW_PIXEL_STATS:
-            bandwidth_label += ",\nand number of pixels rendered"
-        self.bandwidth_graph = self.add_graph_button(bandwidth_label, self.save_graphs)
-        self.latency_graph = self.add_graph_button(None, self.save_graphs)
+        if not is_gtk3():
+            #needs porting to cairo...
+            self.graph_box = gtk.VBox(False, 10)
+            self.add_tab("statistics.png", "Graphs", self.populate_graphs, self.graph_box)
+            bandwidth_label = "Bandwidth used"
+            if SHOW_PIXEL_STATS:
+                bandwidth_label += ",\nand number of pixels rendered"
+            self.bandwidth_graph = self.add_graph_button(bandwidth_label, self.save_graphs)
+            self.connect("realize", self.populate_graphs)
+            self.latency_graph = self.add_graph_button(None, self.save_graphs)
         self.pixel_in_data = maxdeque(N_SAMPLES+4)
         self.net_in_bytecount = maxdeque(N_SAMPLES+4)
         self.net_out_bytecount = maxdeque(N_SAMPLES+4)
@@ -375,7 +376,6 @@ class SessionInfo(gtk.Window):
         self.populate_all()
         gobject.timeout_add(1000, self.populate)
         gobject.timeout_add(100, self.populate_tab)
-        self.connect("realize", self.populate_graphs)
         add_close_accel(self, self.destroy)
 
 
@@ -400,7 +400,7 @@ class SessionInfo(gtk.Window):
             self.show_tab(contents)
         button = imagebutton(title, icon, clicked_callback=show_tab)
         button.connect("clicked", show_tab)
-        button.set_relief(gtk.RELIEF_NONE)
+        button.set_relief(RELIEF_NONE)
         self.tab_button_box.add(button)
         self.tabs.append((title, button, contents, populate_cb))
 
@@ -409,11 +409,11 @@ class SessionInfo(gtk.Window):
         for _, b, t, p_cb in self.tabs:
             if t==table:
                 button = b
-                b.set_relief(gtk.RELIEF_NORMAL)
+                b.set_relief(RELIEF_NORMAL)
                 b.grab_focus()
                 self.populate_cb = p_cb
             else:
-                b.set_relief(gtk.RELIEF_NONE)
+                b.set_relief(RELIEF_NONE)
         assert button
         for x in self.tab_box.get_children():
             if x!=self.tab_button_box:
@@ -446,7 +446,7 @@ class SessionInfo(gtk.Window):
     def scaled_image(self, pixbuf, icon_size=None):
         if not icon_size:
             icon_size = self.get_icon_size()
-        return gtk.image_new_from_pixbuf(pixbuf.scale_simple(icon_size,icon_size, gdk.INTERP_BILINEAR))
+        return scaled_image(pixbuf, icon_size, icon_size)
 
     def add_graph_button(self, tooltip, click_cb):
         button = gtk.EventBox()
@@ -586,10 +586,10 @@ class SessionInfo(gtk.Window):
 
         scaps = self.client.server_capabilities
         self.bool_icon(self.server_mmap_icon, self.client.mmap_enabled)
-        self.bool_icon(self.server_clipboard_icon, scaps.get("clipboard", False))
-        self.bool_icon(self.server_notifications_icon, scaps.get("notifications", False))
-        self.bool_icon(self.server_bell_icon, scaps.get("bell", False))
-        self.bool_icon(self.server_cursors_icon, scaps.get("cursors", False))
+        self.bool_icon(self.server_clipboard_icon,      scaps.boolget("clipboard", False))
+        self.bool_icon(self.server_notifications_icon,  scaps.boolget("notifications", False))
+        self.bool_icon(self.server_bell_icon,           scaps.boolget("bell", False))
+        self.bool_icon(self.server_cursors_icon,        scaps.boolget("cursors", False))
         def pipeline_info(can_use, sound_pipeline):
             if not can_use:
                 return ""   #the icon shows this is not available, status is irrelevant so leave it empty
@@ -607,24 +607,24 @@ class SessionInfo(gtk.Window):
                 return "n/a"
             return ", ".join(codecs or [])
         def populate_speaker_info(*args):
-            can = scaps.get("sound.send", False) and self.client.speaker_allowed
+            can = scaps.boolget("sound.send", False) and self.client.speaker_allowed
             self.bool_icon(self.server_speaker_icon, can)
             self.speaker_codec_label.set_text(pipeline_info(can, self.client.sound_sink))
         populate_speaker_info()
         self.client.connect("speaker-changed", populate_speaker_info)
         def populate_microphone_info(*args):
-            can = scaps.get("sound.receive", False) and self.client.microphone_allowed
+            can = scaps.boolget("sound.receive", False) and self.client.microphone_allowed
             self.bool_icon(self.server_microphone_icon, can)
             self.microphone_codec_label.set_text(pipeline_info(can, self.client.sound_source))
         populate_microphone_info()
         self.client.connect("microphone-changed", populate_microphone_info)
 
         #sound/video codec table:
-        self.server_speaker_codecs_label.set_text(codec_info(scaps.get("sound.send", False), scaps.get("sound.encoders", [])))
+        self.server_speaker_codecs_label.set_text(codec_info(scaps.boolget("sound.send", False), scaps.strlistget("sound.encoders", [])))
         self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
-        self.server_microphone_codecs_label.set_text(codec_info(scaps.get("sound.receive", False), scaps.get("sound.decoders", [])))
+        self.server_microphone_codecs_label.set_text(codec_info(scaps.boolget("sound.receive", False), scaps.strlistget("sound.decoders", [])))
         self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
-        se = scaps.get("encodings.core", scaps.get("encodings", scaps.get("encodings.core", [])))
+        se = scaps.strlistget("encodings.core", scaps.strlistget("encodings"))
         self.server_encodings_label.set_text(", ".join(sorted(se)))
         self.client_encodings_label.set_text(", ".join(sorted(self.client.get_core_encodings())))
 
@@ -815,7 +815,7 @@ class SessionInfo(gtk.Window):
                     pixels = pixels_per_second.get(time_in_seconds, 0)
                     pixels_per_second[time_in_seconds] = pixels + size
                 if int(min_time)+1 < int(max_time):
-                    for t in xrange(int(min_time)+1, int(max_time)):
+                    for t in range(int(min_time)+1, int(max_time)):
                         rps.append(regions_per_second.get(t, 0))
                         pps.append(pixels_per_second.get(t, 0))
             setlabels(self.decoding_labels, decoding_latency)
@@ -848,6 +848,7 @@ class SessionInfo(gtk.Window):
                 #window[1].encoder=x264
                 #window[1].encoder.frames=1
                 for k,v in self.client.server_last_info.items():
+                    k = bytestostr(k)
                     pos = k.find("].encoder")
                     if k.startswith("window[") and pos>0:
                         wid_str = k[len("window["):pos]     #ie: "1"
@@ -876,8 +877,8 @@ class SessionInfo(gtk.Window):
         if self.client.server_info_request:
             self.client.send_info_request()
         box = self.tab_box
-        _, h = box.size_request()
-        _, bh = self.tab_button_box.size_request()
+        _, h = get_preferred_size(box)
+        _, bh = get_preferred_size(self.tab_button_box)
         if h<=0:
             return True
         start_x_offset = min(1.0, (time.time()-self.last_populate_time)*0.95)
