@@ -21,6 +21,7 @@ class KeyboardHelper(object):
         self.keyboard_sync = keyboard_sync
         self.key_shortcuts = self.parse_shortcuts(key_shortcuts)
         self.keyboard = Keyboard()
+        log("KeyboardHelper.__init__(%s, %s, %s) keyboard=%s", net_send, keyboard_sync, key_shortcuts, self.keyboard)
 
     def mask_to_names(self, mask):
         return self.keyboard.mask_to_names(mask)
@@ -268,8 +269,8 @@ class KeyboardHelper(object):
         for x in (self.xkbmap_print, self.xkbmap_query, \
                   self.xkbmap_mod_meanings, self.xkbmap_mod_pointermissing, \
                   self.xkbmap_keycodes, self.xkbmap_x11_keycodes):
-            h.update("/%s" % str(x))
-        self.hash = "%s".join([str(x) for x in (self.xkbmap_layout, self.xkbmap_variant, h.hexdigest()) if bool(x)])
+            h.update(("/%s" % str(x)).encode("utf8"))
+        self.hash = "/".join([str(x) for x in (self.xkbmap_layout, self.xkbmap_variant, h.hexdigest()) if bool(x)])
 
     def get_full_keymap(self):
         return []
@@ -277,7 +278,8 @@ class KeyboardHelper(object):
 
     def get_keymap_properties(self):
         props = {}
-        for x in ["xkbmap_print", "xkbmap_query", "xkbmap_mod_meanings",
-              "xkbmap_mod_managed", "xkbmap_mod_pointermissing", "xkbmap_keycodes", "xkbmap_x11_keycodes"]:
-            props[x] = nn(getattr(self, x))
+        for x in ("print", "query", "mod_meanings",
+                  "mod_managed", "mod_pointermissing", "keycodes", "x11_keycodes"):
+            p = "xkbmap_%s" % x
+            props[p] = nn(getattr(self, p))
         return  props
