@@ -8,7 +8,6 @@
 import os
 import sys
 import time
-import ctypes
 import datetime
 
 from xpra.log import Logger
@@ -21,7 +20,9 @@ traylog = Logger("client", "tray")
 keylog = Logger("client", "keyboard")
 workspacelog = Logger("client", "workspace")
 dbuslog = Logger("client", "dbus")
+grablog = Logger("client", "grab")
 
+from xpra import __version__ as XPRA_VERSION
 from xpra.gtk_common.gobject_util import no_arg_signal
 from xpra.deque import maxdeque
 from xpra.client.client_base import XpraClientBase, EXIT_TIMEOUT, EXIT_MMAP_TOKEN_FAILURE
@@ -30,13 +31,14 @@ from xpra.client.keyboard_helper import KeyboardHelper
 from xpra.platform import set_application_name
 from xpra.platform.features import MMAP_SUPPORTED, SYSTEM_TRAY_SUPPORTED, CLIPBOARD_WANT_TARGETS, CLIPBOARD_GREEDY, CLIPBOARDS
 from xpra.platform.gui import init as gui_init, ready as gui_ready, get_native_notifier_classes, get_native_tray_classes, get_native_system_tray_classes, get_native_tray_menu_helper_classes, ClientExtras
+from xpra.codecs.codec_constants import get_PIL_decodings
 from xpra.codecs.loader import codec_versions, has_codec, get_codec, PREFERED_ENCODING_ORDER, ALL_NEW_ENCODING_NAMES_TO_OLD, OLD_ENCODING_NAMES_TO_NEW
 from xpra.codecs.video_helper import getVideoHelper, ALL_VIDEO_DECODER_OPTIONS, NO_GFX_CSC_OPTIONS
 from xpra.simple_stats import std_unit
 from xpra.net.protocol import Compressed, use_lz4
 from xpra.daemon_thread import make_daemon_thread
-from xpra.os_util import thread, Queue, os_info, platform_name, get_machine_id, get_user_uuid
-from xpra.util import nn, nonl, std, AtomicInteger, log_screen_sizes
+from xpra.os_util import thread, Queue, os_info, platform_name, get_machine_id, get_user_uuid, bytestostr
+from xpra.util import nn, nonl, std, AtomicInteger, AdHocStruct, log_screen_sizes, typedict
 try:
     from xpra.clipboard.clipboard_base import ALL_CLIPBOARDS
 except:
@@ -44,9 +46,7 @@ except:
 
 FAKE_BROKEN_CONNECTION = os.environ.get("XPRA_FAKE_BROKEN_CONNECTION", "0")=="1"
 PING_TIMEOUT = int(os.environ.get("XPRA_PING_TIMEOUT", "60"))
-
-if sys.version > '3':
-    unicode = str           #@ReservedAssignment
+UNGRAB_KEY = os.environ.get("XPRA_UNGRAB_KEY", "Escape")
 
 
 """
@@ -68,6 +68,7 @@ class UIXpraClient(XpraClientBase):
 
     def __init__(self):
         XpraClientBase.__init__(self)
+        log.info("xpra client version %s" % XPRA_VERSION)
         gui_init()
         self.start_time = time.time()
         self._window_to_id = {}
@@ -89,7 +90,6 @@ class UIXpraClient(XpraClientBase):
         self.server_max_desktop_size = None
         self.server_display = None
         self.server_randr = False
-        self.server_auto_refresh_delay = 0
         self.pixel_counter = maxdeque(maxlen=1000)
         self.server_ping_latency = maxdeque(maxlen=1000)
         self.server_load = None
@@ -152,12 +152,14 @@ class UIXpraClient(XpraClientBase):
         self.mmap_token = None
         self.mmap_filename = None
         self.mmap_size = 0
+        self.mmap_group = None
 
         #features:
         self.opengl_enabled = False
         self.opengl_props = {}
         self.toggle_cursors_bell_notify = False
         self.toggle_keyboard_sync = False
+        self.force_ungrab = False
         self.window_unmap = False
         self.window_refresh_config = False
         self.server_generic_encodings = False
@@ -193,6 +195,7 @@ class UIXpraClient(XpraClientBase):
         #helpers and associated flags:
         self.client_extras = None
         self.keyboard_helper = None
+        self.kh_warning = False
         self.clipboard_helper = None
         self.menu_helper = None
         self.tray = None
@@ -200,6 +203,7 @@ class UIXpraClient(XpraClientBase):
 
         #state:
         self._focused = None
+        self._window_with_grab = None
         self._last_screen_settings = None
         self._suspended_at = 0
 
@@ -215,6 +219,8 @@ class UIXpraClient(XpraClientBase):
         self.auto_refresh_delay = opts.auto_refresh_delay
         self.dpi = int(opts.dpi)
         self.xsettings_enabled = opts.xsettings
+        self.supports_mmap = MMAP_SUPPORTED and opts.mmap
+        self.mmap_group = opts.mmap_group
 
         try:
             from xpra.sound.gstreamer_util import has_gst, get_sound_codecs
@@ -253,10 +259,6 @@ class UIXpraClient(XpraClientBase):
         """ initialize user interface """
         self.init_opengl(opts.opengl)
 
-        self.supports_mmap = MMAP_SUPPORTED
-        if self.supports_mmap and opts.mmap:
-            self.init_mmap(opts.mmap_group, self._protocol._conn.filename)
-
         if not self.readonly:
             self.keyboard_helper = self.make_keyboard_helper(opts.keyboard_sync, opts.key_shortcut)
 
@@ -277,7 +279,7 @@ class UIXpraClient(XpraClientBase):
 
         if self.client_supports_notifications:
             self.notifier = self.make_notifier()
-            log("using notifier=%s", self.notifier)
+            traylog("using notifier=%s", self.notifier)
             self.client_supports_notifications = self.notifier is not None
 
         #audio tagging:
@@ -289,7 +291,7 @@ class UIXpraClient(XpraClientBase):
                 log("failed to set pulseaudio audio tagging: %s", e)
 
         if ClientExtras is not None:
-            self.client_extras = ClientExtras(self)
+            self.client_extras = ClientExtras(self, opts)
 
         if opts.border:
             self.parse_border(opts.border)
@@ -298,6 +300,11 @@ class UIXpraClient(XpraClientBase):
         self._draw_queue = Queue()
         self._draw_thread = make_daemon_thread(self._draw_thread_loop, "draw")
 
+    def setup_connection(self, conn):
+        XpraClientBase.setup_connection(self, conn)
+        if self.supports_mmap:
+            self.init_mmap(self.mmap_group, conn.filename)
+
 
     def parse_border(self, border_str):
         #not implemented here (see gtk2 client)
@@ -356,6 +363,7 @@ class UIXpraClient(XpraClientBase):
         elapsed = 0
         if self._suspended_at>0:
             elapsed = time.time()-self._suspended_at
+            self._suspended_at = 0
         delta = datetime.timedelta(seconds=int(elapsed))
         log.info("system resumed, was suspended for %s", delta)
         #this will reset the refresh rate too:
@@ -432,15 +440,10 @@ class UIXpraClient(XpraClientBase):
         """
         #we always support rgb24:
         core_encodings = ["rgb24"]
-        for modules, encodings in {
-              ("dec_webp",)                     : ["webp"],
-              ("PIL",)                          : ["png", "png/L", "png/P", "jpeg"],
-               }.items():
-            missing = [x for x in modules if not has_codec(x)]
-            if len(missing)>0:
-                log("do_get_core_encodings() not adding %s because of missing modules: %s", encodings, missing)
-                continue
-            core_encodings += encodings
+        #PIL:
+        core_encodings += get_PIL_decodings(get_codec("PIL"))
+        if (has_codec("dec_webm") or has_codec("dec_webp")) and "webp" not in core_encodings:
+            core_encodings.append("webp")
         #we enable all the video decoders we know about,
         #what will actually get used by the server will still depend on the csc modes supported
         video_decodings = getVideoHelper().get_decodings()
@@ -465,7 +468,9 @@ class UIXpraClient(XpraClientBase):
 
 
     def make_notifier(self):
-        return self.make_instance(self.get_notifier_classes())
+        nc = self.get_notifier_classes()
+        traylog("make_notifier() notifier classes: %s", nc)
+        return self.make_instance(nc)
 
     def get_notifier_classes(self):
         #subclasses will generally add their toolkit specific variants
@@ -625,7 +630,12 @@ class UIXpraClient(XpraClientBase):
         #trigger multiple calls to screen_size_changed, delayed by some amount
         #(sometimes up to 1s..)
         self.screen_size_change_pending = True
-        self.timeout_add(1000, update_screen_size)
+        delay = 1000
+        #if we are suspending, wait longer:
+        #(better chance that the suspend-resume cycle will have completed)
+        if self._suspended_at>0 and self._suspended_at-time.time()<5*1000:
+            delay = 5*1000
+        self.timeout_add(delay, update_screen_size)
 
     def get_screen_sizes(self):
         raise Exception("override me!")
@@ -708,14 +718,18 @@ class UIXpraClient(XpraClientBase):
         self.send("focus", wid, self.get_current_modifiers())
 
     def update_focus(self, wid, gotit):
-        focuslog("update_focus(%s, %s) _focused=%s", wid, gotit, self._focused)
+        focuslog("update_focus(%s, %s) focused=%s, grabbed=%s", wid, gotit, self._focused, self._window_with_grab)
         if gotit and self._focused is not wid:
             if self.keyboard_helper:
                 self.keyboard_helper.clear_repeat()
             self.send_focus(wid)
             self._focused = wid
         if not gotit:
-            if self._focused!=wid:
+            if self._window_with_grab:
+                self.window_ungrab()
+                self.do_force_ungrab(self._window_with_grab)
+                self._window_with_grab = None
+            if wid and self._focused and self._focused!=wid:
                 #if this window lost focus, it must have had it!
                 #(catch up - makes things like OR windows work:
                 # their parent receives the focus-out event)
@@ -723,8 +737,79 @@ class UIXpraClient(XpraClientBase):
                 self.send_focus(wid)
             if self.keyboard_helper:
                 self.keyboard_helper.clear_repeat()
-            self.send_focus(0)
-            self._focused = None
+            if self._focused:
+                #send the lost-focus via a timer and re-check it
+                #(this allows a new window to gain focus without having to do a reset_focus)
+                def send_lost_focus():
+                    #check that a new window has not gained focus since:
+                    if self._focused is None:
+                        self.send_focus(0)
+                self.timeout_add(20, send_lost_focus)
+                self._focused = None
+
+    def do_force_ungrab(self, wid):
+        grablog("do_force_ungrab(%s) server supports force ungrab: %s", wid, self.force_ungrab)
+        if self.force_ungrab:
+            #ungrab via dedicated server packet:
+            self.send_force_ungrab(wid)
+            return
+        #fallback for older servers: try to find a key to press:
+        kh = self.keyboard_helper
+        if not kh:
+            if not self.kh_warning:
+                self.kh_warning = True
+                grablog.warn("no keyboard support, cannot simulate keypress to lose grab!")
+            return
+        #xkbmap_keycodes is a list of: (keyval, name, keycode, group, level)
+        ungrab_keys = [x for x in kh.xkbmap_keycodes if x[1]==UNGRAB_KEY]
+        if len(ungrab_keys)==0:
+            if not self.kh_warning:
+                self.kh_warning = True
+                grablog.warn("ungrab key %s not found, cannot simulate keypress to lose grab!", UNGRAB_KEY)
+            return
+        #ungrab_keys.append((65307, "Escape", 27, 0, 0))     #ugly hardcoded default value
+        ungrab_key = ungrab_keys[0]
+        grablog("lost focus whilst window %s has grab, simulating keypress: %s", wid, ungrab_key)
+        key_event = AdHocStruct()
+        key_event.keyname = ungrab_key[1]
+        key_event.pressed = True
+        key_event.modifiers = []
+        key_event.keyval = ungrab_key[0]
+        keycode = ungrab_key[2]
+        try:
+            key_event.string = chr(keycode)
+        except:
+            key_event.string = str(keycode)
+        key_event.keycode = keycode
+        key_event.group = 0
+        #press:
+        kh.send_key_action(wid, key_event)
+        #unpress:
+        key_event.pressed = False
+        kh.send_key_action(wid, key_event)
+
+    def _process_pointer_grab(self, packet):
+        wid = packet[1]
+        window = self._id_to_window.get(wid)
+        grablog("grabbing %s: %s", wid, window)
+        if window:
+            self.window_grab(window)
+            self._window_with_grab = wid
+
+    def window_grab(self, window):
+        #subclasses should implement this method
+        pass
+
+    def _process_pointer_ungrab(self, packet):
+        wid = packet[1]
+        window = self._id_to_window.get(wid)
+        grablog("ungrabbing %s: %s", wid, window)
+        self.window_ungrab()
+        self._window_with_grab = None
+
+    def window_ungrab(self):
+        #subclasses should implement this method
+        pass
 
 
     def make_hello(self):
@@ -859,6 +944,8 @@ class UIXpraClient(XpraClientBase):
         if self.min_speed>=0:
             capabilities["encoding.min-speed"] = self.min_speed
 
+        #note: this is mostly for old servers, with newer ones we send the properties
+        #again (and more accurately) once the window is instantiated.
         #figure out the CSC modes supported:
         #these are the RGB modes we want (the ones we can paint with):
         rgb_formats = ["RGB", "RGBX"]
@@ -866,7 +953,6 @@ class UIXpraClient(XpraClientBase):
             #only win32 and osx cannot handle transparency
             rgb_formats.append("RGBA")
         capabilities["encodings.rgb_formats"] = rgb_formats
-
         #figure out which CSC modes (usually YUV) can give us those RGB modes:
         full_csc_modes = getVideoHelper().get_server_full_csc_modes_for_rgb(*rgb_formats)
         log("supported full csc_modes=%s", full_csc_modes)
@@ -1017,7 +1103,7 @@ class UIXpraClient(XpraClientBase):
         assert self.server_info_request
         if not self.info_request_pending:
             self.info_request_pending = True
-            self.send("info-request", [self.uuid], self._id_to_window.keys())
+            self.send("info-request", [self.uuid], list(self._id_to_window.keys()))
 
 
     def send_quality(self):
@@ -1053,6 +1139,7 @@ class UIXpraClient(XpraClientBase):
             self.session_name = c.strget("session_name", "")
         set_application_name(self.session_name or "Xpra")
         self.window_unmap = c.boolget("window_unmap")
+        self.force_ungrab = c.boolget("force_ungrab")
         self.window_refresh_config = c.boolget("window_refresh_config")
         self.suspend_resume = c.boolget("suspend-resume")
         self.server_supports_notifications = c.boolget("notifications")
@@ -1062,7 +1149,7 @@ class UIXpraClient(XpraClientBase):
         self.server_supports_bell = c.boolget("bell")          #added in 0.5, default to True!
         self.bell_enabled = self.server_supports_bell and self.client_supports_bell
         self.server_supports_clipboard = c.boolget("clipboard")
-        self.server_clipboards = c.listget("clipboards", ALL_CLIPBOARDS)
+        self.server_clipboards = c.strlistget("clipboards", ALL_CLIPBOARDS)
         self.clipboard_enabled = self.client_supports_clipboard and self.server_supports_clipboard
         self.server_dbus_proxy = c.boolget("dbus_proxy")
         self.mmap_enabled = self.supports_mmap and self.mmap_enabled and c.boolget("mmap_enabled")
@@ -1077,7 +1164,9 @@ class UIXpraClient(XpraClientBase):
                     self.mmap_enabled = False
                     self.quit(EXIT_MMAP_TOKEN_FAILURE)
                     return
-        self.server_auto_refresh_delay = c.intget("auto_refresh_delay", 0)/1000.0
+        server_auto_refresh_delay = c.intget("auto_refresh_delay", 0)/1000.0
+        if server_auto_refresh_delay==0 and self.auto_refresh_delay>0:
+            log.warn("server does not support auto-refresh!")
         def getenclist(k, default_value=[]):
             #deals with old servers and substitute old encoding names for the new ones
             v = c.strlistget(k, default_value)
@@ -1130,6 +1219,7 @@ class UIXpraClient(XpraClientBase):
             proxy_platform = c.strget("proxy.platform")
             proxy_release = c.strget("proxy.platform.release")
             proxy_version = c.strget("proxy.version")
+            proxy_version = c.strget("proxy.build.version", proxy_version)
             msg = "via: %s proxy version %s" % (platform_name(proxy_platform, proxy_release), std(proxy_version))
             if proxy_hostname:
                 msg += " on '%s'" % std(proxy_hostname)
@@ -1145,7 +1235,7 @@ class UIXpraClient(XpraClientBase):
             self.clipboard_enabled = self.clipboard_helper is not None
         self.set_max_packet_size()
         self.send_deflate_level()
-        server_desktop_size = c.listget("desktop_size")
+        server_desktop_size = c.intlistget("desktop_size")
         log("server desktop size=%s", server_desktop_size)
         if not c.boolget("shadow"):
             assert server_desktop_size
@@ -1156,7 +1246,7 @@ class UIXpraClient(XpraClientBase):
                          "(server: %sx%s vs. client: %sx%s)\n"
                          "You may see strange behavior.\n"
                          "Please see "
-                         "https://www.xpra.org/trac/ticket/10"
+                         "http://xpra.org/trac/wiki/Xdummy#Configuration"
                          % (avail_w, avail_h, root_w, root_h))
         if self.keyboard_helper:
             modifier_keycodes = c.dictget("modifier_keycodes")
@@ -1455,8 +1545,10 @@ class UIXpraClient(XpraClientBase):
 
     def _process_sound_data(self, packet):
         codec, data, metadata = packet[1:4]
+        codec = bytestostr(codec)
+        metadata = typedict(metadata)
         if not self.speaker_enabled:
-            if metadata.get("start-of-stream"):
+            if metadata.boolget("start-of-stream"):
                 #server is asking us to start playing sound
                 if not self.speaker_allowed:
                     #no can do!
@@ -1465,34 +1557,35 @@ class UIXpraClient(XpraClientBase):
                 self.speaker_enabled = True
                 self.emit("speaker-changed")
                 self.on_sink_ready = None
-                codec = metadata.get("codec")
+                codec = metadata.strget("codec")
                 soundlog("starting speaker on server request using codec %s", codec)
                 self.start_sound_sink(codec)
             else:
                 soundlog("speaker is now disabled - dropping packet")
                 return
-        elif metadata.get("end-of-stream"):
+        elif metadata.boolget("end-of-stream"):
             if self.sound_sink:
                 soundlog("server sent end-of-stream, closing sound pipeline")
                 self.stop_receiving_sound(False)
             return
-        seq = metadata.get("sequence", -1)
+        seq = metadata.intget("sequence", -1)
         if self.min_sound_sequence>0 and seq<self.min_sound_sequence:
             soundlog("ignoring sound data with old sequence number %s", seq)
             return
-        if self.sound_sink is not None and codec!=self.sound_sink.codec:
-            log.error("sound codec change not supported! (from %s to %s)", self.sound_sink.codec, codec)
-            self.sound_sink.stop()
+        ss = self.sound_sink
+        if ss is not None and codec!=self.sound_sink.codec:
+            log.error("sound codec change not supported! (from %s to %s)", ss.codec, codec)
+            ss.stop()
             return
-        if self.sound_sink is None:
+        if ss is None:
             soundlog("no sound sink to process sound data, dropping it")
             return
-        elif self.sound_sink.get_state()=="stopped":
+        elif ss.get_state()=="stopped":
             soundlog("sound data received, sound sink is stopped - starting it")
-            self.sound_sink.start()
+            ss.start()
         #(some packets (ie: sos, eos) only contain metadata)
         if len(data)>0:
-            self.sound_sink.add_data(data, metadata)
+            ss.add_data(data, metadata)
 
 
     def send_notify_enabled(self):
@@ -1513,6 +1606,9 @@ class UIXpraClient(XpraClientBase):
         assert self.toggle_cursors_bell_notify, "cannot toggle cursors: server lacks the feature"
         self.send("set-cursors", self.cursors_enabled)
 
+    def send_force_ungrab(self, wid):
+        assert self.force_ungrab
+        self.send("force-ungrab", wid)
 
     def set_deflate_level(self, level):
         self.compression_level = level
@@ -1567,20 +1663,32 @@ class UIXpraClient(XpraClientBase):
         client_properties = {}
         if len(packet)>=8:
             client_properties = packet[7]
-        if self.server_auto_refresh_delay>0:
-            auto_refresh_delay = 0                          #server takes care of it
-        else:
-            auto_refresh_delay = self.auto_refresh_delay    #we do it
-        self.make_new_window(wid, x, y, w, h, metadata, override_redirect, client_properties, auto_refresh_delay)
+        self.make_new_window(wid, x, y, w, h, metadata, override_redirect, client_properties)
 
-    def make_new_window(self, wid, x, y, w, h, metadata, override_redirect, client_properties, auto_refresh_delay):
+    def make_new_window(self, wid, x, y, w, h, metadata, override_redirect, client_properties):
+        metadata = typedict(metadata)
         client_window_classes = self.get_client_window_classes(metadata, override_redirect)
         group_leader_window = self.get_group_leader(metadata, override_redirect)
+        #workaround for "popup" OR windows without a transient-for (like: google chrome popups):
+        #prevents them from being pushed under other windows on OSX
+        #find a "transient-for" value using the pid to find a suitable window
+        #if possible, choosing the currently focused window (if there is one..)
+        pid = metadata.intget("pid", 0)
+        if override_redirect and pid>0 and metadata.intget("transient-for", 0)>0 is None and metadata.get("role")=="popup":
+            tfor = None
+            for twid, twin in self._id_to_window.items():
+                if not twin._override_redirect and twin._metadata.intget("pid", -1)==pid:
+                    tfor = twin
+                    if twid==self._focused:
+                        break
+            if tfor:
+                windowlog("forcing transient for=%s for new window %s", twid, wid)
+                metadata["transient-for"] = twid
         window = None
         windowlog("make_new_window(..) client_window_classes=%s, group_leader_window=%s", client_window_classes, group_leader_window)
         for cwc in client_window_classes:
             try:
-                window = cwc(self, group_leader_window, wid, x, y, w, h, metadata, override_redirect, client_properties, auto_refresh_delay, self.border)
+                window = cwc(self, group_leader_window, wid, x, y, w, h, metadata, override_redirect, client_properties, self.border)
                 break
             except:
                 windowlog.warn("failed to instantiate %s", cwc, exc_info=True)
@@ -1667,9 +1775,10 @@ class UIXpraClient(XpraClientBase):
             def draw_cleanup():
                 if coding=="mmap":
                     assert self.mmap_enabled
+                    from xpra.net.mmap_pipe import int_from_buffer
                     def free_mmap_area():
                         #we need to ack the data to free the space!
-                        data_start = ctypes.c_uint.from_buffer(self.mmap, 0)
+                        data_start = int_from_buffer(self.mmap, 0)
                         offset, length = data[-1]
                         data_start.value = offset+length
                     #clear the mmap area via idle_add so any pending draw requests
@@ -1680,6 +1789,7 @@ class UIXpraClient(XpraClientBase):
         options = {}
         if len(packet)>10:
             options = packet[10]
+        options = typedict(options)
         paintlog("process_draw %s bytes for window %s using %s encoding with options=%s", len(data), wid, coding, options)
         start = time.time()
         def record_decode_time(success):
@@ -1749,6 +1859,7 @@ class UIXpraClient(XpraClientBase):
         wid, metadata = packet[1:3]
         window = self._id_to_window.get(wid)
         if window:
+            metadata = typedict(metadata)
             window.update_metadata(metadata)
 
     def _process_window_icon(self, packet):
@@ -1778,6 +1889,10 @@ class UIXpraClient(XpraClientBase):
     def destroy_window(self, wid, window):
         windowlog("destroy_window(%s, %s)", wid, window)
         window.destroy()
+        if self._window_with_grab==wid:
+            log("destroying window %s which has grab, ungrabbing!", wid)
+            self.window_ungrab()
+            self._window_with_grab = None
 
     def _process_desktop_size(self, packet):
         root_w, root_h, max_w, max_h = packet[1:5]
@@ -1842,7 +1957,8 @@ class UIXpraClient(XpraClientBase):
     def process_packet(self, proto, packet):
         packet_type = packet[0]
         self.check_server_echo(0)
-        if type(packet_type) in (unicode, str) and packet_type.startswith("clipboard-"):
+        packet_type_str = bytestostr(packet_type)
+        if packet_type_str.startswith("clipboard-"):
             if self.clipboard_enabled and self.clipboard_helper:
                 self.process_clipboard_packet(packet)
         else:
