@@ -12,15 +12,25 @@ log = Logger("paint")
 from threading import Lock
 from xpra.net.mmap_pipe import mmap_read
 from xpra.net.protocol import has_lz4, LZ4_uncompress
-from xpra.os_util import BytesIOClass, bytestostr
-from xpra.codecs.codec_constants import get_colorspace_from_avutil_enum
+from xpra.util import typedict
+from xpra.codecs.codec_constants import get_colorspace_from_avutil_enum, get_PIL_decodings
 from xpra.codecs.loader import get_codec
 from xpra.codecs.video_helper import getVideoHelper
+from xpra.os_util import BytesIOClass, bytestostr, builtins
+_memoryview = builtins.__dict__.get("memoryview")
 try:
     from xpra.codecs.xor import xor_str
 except:
     xor_str = None
+#for alpha unpremultiply utility function
+#(subclasses should ensure argb is present before calling it!)
+try:
+    from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place, byte_buffer_to_buffer   #@UnresolvedImport
+except:
+    log.warn("argb module is missing, cannot support alpha channels")
+    unpremultiply_argb, unpremultiply_argb_in_place, byte_buffer_to_buffer  = None, None, None
 
+PIL = get_codec("PIL")
 
 #ie:
 #CSC_OPTIONS = { "YUV420P" : {"RGBX" : [opencl.spec, swscale.spec], "BGRX" : ...} }
@@ -68,17 +78,18 @@ Generic superclass for all Backing code,
 see CairoBacking and GTKWindowBacking for actual implementations
 """
 class WindowBackingBase(object):
-    def __init__(self, wid, idle_add):
+    def __init__(self, wid, window_alpha, idle_add):
         load_csc_options()
         load_video_decoders()
         self.wid = wid
         self.idle_add = idle_add
-        self._has_alpha = False
+        self._alpha_enabled = window_alpha
         self._backing = None
         self._last_pixmap_data = None
         self._video_decoder = None
         self._csc_decoder = None
         self._decoder_lock = Lock()
+        self._PIL_encodings = get_PIL_decodings(PIL)
         self.draw_needs_refresh = True
         self.mmap = None
         self.mmap_enabled = False
@@ -117,6 +128,51 @@ class WindowBackingBase(object):
             self._csc_decoder = None
 
 
+    def get_encoding_properties(self):
+        rgb_modes = self.RGB_MODES
+        return {
+                 "encodings.rgb_formats"    : rgb_modes,
+                 "encoding.transparency"    : self._alpha_enabled,
+                 "encoding.full_csc_modes"  : self._get_full_csc_modes(rgb_modes),
+                 "encoding.csc_modes"       : self._get_csc_modes(rgb_modes)
+                 }
+
+    def _get_full_csc_modes(self, rgb_modes):
+        #calculate the server CSC modes the server is allowed to use
+        #based on the client CSC modes we can convert to in the backing class we use
+        #and trim the transparency if we cannot handle it
+        target_rgb_modes = list(rgb_modes)
+        if not self._alpha_enabled:
+            target_rgb_modes = [x for x in target_rgb_modes if x.find("A")<0]
+        full_csc_modes = getVideoHelper().get_server_full_csc_modes_for_rgb(*target_rgb_modes)
+        log("full csc modes (%s)=%s", target_rgb_modes, full_csc_modes)
+        return full_csc_modes
+
+    def _get_csc_modes(self, rgb_modes):
+        #as above, but for older servers: less detailed than "full" csc modes info
+        csc_modes = []
+        for modes in self._get_full_csc_modes(rgb_modes).values():
+            csc_modes += modes
+        csc_modes = list(set(csc_modes))
+        return csc_modes
+
+
+    def img_data_tobytes(self, img_data):
+        if _memoryview and isinstance(img_data, _memoryview):
+            return img_data.tobytes()
+        return img_data
+
+    def unpremultiply(self, img_data):
+        if type(img_data)==str:
+            #cannot do in-place:
+            assert unpremultiply_argb is not None, "missing argb.unpremultiply_argb"
+            return byte_buffer_to_buffer(unpremultiply_argb(img_data))
+        #assume this is a writeable buffer (ie: ctypes from mmap):
+        assert unpremultiply_argb_in_place is not None, "missing argb.unpremultiply_argb_in_place"
+        unpremultiply_argb_in_place(img_data)
+        return img_data
+
+
     def process_delta(self, raw_data, width, height, rowstride, options):
         """
             Can be called from any thread, decompresses and xors the rgb raw_data,
@@ -124,13 +180,16 @@ class WindowBackingBase(object):
         """
         img_data = raw_data
         if options:
-            if options.get("zlib", 0)>0:
+            if options.intget("zlib", 0)>0:
                 img_data = zlib.decompress(raw_data)
-            elif options.get("lz4", False):
+            elif options.boolget("lz4", False):
                 assert has_lz4
                 img_data = LZ4_uncompress(raw_data)
-        assert len(img_data) == rowstride * height, "expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" % (rowstride * height, width, height, rowstride, len(img_data), len(raw_data))
-        delta = options.get("delta", -1)
+        if len(img_data)!=rowstride * height:
+            log.error("invalid img data %s: %s", type(img_data), str(img_data)[:256])
+            raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
+                                (rowstride * height, width, height, rowstride, len(img_data), len(raw_data)))
+        delta = options.intget("delta", -1)
         rgb_data = img_data
         if delta>=0:
             if not self._last_pixmap_data:
@@ -141,7 +200,7 @@ class WindowBackingBase(object):
             assert width==lwidth and height==lheight and delta==store
             rgb_data = xor_str(img_data, ldata)
         #store new pixels for next delta:
-        store = options.get("store", -1)
+        store = options.intget("store", -1)
         if store>=0:
             self._last_pixmap_data =  width, height, store, rgb_data
         return rgb_data
@@ -150,46 +209,91 @@ class WindowBackingBase(object):
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
         """ can be called from any thread """
         #log("paint_image(%s, %s bytes, %s, %s, %s, %s, %s, %s)", coding, len(img_data), x, y, width, height, options, callbacks)
-        PIL = get_codec("PIL")
         assert PIL, "PIL not found"
         buf = BytesIOClass(img_data)
         img = PIL.Image.open(buf)
         assert img.mode in ("L", "P", "RGB", "RGBA"), "invalid image mode: %s" % img.mode
-        if img.mode in ("P", "L"):
-            transparency = options.get("transparency", -1)
+        transparency = options.get("transparency", -1)
+        if img.mode=="P":
             if transparency>=0:
+                #this deals with alpha without any extra work
                 img = img.convert("RGBA")
             else:
                 img = img.convert("RGB")
+        elif img.mode=="L":
+            if transparency>=0:
+                #why do we have to deal with alpha ourselves??
+                def mask_value(a):
+                    if a!=transparency:
+                        return 255
+                    return 0
+                mask = PIL.Image.eval(img, mask_value)
+                mask = mask.convert("L")
+                def nomask_value(a):
+                    if a!=transparency:
+                        return a
+                    return 0
+                img = PIL.Image.eval(img, nomask_value)
+                img = img.convert("RGBA")
+                img.putalpha(mask)
+            else:
+                img = img.convert("RGB")
+
         raw_data = img.tostring("raw", img.mode)
+        paint_options = typedict(options)
         if img.mode=="RGB":
             #PIL flattens the data to a continuous straightforward RGB format:
             rowstride = width*3
+            paint_options["rgb_format"] = "RGB"
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
-            self.idle_add(self.do_paint_rgb24, img_data, x, y, width, height, rowstride, options, callbacks)
+            self.idle_add(self.do_paint_rgb24, img_data, x, y, width, height, rowstride, paint_options, callbacks)
         elif img.mode=="RGBA":
             rowstride = width*4
+            paint_options["rgb_format"] = "RGBA"
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
-            self.idle_add(self.do_paint_rgb32, img_data, x, y, width, height, rowstride, options, callbacks)
+            self.idle_add(self.do_paint_rgb32, img_data, x, y, width, height, rowstride, paint_options, callbacks)
         return False
 
     def paint_webp(self, img_data, x, y, width, height, options, callbacks):
-        """ can be called from any thread """
         dec_webp = get_codec("dec_webp")
-        assert dec_webp is not None, "webp decoder not found"
+        if dec_webp:
+            return self.paint_webp_using_cwebp(img_data, x, y, width, height, options, callbacks)
+        return self.paint_webp_using_webm(img_data, x, y, width, height, options, callbacks)
+
+    def paint_webp_using_cwebp(self, img_data, x, y, width, height, options, callbacks):
+        dec_webp = get_codec("dec_webp")
+        has_alpha = options.get("has_alpha", False)
+        buffer_wrapper, width, height, stride, has_alpha, rgb_format = dec_webp.decompress(img_data, has_alpha)
+        options["rgb_format"] = rgb_format
+        def free_buffer(*args):
+            buffer_wrapper.free()
+        callbacks.append(free_buffer)
+        data = buffer_wrapper.get_pixels()
+        if has_alpha:
+            return self.paint_rgb32(data, x, y, width, height, stride, options, callbacks)
+        else:
+            return self.paint_rgb24(data, x, y, width, height, stride, options, callbacks)
+
+    def paint_webp_using_webm(self, img_data, x, y, width, height, options, callbacks):
+        """ can be called from any thread """
+        dec_webm = get_codec("dec_webm")
+        assert dec_webm is not None, "webp decoder not found"
+        paint_options = typedict(options)
         if options.get("has_alpha", False):
-            decode = dec_webp.DecodeRGBA
+            decode = dec_webm.DecodeRGBA
             rowstride = width*4
             paint_rgb = self.do_paint_rgb32
+            paint_options["rgb_format"] = "RGBA"
         else:
-            decode = dec_webp.DecodeRGB
+            decode = dec_webm.DecodeRGB
             rowstride = width*3
             paint_rgb = self.do_paint_rgb24
-        log("paint_webp(%s) using decode=%s, paint=%s",
-             ("%s bytes" % len(img_data), x, y, width, height, options, callbacks), decode, paint_rgb)
+            paint_options["rgb_format"] = "RGB"
+        log("paint_webp(%s) using decode=%s, paint=%s, paint_options=%s",
+             ("%s bytes" % len(img_data), x, y, width, height, options, callbacks), decode, paint_rgb, paint_options)
         rgb_data = decode(img_data)
         pixels = str(rgb_data.bitmap)
-        self.idle_add(paint_rgb, pixels, x, y, width, height, rowstride, options, callbacks)
+        self.idle_add(paint_rgb, pixels, x, y, width, height, rowstride, paint_options, callbacks)
         return  False
 
     def paint_rgb24(self, raw_data, x, y, width, height, rowstride, options, callbacks):
@@ -206,7 +310,7 @@ class WindowBackingBase(object):
             the actual paint code is in _do_paint_rgb24
         """
         try:
-            success = self._do_paint_rgb24(img_data, x, y, width, height, rowstride, options, callbacks)
+            success = (self._backing is not None) and self._do_paint_rgb24(img_data, x, y, width, height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
         except KeyboardInterrupt:
             raise
@@ -214,7 +318,7 @@ class WindowBackingBase(object):
             log.error("do_paint_rgb24 error", exc_info=True)
             fire_paint_callbacks(callbacks, False)
 
-    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options, callbacks):
+    def _do_paint_rgb24(self, img_data, x, y, width, height, rowstride, options):
         raise Exception("override me!")
 
 
@@ -232,7 +336,7 @@ class WindowBackingBase(object):
             the actual paint code is in _do_paint_rgb32
         """
         try:
-            success = self._do_paint_rgb32(img_data, x, y, width, height, rowstride, options, callbacks)
+            success = (self._backing is not None) and self._do_paint_rgb32(img_data, x, y, width, height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
         except KeyboardInterrupt:
             raise
@@ -240,7 +344,7 @@ class WindowBackingBase(object):
             log.error("do_paint_rgb32 error", exc_info=True)
             fire_paint_callbacks(callbacks, False)
 
-    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options, callbacks):
+    def _do_paint_rgb32(self, img_data, x, y, width, height, rowstride, options):
         raise Exception("override me!")
 
 
@@ -280,12 +384,12 @@ class WindowBackingBase(object):
                 log("window %s is already gone!", self.wid)
                 fire_paint_callbacks(callbacks, False)
                 return  False
-            enc_width, enc_height = options.get("scaled_size", (width, height))
-            input_colorspace = options.get("csc")
+            enc_width, enc_height = options.intpair("scaled_size", (width, height))
+            input_colorspace = options.strget("csc")
             if not input_colorspace:
                 # Backwards compatibility with pre 0.10.x clients
                 # We used to specify the colorspace as an avutil PixelFormat constant
-                old_csc_fmt = options.get("csc_pixel_format")
+                old_csc_fmt = options.intget("csc_pixel_format")
                 input_colorspace = get_colorspace_from_avutil_enum(old_csc_fmt)
                 if input_colorspace is None:
                     #completely broken and out of date servers (ie: v0.3.x):
@@ -351,7 +455,7 @@ class WindowBackingBase(object):
             #use higher quality csc to compensate for lower quality source
             #(which generally means that we downscaled via YUV422P or lower)
             #or when upscaling the video:
-            q = options.get("quality", 50)
+            q = options.intget("quality", 50)
             csc_speed = int(min(100, 100-q, 100.0 * (enc_width*enc_height) / (width*height)))
             self._csc_decoder = self.make_csc(enc_width, enc_height, pixel_format,
                                            width, height, target_rgb_formats, csc_speed)
@@ -361,15 +465,18 @@ class WindowBackingBase(object):
         log("do_video_paint rgb using %s.convert_image(%s)=%s", self._csc_decoder, img, rgb)
         img.free()
         assert rgb.get_planes()==0, "invalid number of planes for %s: %s" % (rgb_format, rgb.get_planes())
+        #make a new options dict and set the rgb format:
+        paint_options = typedict(options)
+        paint_options["rgb_format"] = rgb_format
         #this will also take care of firing callbacks (from the UI thread):
         def paint():
             data = rgb.get_pixels()
             rowstride = rgb.get_rowstride()
-            if rgb_format=="RGB":
-                self.do_paint_rgb24(data, x, y, width, height, rowstride, options, callbacks)
+            if len(rgb_format)==3:
+                self.do_paint_rgb24(data, x, y, width, height, rowstride, paint_options, callbacks)
             else:
-                #assert rgb_format=="RGBX"
-                self.do_paint_rgb32(data, x, y, width, height, rowstride, options, callbacks)
+                assert len(rgb_format)==4
+                self.do_paint_rgb32(data, x, y, width, height, rowstride, paint_options, callbacks)
             rgb.free()
         self.idle_add(paint)
 
@@ -381,11 +488,11 @@ class WindowBackingBase(object):
         """ see _mmap_send() in server.py for details """
         assert self.mmap_enabled
         data = mmap_read(self.mmap, img_data)
-        rgb_format = options.get("rgb_format", "rgb24")
+        rgb_format = options.strget("rgb_format", "RGB")
         #Note: BGR(A) is only handled by gl_window_backing
         if rgb_format in ("RGB", "BGR"):
             self.do_paint_rgb24(data, x, y, width, height, rowstride, options, callbacks)
-        elif rgb_format in ("RGBA", "BGRA"):
+        elif rgb_format in ("RGBA", "BGRA", "BGRX", "RGBX"):
             self.do_paint_rgb32(data, x, y, width, height, rowstride, options, callbacks)
         else:
             raise Exception("invalid rgb format: %s" % rgb_format)
@@ -395,6 +502,7 @@ class WindowBackingBase(object):
     def draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
         """ dispatches the paint to one of the paint_XXXX methods """
         log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)", x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
+        img_data = self.img_data_tobytes(img_data)
         coding = bytestostr(coding)
         if coding == "mmap":
             self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
@@ -410,7 +518,10 @@ class WindowBackingBase(object):
             self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
         elif coding == "webp":
             self.paint_webp(img_data, x, y, width, height, options, callbacks)
-        elif coding[:3]=="png" or coding=="jpeg":
+        elif coding in self._PIL_encodings:
             self.paint_image(coding, img_data, x, y, width, height, options, callbacks)
         else:
-            raise Exception("invalid encoding: %s" % coding)
+            self.do_draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
+
+    def do_draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
+        raise Exception("invalid encoding: %s" % coding)
