@@ -59,9 +59,8 @@ def init_all_devices():
     driver.init()
     DEVICES = []
     log("CUDA driver version=%s", driver.get_driver_version())
-    log.info("PyCUDA version=%s", pycuda.VERSION_TEXT)
     ngpus = driver.Device.count()
-    log.info("CUDA version=%s found %s device(s):", ".".join([str(x) for x in driver.get_version()]), ngpus)
+    log.info("CUDA %s / PyCUDA %s, found %s device(s):", ".".join([str(x) for x in driver.get_version()]), pycuda.VERSION_TEXT, ngpus)
     da = driver.device_attribute
     cf = driver.ctx_flags
     for i in range(ngpus):
@@ -91,7 +90,7 @@ def init_all_devices():
             log("   compute capability: %#x (%s.%s)", compute, SMmajor, SMminor)
             try:
                 DEVICES.append(i)
-                log.info(" + %s (memory %s%% free, compute %#x)", device_info(device), 100*free/total, compute)
+                log.info("  + %s (memory: %s%% free, compute: %s.%s)", device_info(device), 100*free/total, SMmajor, SMminor)
             finally:
                 context.pop()
         except Exception, e:
@@ -147,7 +146,7 @@ def select_device(preferred_device_id=DEFAULT_CUDA_DEVICE_ID, min_compute=0):
         if selected_device_id>=0 and selected_device:
             log("select device: %s / %s", device_id, device)
             return selected_device_id, selected_device
-    return None, None
+    return -1, None
 
 
 #cache pre-compiled kernel cubins per device:
