@@ -4,16 +4,28 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os.path
 from xpra.log import Logger
 log = Logger("codec", "loader")
 
+#these codecs may well not load because we
+#do not require the libraries to be installed
+NOWARN = ["nvenc", "opencl"]
+
 codec_errors = {}
 codecs = {}
 def codec_import_check(name, description, top_module, class_module, *classnames):
-    log("codec_import_check%s", (name, description, top_module, class_module, classnames))
+    log("%s:", name)
+    log(" codec_import_check%s", (name, description, top_module, class_module, classnames))
     try:
         try:
             __import__(top_module, {}, {}, [])
+        except ImportError, e:
+            log(" cannot import %s (%s): %s", name, description, e)
+            codec_errors[name] = e
+            return None
+        #module is present
+        try:
             log(" %s found, will check for %s in %s", top_module, classnames, class_module)
             for classname in classnames:
                 ic =  __import__(class_module, {}, {}, classname)
@@ -23,38 +35,44 @@ def codec_import_check(name, description, top_module, class_module, *classnames)
                 return ic
         except ImportError, e:
             codec_errors[name] = e
-            log(" cannot import %s (%s): %s", name, description, e)
-            #the required module does not exist
-            log(" xpra was probably built with the option: --without-%s", name)
+            l = log.warn
+            if name in NOWARN:
+                l = log.debug
+            l(" cannot import %s (%s): %s", name, description, e)
     except Exception, e:
         codec_errors[name] = e
-        log.warn("cannot load %s (%s): %s missing from %s: %s", name, description, classname, class_module, e)
+        log.warn(" cannot load %s (%s): %s missing from %s: %s", name, description, classname, class_module, e)
     return None
 codec_versions = {}
-def add_codec_version(name, top_module, version="get_version()"):
+def add_codec_version(name, top_module, version="get_version()", alt_version=None):
     try:
-        fieldname = version
-        if version.endswith("()"):
-            fieldname = version[:-2]
-        module = __import__(top_module, {}, {}, [fieldname])
-        if not hasattr(module, fieldname):
-            log.warn("cannot find %s in %s", fieldname, module)
-            return
-        v = getattr(module, fieldname)
-        if version.endswith("()") and v:
-            v = v()
-        global codec_versions
-        codec_versions[name] = v
-        #optional info:
-        if hasattr(module, "get_info"):
-            info = getattr(module, "get_info")
-            log("info(%s)=%s", top_module, info())
+        fieldnames = [x for x in (version, alt_version) if x is not None]
+        for fieldname in fieldnames:
+            if version.endswith("()"):
+                fieldname = version[:-2]
+            module = __import__(top_module, {}, {}, [fieldname])
+            if not hasattr(module, fieldname):
+                continue
+            v = getattr(module, fieldname)
+            if version.endswith("()") and v:
+                v = v()
+            global codec_versions
+            codec_versions[name] = v
+            #optional info:
+            if hasattr(module, "get_info"):
+                info = getattr(module, "get_info")
+                log(" %s info(%s)=%s", name, top_module, info())
+            return v
+        if name in codecs:
+            log.warn(" cannot find %s in %s", " or ".join(fieldnames), module)
+        else:
+            log(" no version information for missing codec %s", name)
     except ImportError, e:
-        log("cannot import %s: %s", name, e)
         #not present
-        pass
+        log(" cannot import %s: %s", name, e)
     except Exception, e:
         log.warn("error during codec import: %s", e)
+    return None
 
 
 loaded = False
@@ -65,7 +83,7 @@ def load_codecs():
     loaded = True
     log("loading codecs")
     codec_import_check("PIL", "Python Imaging Library", "PIL", "PIL", "Image")
-    add_codec_version("PIL", "PIL.Image", "VERSION")
+    add_codec_version("PIL", "PIL.Image", "PILLOW_VERSION", "VERSION")
 
     codec_import_check("enc_vpx", "vpx encoder", "xpra.codecs.vpx", "xpra.codecs.vpx.encoder", "Encoder")
     codec_import_check("dec_vpx", "vpx decoder", "xpra.codecs.vpx", "xpra.codecs.vpx.decoder", "Decoder")
@@ -89,9 +107,6 @@ def load_codecs():
     codec_import_check("csc_opencl", "OpenCL colorspace conversion", "xpra.codecs.csc_opencl", "xpra.codecs.csc_opencl.colorspace_converter", "ColorspaceConverter")
     add_codec_version("opencl", "xpra.codecs.csc_opencl.colorspace_converter")
 
-    codec_import_check("csc_nvcuda", "CUDA colorspace conversion", "xpra.codecs.csc_nvcuda", "xpra.codecs.csc_nvcuda.colorspace_converter", "ColorspaceConverter")
-    add_codec_version("nvcuda", "xpra.codecs.csc_nvcuda.colorspace_converter")
-
     #ffmpeg v1:
     codec_import_check("dec_avcodec", "avcodec decoder", "xpra.codecs.dec_avcodec", "xpra.codecs.dec_avcodec.decoder", "Decoder")
     add_codec_version("avcodec", "xpra.codecs.dec_avcodec.decoder")
@@ -100,32 +115,30 @@ def load_codecs():
     codec_import_check("dec_avcodec2", "avcodec2 decoder", "xpra.codecs.dec_avcodec2", "xpra.codecs.dec_avcodec2.decoder", "Decoder")
     add_codec_version("avcodec2", "xpra.codecs.dec_avcodec2.decoder")
 
-    import __builtin__
-    if "bytearray" in __builtin__.__dict__:
-        def nowebp(remove=["enc_webp", "enc_webp_lossless"]):
-            for x in remove:
-                if x in codecs:
-                    del codecs[x]
-        #no bytearray (python 2.6 or later), no webp
-        try:
+    #webp via cython:
+    codec_import_check("enc_webp", "webp encoder", "xpra.codecs.webp", "xpra.codecs.webp.encode", "compress")
+    add_codec_version("enc_webp", "xpra.codecs.webp.encode")
+
+    codec_import_check("dec_webp", "webp decoder", "xpra.codecs.webp", "xpra.codecs.webp.decode", "decompress")
+    add_codec_version("dec_webp", "xpra.codecs.webp.decode")
+
+    #no bytearray (python 2.6 or later) or no bitmap handlers, no webm:
+    from xpra.os_util import builtins
+    webm_handlers = codec_import_check("webm_bitmap_handlers", "webp bitmap handler", "xpra.codecs.webm", "xpra.codecs.webm.handlers", "BitmapHandler")
+    if ("bytearray" in builtins.__dict__) and webm_handlers:
+        codec_import_check("enc_webm", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", \
+                           "EncodeRGB", "EncodeRGBA", "EncodeBGR", "EncodeBGRA", \
+                           "EncodeLosslessRGB", "EncodeLosslessRGBA", "EncodeLosslessBGRA", "EncodeLosslessBGR")
+
+        v = add_codec_version("webm", "xpra.codecs.webm", "__VERSION__")
+        MIN_V = "0.2.3"
+        if v<MIN_V:
+            log.warn("python-webm error: found version %s but the minimum required is %s", v, MIN_V)
+            log.warn(" webm decoding has been disabled to prevent memory leaks")
+        else:
             #these symbols are all available upstream as of libwebp 0.2:
-            codec_import_check("enc_webp", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", "EncodeRGB", "EncodeRGBA", "EncodeBGR", "EncodeBGRA")
-            codec_import_check("dec_webp", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.decode", "DecodeRGB", "DecodeRGBA", "DecodeBGR", "DecodeBGRA")
-            #these symbols were added in libwebp 0.4, and we added HAS_LOSSLESS to the wrapper:
-            _enc_webp_lossless = codec_import_check("enc_webp_lossless", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", "HAS_LOSSLESS", "EncodeLosslessRGB", "EncodeLosslessRGBA", "EncodeLosslessBGRA", "EncodeLosslessBGR")
-            if _enc_webp_lossless:
-                #the fact that the python functions are defined is not enough
-                #we need to check if the underlying C functions actually exist:
-                if not _enc_webp_lossless.HAS_LOSSLESS:
-                    nowebp(["enc_webp_lossless"])
-            add_codec_version("webp", "xpra.codecs.webm", "__VERSION__")
-            webp_handlers = codec_import_check("webp_bitmap_handlers", "webp bitmap handler", "xpra.codecs.webm", "xpra.codecs.webm.handlers", "BitmapHandler")
-            #we need the handlers to encode:
-            if not webp_handlers:
-                nowebp()
-        except Exception, e:
-            log.warn("cannot load webp: %s", e)
-            nowebp()
+            codec_import_check("dec_webm", "webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.decode", "DecodeRGB", "DecodeRGBA", "DecodeBGR", "DecodeBGRA")
+
     log("done loading codecs")
     log("found:")
     #print("codec_status=%s" % codecs)
@@ -135,6 +148,7 @@ def load_codecs():
     for name, version in codec_versions.items():
         log("* %s : %s" % (name.ljust(20), version))
 
+
 def get_codec_error(name):
     return codec_errors.get(name)
 
@@ -142,6 +156,10 @@ def get_codec(name):
     load_codecs()
     return codecs.get(name)
 
+def get_codec_version(name):
+    load_codecs()
+    return codec_versions.get(name)
+
 def has_codec(name):
     load_codecs()
     return name in codecs
@@ -152,9 +170,12 @@ ALL_OLD_ENCODING_NAMES_TO_NEW = {"x264" : "h264", "vpx" : "vp8", "rgb24" : "rgb"
 ALL_NEW_ENCODING_NAMES_TO_OLD = {"h264" : "x264", "vp8" : "vpx", "rgb" : "rgb24"}
 
 ALL_CODECS = "PIL", "enc_vpx", "dec_vpx", "enc_x264", "enc_x265", "nvenc", \
-            "csc_swscale", "csc_cython", "csc_opencl", "csc_nvcuda", \
+            "csc_swscale", "csc_cython", "csc_opencl", \
             "dec_avcodec", "dec_avcodec2", \
-            "enc_webp", "enc_webp_lossless", "webp_bitmap_handlers", "dec_webp"
+            "enc_webm", \
+            "dec_webm", \
+            "enc_webp", \
+            "dec_webp"
 
 #note: this is just for defining the order of encodings,
 #so we have both core encodings (rgb24/rgb32) and regular encodings (rgb) in here:
@@ -200,10 +221,14 @@ def encodings_help(encodings):
     h = []
     for e in HELP_ORDER:
         if e in encodings:
-            ehelp = ENCODINGS_HELP.get(e)
-            h.append(e.ljust(12) + ehelp)
+            h.append(encoding_help(e))
     return h
 
+def encoding_help(encoding):
+    ehelp = ENCODINGS_HELP.get(encoding, "")
+    return encoding.ljust(12) + ehelp
+
+
 
 def main():
     from xpra.platform import init, clean
@@ -214,17 +239,28 @@ def main():
             log.enable_debug()
 
         load_codecs()
-        print("codecs/csc modules found:")
+        print("codecs and csc modules found:")
         #print("codec_status=%s" % codecs)
         for name in ALL_CODECS:
             mod = codecs.get(name, "")
             if mod and hasattr(mod, "__file__"):
                 mod = mod.__file__
-            print("* %s : %s %s" % (name.ljust(20), str(name in codecs).ljust(10), mod))
+                if mod.startswith(os.getcwd()):
+                    mod = mod[len(os.getcwd()):]
+                    if mod.startswith(os.path.sep):
+                        mod = mod[1:]
+            print("* %s : %s" % (name.ljust(20), mod))
         print("")
         print("codecs versions:")
-        for name, version in codec_versions.items():
-            print("* %s : %s" % (name.ljust(20), version))
+        def pver(v):
+            if type(v)==tuple:
+                return ".".join([str(x) for x in v])
+            elif type(v)==str and v.startswith("v"):
+                return v[1:]
+            return str(v)
+        for name in sorted(codec_versions.keys()):
+            version = codec_versions[name]
+            print("* %s : %s" % (name.ljust(20), pver(version)))
     finally:
         #this will wait for input on win32:
         clean()
