@@ -22,7 +22,6 @@ CODEC_TO_MODULE = {"vpx"        : "vpx",
                    "swscale"    : "csc_swscale",
                    "cython"     : "csc_cython",
                    "opencl"     : "csc_opencl",
-                   "nvcuda"     : "csc_nvcuda",
                    "avcodec"    : "dec_avcodec",
                    "avcodec2"   : "dec_avcodec2"}
 
@@ -47,8 +46,8 @@ def try_import_modules(codec_names):
 #all the codecs we know about:
 #try to import the module that contains them (cheap check):
 ALL_VIDEO_ENCODER_OPTIONS = try_import_modules(["x264", "vpx", "x265", "nvenc"])
-ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "cython", "opencl", "nvcuda"])
-NO_GFX_CSC_OPTIONS = [x for x in ALL_CSC_MODULE_OPTIONS if x not in ("opencl", "nvcuda")]
+ALL_CSC_MODULE_OPTIONS = try_import_modules(["swscale", "cython", "opencl"])
+NO_GFX_CSC_OPTIONS = [x for x in ALL_CSC_MODULE_OPTIONS if x not in ("opencl", )]
 ALL_VIDEO_DECODER_OPTIONS = try_import_modules(["avcodec", "avcodec2", "vpx"])
 
 PREFERRED_ENCODER_ORDER = ["nvenc", "x264", "vpx", "x265"]
@@ -134,6 +133,8 @@ class VideoHelper(object):
         self.video_encoders = [x for x in video_encoders if x in ALL_VIDEO_ENCODER_OPTIONS]
         self.csc_modules    = [x for x in csc_modules    if x in ALL_CSC_MODULE_OPTIONS]
         self.video_decoders = [x for x in video_decoders if x in ALL_VIDEO_DECODER_OPTIONS]
+        log("VideoHelper.set_modules(%s, %s, %s) video encoders=%s, csc=%s, video decoders=%s",
+            video_encoders, csc_modules, video_decoders, self.video_encoders, self.csc_modules, self.video_decoders)
 
     def cleanup(self):
         try:
@@ -162,21 +163,16 @@ class VideoHelper(object):
             self.init()
         #manual deep-ish copy: make new dictionaries and lists,
         #but keep the same codec specs:
-        ves = {}
-        for enc, d in self._video_encoder_specs.items():
-            for ifmt, l in d.items():
-                for cspec in l:
-                    ves.setdefault(enc, {}).setdefault(ifmt, []).append(cspec)
-        ces = {}
-        for ifmt, d in self._csc_encoder_specs.items():
-            for ofmt, l in d.items():
-                for cspec in l:
-                    ces.setdefault(ifmt, {}).setdefault(ofmt, []).append(cspec)
-        vds = {}
-        for enc, d in self._video_encoder_specs.items():
-            for ifmt, l in d.items():
-                for dclass in l:
-                    ves.setdefault(enc, {}).setdefault(ifmt, []).append(dclass)
+        def deepish_clone_dict(indict):
+            outd = {}
+            for enc, d in indict.items():
+                for ifmt, l in d.items():
+                    for v in l:
+                        outd.setdefault(enc, {}).setdefault(ifmt, []).append(v)
+            return outd
+        ves = deepish_clone_dict(self._video_encoder_specs)
+        ces = deepish_clone_dict(self._csc_encoder_specs)
+        vds = deepish_clone_dict(self._video_decoder_specs)
         return VideoHelper(ves, ces, vds, True)
 
     def get_info(self):
@@ -196,10 +192,12 @@ class VideoHelper(object):
         return d
 
     def init(self):
+        log("VideoHelper.init()")
         load_codecs()
         try:
             self._lock.acquire()
             #check again with lock held (in case of race):
+            log("VideoHelper.init() initialized=%s", self._initialized)
             if self._initialized:
                 return
             self.init_video_encoders_options()
@@ -208,6 +206,7 @@ class VideoHelper(object):
             self._initialized = True
         finally:
             self._lock.release()
+        log("VideoHelper.init() done")
 
     def get_encodings(self):
         return self._video_encoder_specs.keys()
@@ -243,7 +242,7 @@ class VideoHelper(object):
         encoder_module = get_codec(encoder_name)
         log("init_video_encoder_option(%s) module=%s", encoder_name, encoder_module)
         if not encoder_module:
-            log.warn("video encoder %s could not be loaded: %s", encoder_module, get_codec_error(encoder_name))
+            log.warn("video encoder %s could not be loaded: %s", encoder_name, get_codec_error(encoder_name))
             return
         encoder_type = encoder_module.get_type()
         try:
@@ -279,7 +278,7 @@ class VideoHelper(object):
         for src_format, d in sorted(self._csc_encoder_specs.items()):
             log("%s - %s options:", src_format, len(d))
             for dst_format, specs in sorted(d.items()):
-                log(" * %s via: %s", dst_format, sorted(list(specs)))
+                log(" * %s via: %s", dst_format, sorted(list(specs), key=lambda spec: spec.codec_type))
 
     def init_csc_option(self, csc_name):
         csc_module = get_codec(csc_name)
