@@ -181,3 +181,16 @@ for _, _, _, _, layout, variants in WIN32_LAYOUTS.values():
     for v in variants:
         if v not in l:
             l.append(v)
+
+def parse_xkbmap_query(xkbmap_query):
+    """ parses the output of "setxkbmap -query" into a dict """
+    import re
+    settings = {}
+    opt_re = re.compile("(\w*):\s*(.*)")
+    for line in xkbmap_query.splitlines():
+        m = opt_re.match(line)
+        if m:
+            v = m.group(2).strip()
+            if v!=",":
+                settings[m.group(1)] = v
+    return settings
