@@ -28,6 +28,8 @@ from xpra.os_util import Queue, strtobytes, get_hex_uuid
 from xpra.daemon_thread import make_daemon_thread
 from xpra.simple_stats import std_unit, std_unit_dec
 from xpra.net.bytestreams import ABORT
+from xpra.os_util import builtins
+_memoryview = builtins.__dict__.get("memoryview")
 
 try:
     from Crypto.Cipher import AES
@@ -43,6 +45,8 @@ try:
     from lz4 import LZ4_compress, LZ4_uncompress        #@UnresolvedImport
     has_lz4 = True
     def lz4_compress(packet, level):
+        if _memoryview and isinstance(packet, _memoryview):
+            packet = packet.tobytes()
         return level + LZ4_FLAG, LZ4_compress(packet)
 except Exception, e:
     log("lz4 not found: %s", e)
@@ -59,27 +63,35 @@ try:
         from xpra.net.rencode import loads as rencode_loads  #@UnresolvedImport
         from xpra.net.rencode import __version__ as rencode_version
     except ImportError, e:
-        log.warn("rencode load error: %s", e)
+        log.warn("rencode import error: %s", e)
 except Exception, e:
-    log.warn("xpra.rencode is missing: %s", e)
+    log.error("error loading rencode", exc_info=True)
 has_rencode = rencode_dumps is not None and rencode_loads is not None and rencode_version is not None
 use_rencode = has_rencode and os.environ.get("XPRA_USE_RENCODER", "1")=="1"
+log("protocol: has_rencode=%s, use_rencode=%s, version=%s", has_rencode, use_rencode, rencode_version)
 
-bencode, bdecode = None, None
-try:
-    from xpra.net.bencode import bencode, bdecode, __version__ as bencode_version
-except Exception, e:
-    log.warn("xpra.bencode is missing: %s", e)
+bencode, bdecode, bencode_version = None, None, None
+if sys.version_info[0]<3:
+    #bencode needs porting to Python3..
+    try:
+        try:
+            from xpra.net.bencode import bencode, bdecode, __version__ as bencode_version
+        except ImportError, e:
+            log.warn("bencode import error: %s", e, exc_info=True)
+    except Exception, e:
+        log.error("error loading bencoder", exc_info=True)
 has_bencode = bencode is not None and bdecode is not None
 use_bencode = has_bencode and os.environ.get("XPRA_USE_BENCODER", "1")=="1"
-
+log("protocol: has_bencode=%s, use_bencode=%s, version=%s", has_bencode, use_bencode, bencode_version)
 
 #stupid python version breakage:
 if sys.version > '3':
     long = int          #@ReservedAssignment
     unicode = str           #@ReservedAssignment
     def zcompress(packet, level):
-        return level + ZLIB_FLAG, compress(bytes(packet, 'UTF-8'), level)
+        if type(packet)!=bytes:
+            packet = bytes(packet, 'UTF-8')
+        return level + ZLIB_FLAG, compress(packet, level)
 else:
     def zcompress(packet, level):
         return level + ZLIB_FLAG, compress(str(packet), level)
@@ -126,17 +138,26 @@ def new_cipher_caps(proto, cipher, encryption_key):
                  "cipher.key_stretch_iterations" : iterations
                  }
 
-def get_network_caps():
+def get_network_caps(legacy=True):
+    try:
+        from xpra.net.mmap_pipe import can_use_mmap
+        mmap = can_use_mmap()
+    except:
+        mmap = False
     caps = {
-                "raw_packets"           : True,
-                #for backwards compatibility only:
-                "chunked_compression"   : True,
                 "digest"                : ("hmac", "xor"),
                 "rencode"               : use_rencode,
                 "bencode"               : use_bencode,
                 "lz4"                   : use_lz4,
-                "zlib"                  : True,
+                "mmap"                  : mmap,
                }
+    if legacy:
+        #for backwards compatibility only:
+        caps.update({
+                "raw_packets"           : True,
+                "zlib"                  : True,
+                "chunked_compression"   : True
+                })
     try:
         import Crypto
         caps["pycrypto.version"] = Crypto.__version__
@@ -178,7 +199,7 @@ class Compressed(object):
         self.data = data
     def __len__(self):
         return len(self.data)
-    def __str__(self):
+    def __repr__(self):
         return  "Compressed(%s: %s bytes)" % (self.datatype, len(self.data))
 
 class LevelCompressed(Compressed):
@@ -188,7 +209,7 @@ class LevelCompressed(Compressed):
         self.level = level
     def __len__(self):
         return len(self.data)
-    def __str__(self):
+    def __repr__(self):
         return  "LevelCompressed(%s: %s bytes as %s/%s)" % (self.datatype, len(self.data), self.algorithm, self.level)
 
 def compressed_wrapper(datatype, data, level=5, lz4=False):
@@ -237,7 +258,7 @@ class Protocol(object):
         self.output_raw_packetcount = 0
         #initial value which may get increased by client/server after handshake:
         self.max_packet_size = 32*1024
-        self.abs_max_packet_size = 32*1024*1024
+        self.abs_max_packet_size = 256*1024*1024
         self.large_packets = ["hello"]
         self.send_aliases = {}
         self.receive_aliases = {}
@@ -326,7 +347,7 @@ class Protocol(object):
         log("set_cipher_out%s", (ciphername, iv, password, key_salt, iterations))
         self.cipher_out, self.cipher_out_block_size = self.get_cipher(ciphername, iv, password, key_salt, iterations)
 
-    def __str__(self):
+    def __repr__(self):
         return "Protocol(%s)" % self._conn
 
     def get_threads(self):
@@ -413,6 +434,8 @@ class Protocol(object):
         chunks, proto_flags = self.encode(packet)
         try:
             self._write_lock.acquire()
+            if self._closed:
+                return
             self._add_chunks_to_queue(chunks, proto_flags, start_send_cb, end_send_cb)
         finally:
             self._write_lock.release()
@@ -637,13 +660,17 @@ class Protocol(object):
             self.close()
             return
         for buf, start_cb, end_cb in items:
+            con = self._conn
+            if not con:
+                return
             if start_cb:
                 try:
-                    start_cb(self._conn.output_bytecount)
+                    start_cb(con.output_bytecount)
                 except:
-                    log.error("error on %s", start_cb, exc_info=True)
+                    if not self._closed:
+                        log.error("error on %s", start_cb, exc_info=True)
             while buf and not self._closed:
-                written = self._conn.write(buf)
+                written = con.write(buf)
                 if written:
                     buf = buf[written:]
                     self.output_raw_packetcount += 1
@@ -804,7 +831,7 @@ class Protocol(object):
                 if compression_level>0:
                     try:
                         if compression_level & LZ4_FLAG:
-                            assert has_lz4
+                            assert use_lz4
                             data = LZ4_uncompress(data)
                         else:
                             data = decompress(data)
@@ -877,6 +904,7 @@ class Protocol(object):
             then no matter what, we close the connection and stop the threads.
         """
         if self._closed:
+            log("flush_then_close: already closed")
             return
         def wait_for_queue(timeout=10):
             #IMPORTANT: if we are here, we have the write lock held!
@@ -892,11 +920,28 @@ class Protocol(object):
             else:
                 log("flush_then_close: queue is now empty, sending the last packet and closing")
                 chunks, proto_flags = self.encode(last_packet)
-                def close_cb(*args):
+                def close_and_release():
+                    log("flush_then_close: wait_for_packet_sent() close_and_release()")
                     self.close()
-                self._add_chunks_to_queue(chunks, proto_flags, start_send_cb=None, end_send_cb=close_cb)
-                self._write_lock.release()
-                self.timeout_add(5*1000, self.close)
+                    try:
+                        self._write_lock.release()
+                    except:
+                        pass
+                def wait_for_packet_sent():
+                    log("flush_then_close: wait_for_packet_sent() queue.empty()=%s, closed=%s", self._write_queue.empty(), self._closed)
+                    if self._write_queue.empty() or self._closed:
+                        #it got sent, we're done!
+                        close_and_release()
+                        return False
+                    return not self._closed     #run until we manage to close (here or via the timeout)
+                def packet_queued(*args):
+                    #if we're here, we have the lock and the packet is in the write queue
+                    log("flush_then_close: packet_queued() closed=%s", self._closed)
+                    #check every 100ms
+                    self.timeout_add(100, wait_for_packet_sent)
+                self._add_chunks_to_queue(chunks, proto_flags, start_send_cb=None, end_send_cb=packet_queued)
+                #just in case wait_for_packet_sent never fires:
+                self.timeout_add(5*1000, close_and_release)
 
         def wait_for_write_lock(timeout=100):
             if not self._write_lock.acquire(False):
@@ -914,7 +959,10 @@ class Protocol(object):
         # -> wait_for_write_lock
         # -> wait_for_queue
         # -> _add_chunks_to_queue
-        # -> close
+        # -> packet_queued
+        # -> wait_for_packet_sent
+        # -> close_and_release
+        log("flush_then_close: wait_for_write_lock()")
         wait_for_write_lock()
 
     def close(self):
