@@ -21,9 +21,14 @@ try:
 except ImportError:
     from Queue import Queue             #@Reimport @UnusedImport
 
+try:
+    import builtins                     #@UnresolvedImport @UnusedImport (python3)
+except:
+    import __builtin__ as builtins      #@Reimport @UnusedImport
+
 
 SIGNAMES = {}
-for x in [x for x in dir(signal) if x.startswith("SIG")]:
+for x in [x for x in dir(signal) if x.startswith("SIG") and not x.startswith("SIG_")]:
     SIGNAMES[getattr(signal, x)] = x
 
 
@@ -45,6 +50,12 @@ except:
         from io import StringIO as StringIOClass        #@Reimport @UnusedImport
 
 
+if sys.version_info < (2, 6):
+    from sets import ImmutableSet
+else:
+    ImmutableSet = frozenset
+
+
 if sys.version < '3':
     def strtobytes(x):
         return x
@@ -56,6 +67,8 @@ else:
             return x
         return x.encode()
     def bytestostr(x):
+        if type(x)==str:
+            return x
         return x.decode()
 
 
@@ -75,7 +88,7 @@ def platform_name(sys_platform, release):
         return "unknown"
     PLATFORMS = {"win32"    : "Microsoft Windows",
                  "cygwin"   : "Windows/Cygwin",
-                 "linux2"   : "Linux",
+                 "linux.*"  : "Linux",
                  "darwin"   : "Mac OSX",
                  "freebsd.*": "FreeBSD",
                  "os2"      : "OS/2",
@@ -93,7 +106,7 @@ def platform_name(sys_platform, release):
 def os_info(sys_platform, platform_release, platform_platform, platform_linux_distribution):
     s = [platform_name(sys_platform, platform_release)]
     if platform_linux_distribution and len(platform_linux_distribution)==3 and len(platform_linux_distribution[0])>0:
-        s.append(" ".join(platform_linux_distribution))
+        s.append(" ".join([str(x) for x in platform_linux_distribution]))
     elif platform_platform:
         s.append(platform_platform)
     return s
@@ -161,6 +174,13 @@ def get_user_uuid():
     uupdate(os.environ.get("HOME", ""))
     return u.hexdigest()
 
+def is_Ubuntu():
+    try:
+        v = load_binary_file("/etc/issue")
+        return bool(v) and v.find("Ubuntu")>=0
+    except:
+        pass
+    return False
 
 def load_binary_file(filename):
     if not os.path.exists(filename):
