@@ -132,7 +132,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
             log("swapkeys_toggled(%s) swap keys enabled=%s", args, v)
             if self.keyboard:
                 self.keyboard.swap_keys = v
-        self.swapkeys_menuitem = self.checkitem("Control/Option Key Swap", swapkeys_toggled)
+        self.swapkeys_menuitem = self.checkitem("Control/Command Key Swap", swapkeys_toggled)
         def set_swapkeys_menuitem(*args):
             if self.keyboard:
                 log("set_swapkeys_menuitem(%s) swap_keys=%s", args, self.keyboard.swap_keys)
@@ -146,7 +146,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
     def make_numlockmenuitem(self):
         def numlock_toggled(*args):
             v = self.numlock_menuitem.get_active()
-            log("numlock_toggled(%s) %s", args, v)
+            log("numlock_toggled(%s) menu active=%s", args, v)
             if self.keyboard:
                 self.keyboard.num_lock_state = v
         self.numlock_menuitem = self.checkitem("Num Lock", cb=numlock_toggled)
