@@ -12,6 +12,12 @@ log = Logger("keyboard", "win32")
 
 
 class Keyboard(KeyboardBase):
+    """ This is for getting keys from the keyboard on the client side.
+        Deals with GTK bugs and oddities:
+        * missing 'Num_Lock'
+        * simulate 'Alt_Gr'
+    """
+
     def __init__(self):
         KeyboardBase.__init__(self)
         self.emulate_altgr = False
@@ -107,6 +113,7 @@ class Keyboard(KeyboardBase):
             So we just ignore those and rely on the list of "modifiers" passed
             with each keypress to let the server set them for us when needed.
         """
+        log("win32.process_key_event(%s, %s, %s)", send_key_action_cb, wid, key_event)
         if key_event.keyval==2**24-1 and key_event.keyname=="VoidSymbol":
             return
         #self.modifier_mappings = None       #{'control': [(37, 'Control_L'), (105, 'Control_R')], 'mod1':
@@ -122,12 +129,20 @@ class Keyboard(KeyboardBase):
                 self.emulate_altgr = key_event.pressed
                 if key_event.pressed and self.last_key_event_sent:
                     #check for spurious control and undo it
-                    last_wid, last_keyname, last_pressed = self.last_key_event_sent[:3]
-                    if last_wid==wid and last_keyname=="Control_L" and last_pressed==True:
+                    last_wid, last_key_event = self.last_key_event_sent
+                    if last_wid==wid and last_key_event.keyname=="Control_L" and last_key_event.pressed==True:
                         #undo it:
-                        undo = self.last_key_event_sent[:]
-                        undo[2] = False
-                        send_key_action_cb(*undo)
+                        last_key_event.pressed = False
+                        KeyboardBase.process_key_event(self, send_key_action_cb, last_wid, last_key_event)
                 self.AltGr_modifiers(key_event.modifiers, not key_event.pressed)
-        self.last_key_event_sent = key_event
+        if key_event.keyname=="period" and key_event.keyval==46 and key_event.keycode==110:
+            log("patched 'period' keyname to 'KP_Decimal'")
+            key_event.keyname = "KP_Decimal"
+        elif key_event.keyname=="dead_tilde" and key_event.keyval==65107 and key_event.keycode==50:
+            log("patched 'dead_tilde' keyname to 'asciitilde'")
+            key_event.keyname = "asciitilde"
+        elif key_event.keyname=="dead_grave" and key_event.keyval==65104 and key_event.keycode==55:
+            log("patched 'dead_grave' keyname to 'grave'")
+            key_event.keyname = "grave"
+        self.last_key_event_sent = (wid, key_event)
         send_key_action_cb(wid, key_event)
