@@ -12,6 +12,7 @@ log = Logger("shadow", "win32")
 from xpra.server.gtk_root_window_model import GTKRootWindowModel
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.server.shadow_server_base import ShadowServerBase
+from xpra.platform.win32.keyboard_config import KeyboardConfig
 
 BUTTON_EVENTS = {
                  #(button,up-or-down)  : win-event-name
@@ -43,13 +44,23 @@ class ShadowServer(ShadowServerBase, GTKServerBase):
         rx, ry = x-wx, y-wy
         win32api.SetCursorPos((rx, ry))
 
+    def get_keyboard_config(self, props):
+        return KeyboardConfig()
+
     def fake_key(self, keycode, press):
-        kc = self.keycodes.get(keycode)
-        if kc is None:
+        if keycode<=0:
             log.warn("no keycode found for %s", keycode)
             return
+        #KEYEVENTF_SILENT = 0X4;
+        flags = 0   #KEYEVENTF_SILENT
+        if press:
+            flags |= win32con.KEYEVENTF_KEYUP
+        #get the scancode:
+        MAPVK_VK_TO_VSC = 0
+        scancode = win32api.MapVirtualKey(keycode, MAPVK_VK_TO_VSC)
         #see: http://msdn.microsoft.com/en-us/library/windows/desktop/ms646304(v=vs.85).aspx
-        win32api.keybd_event(win32con.SHIFT_PRESSED, 0, win32con.KEYEVENTF_EXTENDEDKEY, 0)
+        log("fake_key(%s, %s) calling keybd_event(%s, %s, %s, 0)", keycode, press, keycode, scancode, flags)
+        win32api.keybd_event(keycode, scancode, flags, 0)
 
     def _process_button_action(self, proto, packet):
         wid, button, pressed, pointer, modifiers = packet[1:6]
