@@ -40,7 +40,7 @@ def OpenGL_safety_check():
         ur = distro[1]  #ie: "12.04"
         try:
             rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
-            if rnum<=[12, 04]:
+            if rnum<=[12, 4]:
                 return "Ubuntu %s is too buggy" % ur
         except:
             pass
@@ -78,7 +78,7 @@ def get_build_info():
     info = []
     try:
         from xpra.src_info import REVISION, LOCAL_MODIFICATIONS
-        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, CYTHON_VERSION, COMPILER_INFO
+        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, CYTHON_VERSION, COMPILER_VERSION
         info.append("Built on %s by %s" % (BUILT_ON, BUILT_BY))
         if BUILD_DATE:
             info.append(BUILD_DATE)
@@ -90,12 +90,12 @@ def get_build_info():
             info.append("revision %s" % REVISION)
         else:
             info.append("revision %s with %s local changes" % (REVISION, LOCAL_MODIFICATIONS))
-        if CYTHON_VERSION!="unknown" or COMPILER_INFO!="unknown":
+        if CYTHON_VERSION!="unknown" or COMPILER_VERSION!="unknown":
             info.append("")
         if CYTHON_VERSION!="unknown":
             info.append("built with Cython %s" % CYTHON_VERSION)
-        if COMPILER_INFO!="unknown":
-            info.append(COMPILER_INFO)
+        if COMPILER_VERSION!="unknown":
+            info.append(COMPILER_VERSION)
     except Exception, e:
         warn("Error: could not find the source or build information: %s" % e)
     return info
@@ -234,6 +234,7 @@ OPTION_TYPES = {
                     "window-layout"     : str,
                     "display"           : str,
                     "tcp-proxy"         : str,
+                    "debug"             : str,
                     #int options:
                     "quality"           : int,
                     "min-quality"       : int,
@@ -246,7 +247,6 @@ OPTION_TYPES = {
                     "max-bandwidth"     : float,
                     "auto-refresh-delay": float,
                     #boolean options:
-                    "debug"             : bool,
                     "daemon"            : bool,
                     "use-display"       : bool,
                     "fake-xinerama"     : bool,
@@ -275,6 +275,7 @@ OPTION_TYPES = {
                     "exit-ssh"          : bool,
                     "opengl"            : bool,
                     "mdns"              : bool,
+                    "swap-keys"         : bool,
                     #arrays of strings (default value, allowed options):
                     "video-encoders"    : list,
                     "csc-modules"       : list,
@@ -304,7 +305,7 @@ def get_defaults():
                     "host"              : "",
                     "username"          : username,
                     "auth"              : "",
-                    "remote-xpra"       : ".xpra/run-xpra",
+                    "remote-xpra"       : "~/.xpra/run-xpra",
                     "session-name"      : "",
                     "client-toolkit"    : "",
                     "dock-icon"         : "",
@@ -328,6 +329,7 @@ def get_defaults():
                     "window-layout"     : "",
                     "display"           : "",
                     "tcp-proxy"         : "",
+                    "debug"             : "",
                     "quality"           : -1,
                     "min-quality"       : 50,
                     "speed"             : -1,
@@ -337,7 +339,6 @@ def get_defaults():
                     "dpi"               : 96,
                     "max-bandwidth"     : 0.0,
                     "auto-refresh-delay": 0.25,
-                    "debug"             : False,
                     "daemon"            : True,
                     "use-display"       : False,
                     "fake-xinerama"     : True,
@@ -365,7 +366,8 @@ def get_defaults():
                     "exit-with-client"  : False,
                     "exit-ssh"          : True,
                     "opengl"            : OPENGL_DEFAULT,
-                    "mdns"              : True,
+                    "mdns"              : os.name=="posix" and not sys.platform.startswith("darwin"),
+                    "swap-keys"         : sys.platform.startswith("darwin"),    #only used on osx
                     "video-encoders"    : [],
                     "csc-modules"       : [],
                     "speaker-codec"     : [],
