@@ -63,16 +63,42 @@ def main(script_file, cmdline):
     from xpra.platform import init as platform_init, clean as platform_clean
     try:
         platform_init("Xpra")
-        parser, options, args = parse_cmdline(cmdline)
-        if not args:
-            parser.error("need a mode")
-        mode = args.pop(0)
-        return run_mode(script_file, parser, options, args, mode)
+        try:
+            defaults = make_defaults_struct()
+            parser, options, args = do_parse_cmdline(cmdline, defaults)
+            if not args:
+                print("xpra: need a mode")
+                return -1
+            mode = args.pop(0)
+            return run_mode(script_file, parser, options, args, mode, defaults)
+        except SystemExit:
+            raise
+        except:
+            e = sys.exc_info()[1]
+            print("xpra main exception: %s" % e)
+            sys.exit(1)
     finally:
         platform_clean()
 
 
+def fixup_debug_option(value):
+    """ backwards compatible parsing of the debug option, which used to be a boolean """
+    if not value:
+        return ""
+    value = str(value)
+    if value.strip().lower() in ("yes", "true", "on", "1"):
+        return "all"
+    if value.strip().lower() in ("no", "false", "off", "0"):
+        return ""
+    #if we're here, the value should be a CSV list of categories
+    return value
+
+
 def parse_cmdline(cmdline):
+    defaults = make_defaults_struct()
+    return do_parse_cmdline(cmdline, defaults)
+
+def do_parse_cmdline(cmdline, defaults):
     #################################################################
     ## NOTE NOTE NOTE
     ##
@@ -107,7 +133,6 @@ def parse_cmdline(cmdline):
 
     parser = OptionParser(version="xpra v%s" % XPRA_VERSION,
                           usage="\n" + "".join(command_options))
-    defaults = make_defaults_struct()
     hidden_options = {"display" : defaults.display}
     if len(server_modes):
         group = OptionGroup(parser, "Server Options",
@@ -385,7 +410,7 @@ def parse_cmdline(cmdline):
                           help="Path to the default image which will be used for all windows (the application may override this)")
     # let the platform specific code add its own options:
     # adds "--no-tray" for platforms that support it
-    add_client_options(group)
+    add_client_options(group, defaults)
     hidden_options["tray"] =  True
     hidden_options["delay_tray"] =  False
     group.add_option("--tray-icon", action="store",
@@ -397,10 +422,10 @@ def parse_cmdline(cmdline):
                       + "If no shortcuts are defined, it defaults to '%s'" % (",".join(defaults.key_shortcut or [])))
     group.add_option("--keyboard-sync", action="store_true",
                       dest="keyboard_sync", default=defaults.keyboard_sync,
-                      help="Enable keyboard state synchronization (default: %)")
+                      help="Enable keyboard state synchronization (default: %s)" % enabled_str(defaults.keyboard_sync))
     group.add_option("--no-keyboard-sync", action="store_false",
                       dest="keyboard_sync", default=defaults.keyboard_sync,
-                      help="Disable keyboard state synchronization, prevents keys from repeating on high latency links but also may disrupt applications which access the keyboard directly (default: %default)")
+                      help="Disable keyboard state synchronization, prevents keys from repeating on high latency links but also may disrupt applications which access the keyboard directly (default: %s)" % enabled_str(defaults.keyboard_sync))
 
     group = OptionGroup(parser, "Advanced Options",
                 "These options apply to both client and server. Please refer to the man page for details.")
@@ -415,12 +440,9 @@ def parse_cmdline(cmdline):
     group.add_option("--socket-dir", action="store",
                       dest="socket_dir", default=defaults.socket_dir,
                       help="Directory to place/look for the socket files in (default: %s)" % default_socket_dir_str)
-    debug_default = ""
-    if defaults.debug:
-        debug_default = "all"
     group.add_option("-d", "--debug", action="store",
-                      dest="debug", default=debug_default, metavar="FILTER1,FILTER2,...",
-                      help="List of categories to enable debugging for (or \"all\", \"help\")")
+                      dest="debug", default=defaults.debug, metavar="FILTER1,FILTER2,...",
+                      help="List of categories to enable debugging for (you can also use \"all\" or \"help\", default: '%default')")
     group.add_option("--ssh", action="store",
                       dest="ssh", default=defaults.ssh, metavar="CMD",
                       help="How to run ssh (default: '%default')")
@@ -441,7 +463,7 @@ def parse_cmdline(cmdline):
                       help="When creating the mmap file with the client, set the group permission on the mmap file to the same value as the owner of the server socket file we connect to (default: '%default')")
     group.add_option("--enable-pings", action="store_true",
                       dest="pings", default=defaults.pings,
-                      help="Send ping packets every second to gather latency statistics")
+                      help="Send ping packets every second to gather latency statistics (default: %s)" % enabled_str(defaults.pings))
     group.add_option("--clipboard-filter-file", action="store",
                       dest="clipboard_filter_file", default=defaults.clipboard_filter_file,
                       help="Name of a file containing regular expressions of clipboard contents that must be filtered out")
@@ -472,12 +494,13 @@ def parse_cmdline(cmdline):
 
     #process "help" arguments early:
     from xpra.log import KNOWN_FILTERS
+    options.debug = fixup_debug_option(options.debug)
     if options.debug:
         categories = options.debug.split(",")
         for cat in categories:
             if cat=="help":
                 print("known logging filters (there may be others): %s" % ", ".join(KNOWN_FILTERS))
-                sys.exit(1)
+                sys.exit(0)
 
     if options.encoding:
         #fix old encoding names if needed:
@@ -592,7 +615,7 @@ def configure_logging(options, mode):
         signal.signal(signal.SIGUSR1, sigusr1)
 
 
-def run_mode(script_file, parser, options, args, mode):
+def run_mode(script_file, parser, options, args, mode, defaults):
     #configure default logging handler:
     if os.name=="posix" and os.getuid()==0 and mode!="proxy":
         warn("\nWarning: running as root")
@@ -603,7 +626,7 @@ def run_mode(script_file, parser, options, args, mode):
         ssh_display = len(args)>0 and (args[0].startswith("ssh/") or args[0].startswith("ssh:"))
         if mode in ("start", "shadow") and ssh_display:
             #ie: "xpra start ssh:HOST:DISPLAY --start-child=xterm"
-            return run_remote_server(parser, options, args, mode)
+            return run_remote_server(parser, options, args, mode, defaults)
         elif (mode in ("start", "upgrade", "proxy") and supports_server) or (mode=="shadow" and supports_shadow):
             nox()
             from xpra.scripts.server import run_server
@@ -617,7 +640,13 @@ def run_mode(script_file, parser, options, args, mode):
             return run_list(parser, options, args)
         elif mode in ("_proxy", "_proxy_start", "_shadow_start") and (supports_server or supports_shadow):
             nox()
-            return run_proxy(parser, options, script_file, args, mode)
+            return run_proxy(parser, options, script_file, args, mode, defaults)
+        elif mode == "initenv":
+            from xpra.scripts.server import xpra_runner_shell_script, write_runner_shell_script
+            script = xpra_runner_shell_script(script_file, os.getcwd(), options.socket_dir)
+            dotxpra = DotXpra(options.socket_dir)
+            write_runner_shell_script(dotxpra, script, False)
+            return 0
         else:
             parser.error("invalid mode '%s'" % mode)
             return 1
@@ -801,7 +830,8 @@ def connect_to(display_desc, debug_cb=None, ssh_fail_cb=ssh_connect_failed):
     conn = None
     if dtype == "ssh":
         cmd = display_desc["full_ssh"]
-        cmd += display_desc["remote_xpra"] + display_desc["proxy_command"] + display_desc["display_as_args"]
+        proxy_cmd = display_desc["remote_xpra"] + display_desc["proxy_command"] + display_desc["display_as_args"]
+        cmd += ["sh -c 'xpra initenv >> /dev/null 2>&1;"+(" ".join(proxy_cmd))+"'"]
         try:
             kwargs = {}
             kwargs["stderr"] = sys.stderr
@@ -926,22 +956,7 @@ def run_client(parser, opts, extra_args, mode):
         from xpra.client.gobject_client_base import DetachXpraClient
         app = DetachXpraClient(connect(), opts)
     else:
-        if mode in ("attach"):
-            sys.stdout.write("xpra client version %s\n" % XPRA_VERSION)
-            sys.stdout.flush()
         app = make_client(parser.error, opts)
-        app.init(opts)
-        if opts.encoding:
-            #fix old encoding names if needed:
-            from xpra.codecs.loader import encodings_help
-            err = opts.encoding not in app.get_encodings()
-            if err and opts.encoding!="help":
-                print("invalid encoding: %s" % opts.encoding)
-            if opts.encoding=="help" or err:
-                print("%s xpra client supports the following encodings:\n * %s" % (app.client_toolkit(), "\n * ".join(encodings_help(app.get_encodings()))))
-                if err:
-                    return 1
-                return 0
         layouts = app.get_supported_window_layouts() or ["default"]
         layouts_str = ", ".join(layouts)
         if opts.window_layout and opts.window_layout.lower()=="help":
@@ -949,15 +964,27 @@ def run_client(parser, opts, extra_args, mode):
             return 0
         if opts.window_layout and opts.window_layout not in layouts:
             parser.error("window layout '%s' is not supported by the %s toolkit, valid options are: %s" % (opts.window_layout, app.client_toolkit(), layouts_str))
-        conn = connect()
+        app.init(opts)
+        if opts.encoding:
+            #fix old encoding names if needed:
+            err = opts.encoding not in app.get_encodings()
+            if err and opts.encoding!="help":
+                print("invalid encoding: %s" % opts.encoding)
+            if opts.encoding=="help" or err:
+                from xpra.codecs.loader import encodings_help
+                print("%s xpra client supports the following encodings:\n * %s" % (app.client_toolkit(), "\n * ".join(encodings_help(app.get_encodings()))))
+                if err:
+                    return 1
+                return 0
         def handshake_complete(*args):
             from xpra.log import Logger
             log = Logger()
             log.info("Attached to %s (press Control-C to detach)\n" % conn.target)
         if hasattr(app, "connect"):
             app.connect("handshake-complete", handshake_complete)
-        app.setup_connection(conn)
         app.init_ui(opts)
+        conn = connect()
+        app.setup_connection(conn)
     return do_run_client(app)
 
 def make_client(error_cb, opts):
@@ -1009,7 +1036,21 @@ def do_run_client(app):
     finally:
         app.cleanup()
 
-def run_remote_server(parser, opts, args, mode):
+def shellquote(s):
+    return "'" + s.replace("'", "'\\''") + "'"
+
+def strip_defaults_start_child(start_child, defaults_start_child):
+    if start_child and defaults_start_child:
+        #ensure we don't pass start-child commands
+        #which came from defaults (the configuration files)
+        #only the ones specified on the command line:
+        #(and only remove them once so the command line can re-add the same ones!)
+        for x in defaults_start_child:
+            if x in start_child:
+                start_child.remove(x)
+    return start_child
+
+def run_remote_server(parser, opts, args, mode, defaults):
     """ Uses the regular XpraClient with patched proxy arguments to tell run_proxy to start the server """
     params = parse_display_name(parser.error, opts, args[0])
     #add special flags to "display_as_args"
@@ -1017,10 +1058,23 @@ def run_remote_server(parser, opts, args, mode):
     if params["display"] is not None:
         proxy_args.append(params["display"])
     if opts.start_child:
-        for c in opts.start_child:
-            proxy_args.append("--start-child=%s" % c)
-    if opts.exit_with_children:
-        proxy_args.append("--exit-with-children")
+        start_child = strip_defaults_start_child(opts.start_child, defaults.start_child)
+        for c in start_child:
+            proxy_args.append(shellquote("--start-child=%s" % c))
+    #key=value options we forward:
+    for x in ("session-name", "encoding", "socket-dir", "dpi"):
+        v = getattr(opts, x.replace("-", "_"))
+        if v:
+            proxy_args.append("--%s=%s" % (x, v))
+    #thse options must be enabled explicitly (no disable option for most of them):
+    for e in ("exit-with-children", "mmap-group", "readonly"):
+        if getattr(opts, e.replace("-", "_")) is True:
+            proxy_args.append("--%s" % e)
+    #older versions only support disabling:
+    for e in ("pulseaudio", "mmap",
+              "system-tray", "clipboard", "bell"):
+        if getattr(opts, e.replace("-", "_")) is False:
+            proxy_args.append("--no-%s" % e)
     params["display_as_args"] = proxy_args
     #and use _proxy_start subcommand:
     if mode=="shadow":
@@ -1028,11 +1082,11 @@ def run_remote_server(parser, opts, args, mode):
     else:
         assert mode=="start"
         params["proxy_command"] = ["_proxy_start"]
-    conn = connect_or_fail(params)
     app = make_client(parser.error, opts)
-    app.setup_connection(conn)
     app.init(opts)
     app.init_ui(opts)
+    conn = connect_or_fail(params)
+    app.setup_connection(conn)
     do_run_client(app)
 
 def find_X11_displays(max_display_no=10):
@@ -1063,7 +1117,7 @@ def guess_X11_display():
     assert len(displays)==1, "too many live X11 displays to choose from"
     return ":%s" % displays[0]
 
-def run_proxy(parser, opts, script_file, args, mode):
+def run_proxy(parser, opts, script_file, args, mode, defaults):
     from xpra.server.proxy import XpraProxy
     assert "gtk" not in sys.modules
     if mode in ("_proxy_start", "_shadow_start"):
@@ -1081,9 +1135,12 @@ def run_proxy(parser, opts, script_file, args, mode):
                 display_name = args[0]
             else:
                 display_name = guess_X11_display()
+                #we now know the display name, so add it:
+                args = [display_name]
         cmd += args
-        if opts.start_child and len(opts.start_child)>0:
-            for x in opts.start_child:
+        start_child = strip_defaults_start_child(opts.start_child, defaults.start_child)
+        if start_child:
+            for x in start_child:
                 cmd.append("--start-child=%s" % x)
         if opts.exit_with_children:
             cmd.append("--exit-with-children")
@@ -1091,14 +1148,20 @@ def run_proxy(parser, opts, script_file, args, mode):
             cmd.append("--exit-with-client")
         def setsid():
             os.setsid()
-        Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE, preexec_fn=setsid)
+        proc = Popen(cmd, preexec_fn=setsid, shell=False, close_fds=True)
         dotxpra = DotXpra()
         start = time.time()
         while dotxpra.server_state(display_name, 1)!=DotXpra.LIVE:
-            if time.time()-start>5:
+            if time.time()-start>15:
                 warn("server failed to start after %.1f seconds - sorry!" % (time.time()-start))
                 return
             time.sleep(0.10)
+        #start a thread just to reap server startup process (yuk)
+        #(as the server process will exit as it daemonizes)
+        def reaper():
+            proc.wait()
+        from xpra.daemon_thread import make_daemon_thread
+        make_daemon_thread(reaper, "server-startup-reaper").start()
     server_conn = connect_or_fail(pick_display(parser, opts, args))
     app = XpraProxy(TwoFileConnection(sys.stdout, sys.stdin, info="stdin/stdout"), server_conn)
     signal.signal(signal.SIGINT, app.quit)
