@@ -9,7 +9,6 @@
 #  http://lists.partiwm.org/pipermail/parti-discuss/2008-September/000042.html
 # (also do not import anything that imports gtk)
 import gobject
-import glib
 import subprocess
 import sys
 import os.path
@@ -60,6 +59,7 @@ class ChildReaper(object):
         self._quit = quit_cb
         self._children_pids = {}
         self._dead_pids = set()
+        self._ignored_pids = set()
         from xpra.log import Logger
         self._logger = Logger("server", "util")
         old_python = sys.version_info < (2, 7) or sys.version_info[:2] == (3, 0)
@@ -95,16 +95,19 @@ class ChildReaper(object):
                 return False # Only call once
             gobject.timeout_add(0, check_once)
 
-    def add_process(self, process, command):
+    def add_process(self, process, command, ignore=False):
         process.command = command
+        assert process.pid>0
         self._children_pids[process.pid] = process
+        if ignore:
+            self._ignored_pids.add(process.pid)
 
     def check(self):
-        if self._children_pids:
+        pids = set(self._children_pids.keys()) - self._ignored_pids
+        if pids:
             for pid, proc in self._children_pids.items():
                 if proc.poll() is not None:
                     self.add_dead_pid(pid)
-            pids = set(self._children_pids.keys())
             if pids.issubset(self._dead_pids):
                 self._quit()
 
@@ -200,7 +203,7 @@ fi
 """)
     return "".join(script)
 
-def write_runner_shell_script(dotxpra, contents):
+def write_runner_shell_script(dotxpra, contents, overwrite=True):
     # This used to be given a display-specific name, but now we give it a
     # single fixed name and if multiple servers are started then the last one
     # will clobber the rest.  This isn't great, but the tradeoff is that it
@@ -210,6 +213,8 @@ def write_runner_shell_script(dotxpra, contents):
     # people run into problems or autodiscovery turns out to be less useful
     # than expected.
     scriptpath = os.path.join(dotxpra.confdir(), "run-xpra")
+    if os.path.exists(scriptpath) and not overwrite:
+        return
     # Write out a shell-script so that we can start our proxy in a clean
     # environment:
     scriptfile = open(scriptpath, "w")
@@ -294,7 +299,7 @@ def create_tcp_socket(host, iport):
 
 def setup_tcp_socket(host, iport):
     from xpra.log import Logger
-    log = Logger("net")
+    log = Logger("network")
     tcp_socket = create_tcp_socket(host, iport)
     def cleanup_tcp_socket():
         log.info("closing tcp socket %s:%s", host, iport)
@@ -326,7 +331,7 @@ def setup_local_socket(dotxpra, display_name, clobber, mmap_group):
     if sys.platform.startswith("win"):
         return None, None
     from xpra.log import Logger
-    log = Logger("net")
+    log = Logger("network")
     #print("creating server socket %s" % sockpath)
     try:
         display_name = dotxpra.normalize_local_display_name(display_name)
@@ -434,7 +439,7 @@ def start_pulseaudio(child_reaper, pulseaudio_command):
     log = Logger("sound")
     log("pulseaudio_command=%s", pulseaudio_command)
     pa_proc = subprocess.Popen(pulseaudio_command, stdin=subprocess.PIPE, shell=True, close_fds=True)
-    child_reaper.add_process(pa_proc, "pulseaudio")
+    child_reaper.add_process(pa_proc, "pulseaudio", ignore=True)
     log.info("pulseaudio server started with pid %s", pa_proc.pid)
     def check_pa_start():
         if pa_proc.poll() is not None or pa_proc.pid in child_reaper._dead_pids:
@@ -469,14 +474,14 @@ def start_Xvfb(xvfb_str, display_name):
     for var,value in subs.items():
         xvfb_str = xvfb_str.replace("$%s" % var, value)
         xvfb_str = xvfb_str.replace("${%s}" % var, value)
-    xvfb_cmd = xvfb_str.split()
+    xvfb_cmd = xvfb_str.split()+[display_name]
     xvfb_executable = xvfb_cmd[0]
     xvfb_cmd[0] = "%s-for-Xpra-%s" % (xvfb_executable, display_name)
     def setsid():
         #run in a new session
         if os.name=="posix":
             os.setsid()
-    xvfb = subprocess.Popen(xvfb_cmd+[display_name], executable=xvfb_executable, close_fds=True,
+    xvfb = subprocess.Popen(xvfb_cmd, executable=xvfb_executable, close_fds=True,
                                 stdin=subprocess.PIPE, preexec_fn=setsid)
     from xpra.os_util import get_hex_uuid
     xauth_cmd = ["xauth", "add", display_name, "MIT-MAGIC-COOKIE-1", get_hex_uuid()]
@@ -518,6 +523,7 @@ def verify_display_ready(xvfb, display_name, shadowing):
     assert "gtk" not in sys.modules
     import gtk.gdk          #@Reimport
     try:
+        import glib
         glib.threads_init()
     except:
         #old versions do not have this method
@@ -538,6 +544,25 @@ def verify_display_ready(xvfb, display_name, shadowing):
         return  None
     return display
 
+def find_lib(libname):
+    #it would be better to rely on dlopen to find the paths
+    #but I cannot find a way of getting ctypes to tell us the path
+    #it found the library in
+    libpaths = os.environ.get("LD_LIBRARY_PATH", "").split(":")
+    libpaths.append("/usr/lib64")
+    libpaths.append("/usr/lib")
+    for libpath in libpaths:
+        if not libpath or not os.path.exists(libpath):
+            continue
+        libname_so = os.path.join(libpath, libname)
+        if os.path.exists(libname_so):
+            return libname_so
+    return None
+
+def find_fakeXinerama():
+    return find_lib("libfakeXinerama.so.1")
+
+
 def start_children(child_reaper, commands, fake_xinerama):
     assert os.name=="posix"
     from xpra.log import Logger
@@ -545,19 +570,9 @@ def start_children(child_reaper, commands, fake_xinerama):
     env = os.environ.copy()
     #add fake xinerama:
     if fake_xinerama:
-        #locate the fakeXinerama lib:
-        #it would be better to rely on dlopen to find the paths
-        #but I cannot find a way of getting ctypes to tell us the path
-        #it found the library in
-        libpaths = os.environ.get("LD_LIBRARY_PATH", "").split(":")
-        libpaths.append("/usr/lib64")
-        libpaths.append("/usr/lib")
-        for libpath in libpaths:
-            if not libpath or not os.path.exists(libpath):
-                continue
-            libfakeXinerama_so = "%s/%s" % (libpath, "libfakeXinerama.so.1")
-            if os.path.exists(libfakeXinerama_so):
-                env["LD_PRELOAD"] = libfakeXinerama_so
+        libfakeXinerama_so = find_fakeXinerama()
+        if libfakeXinerama_so:
+            env["LD_PRELOAD"] = libfakeXinerama_so
     #disable ubuntu's global menu using env vars:
     env.update({
         "UBUNTU_MENUPROXY"          : "",
@@ -575,9 +590,30 @@ def start_children(child_reaper, commands, fake_xinerama):
 
 def run_server(parser, opts, mode, xpra_file, extra_args):
     if opts.encoding and opts.encoding=="help":
-        from xpra.codecs.loader import encodings_help
+        #avoid errors and warnings:
+        opts.encoding = ""
+        opts.clipboard = False
+        opts.notifications = False
+        print("xpra server supports the following encodings:")
+        print("(please wait, encoder initialization may take a few seconds)")
+        #disable info logging which would be confusing here
+        from xpra.log import get_all_loggers, set_default_level
+        import logging
+        set_default_level(logging.WARN)
+        logging.root.setLevel(logging.WARN)
+        for x in get_all_loggers():
+            x.logger.setLevel(logging.WARN)
         from xpra.server.server_base import ServerBase
-        print("xpra server supports the following encodings:\n * %s" % ("\n * ".join(encodings_help(ServerBase().encodings))))
+        sb = ServerBase()
+        sb.init(opts)
+        #ensures that the threaded video helper init has completed
+        #(by running it again, which will block on the init lock)
+        from xpra.codecs.video_helper import getVideoHelper
+        getVideoHelper().init()
+        sb.init_encodings()
+        from xpra.codecs.loader import encoding_help
+        for e in sb.encodings:
+            print(" * %s" % encoding_help(e))
         return 0
 
     assert mode in ("start", "upgrade", "shadow", "proxy")
@@ -599,7 +635,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     if not shadowing and not proxying:
         display_name_check(display_name)
 
-    if not shadowing and not proxying and opts.exit_with_children and not opts.start_child:
+    if not shadowing and not proxying and not upgrading and opts.exit_with_children and not opts.start_child:
         sys.stderr.write("--exit-with-children specified without any children to spawn; exiting immediately")
         return  1
 
@@ -616,6 +652,10 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
 
     # Daemonize:
     if opts.daemon:
+        #daemonize will chdir to "/", so try to use an absolute path:
+        if opts.password_file:
+            opts.password_file = os.path.abspath(opts.password_file)
+
         logfd = open_log_file(dotxpra, opts.log_file, display_name)
         assert logfd > 2
         daemonize(logfd)
@@ -746,7 +786,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     if xvfb_pid is not None and not opts.use_display and not shadowing:
         _cleanups.append(kill_xvfb)
 
-    if os.name=="posix" and not proxying:
+    if os.name=="posix" and not proxying and not upgrading and not shadowing:
         def reaper_quit():
             if opts.exit_with_children:
                 log.info("all children have exited and --exit-with-children was specified, exiting")
