@@ -104,7 +104,7 @@ def make_window_metadata(window, propname, generic_window_types=False, client_su
         return {"window-type" : wts}
     elif propname in ("has-alpha", "override-redirect", "tray", "modal", "fullscreen", "maximized"):
         return {propname : window.get_property(propname)}
-    elif propname in ("role"):
+    elif propname in ("role", "opacity"):
         v = window.get_property(propname)
         if v is None:
             return {}
@@ -351,9 +351,6 @@ class ServerSource(object):
             self.mmap = None
             self.mmap_size = 0
         self.stop_sending_sound()
-        if self.protocol:
-            self.protocol.close()
-            self.protocol = None
 
 
     def recalculate_delays(self):
@@ -527,6 +524,11 @@ class ServerSource(object):
         self.supports_transparency = HAS_ALPHA and c.boolget("encoding.transparency")
 
         self.desktop_size = c.intpair("desktop_size")
+        if self.desktop_size is not None:
+            w, h = self.desktop_size
+            if w<=0 or h<=0 or w>=32768 or h>=32768:
+                log.warn("ignoring invalid desktop dimensions: %sx%s", w, h)
+                self.desktop_size = None
         self.set_screen_sizes(c.listget("screen_sizes"))
 
         #sound stuff:
@@ -555,6 +557,7 @@ class ServerSource(object):
             proxy_platform = c.strget("proxy.platform")
             proxy_release = c.strget("proxy.platform.release")
             proxy_version = c.strget("proxy.version")
+            proxy_version = c.strget("proxy.build.version", proxy_version)
             msg = "via %s proxy version %s" % (platform_name(proxy_platform, proxy_release), std(proxy_version))
             if proxy_hostname:
                 msg += " on '%s'" % std(proxy_hostname)
@@ -564,18 +567,8 @@ class ServerSource(object):
             if msg:
                 log.warn("Warning: proxy version may not be compatible: %s", msg)
 
-        #keyboard:
-        try:
-            from xpra.x11.server_keyboard_config import KeyboardConfig
-            self.keyboard_config = KeyboardConfig()
-            self.keyboard_config.enabled = self.send_windows and c.boolget("keyboard", True)
-            self.assign_keymap_options(c)
-            self.keyboard_config.xkbmap_layout = c.strget("xkbmap_layout")
-            self.keyboard_config.xkbmap_variant = c.strget("xkbmap_variant")
-            keylog("keyboard_config=%s", self.keyboard_config)
-        except ImportError, e:
-            keylog.error("failed to load keyboard support: %s", e)
-            self.keyboard_config = None
+        #keyboard is now injected into this class, default to undefined:
+        self.keyboard_config = None
 
         #encodings:
         def getenclist(k, default_value=[]):
@@ -865,21 +858,6 @@ class ServerSource(object):
             return True
         return False
 
-    def assign_keymap_options(self, props):
-        """ used by both process_hello and process_keymap
-            to set the keyboard attributes """
-        modded = []
-        for x in ["xkbmap_print", "xkbmap_query", "xkbmap_mod_meanings",
-                  "xkbmap_mod_managed", "xkbmap_mod_pointermissing",
-                  "xkbmap_keycodes", "xkbmap_x11_keycodes"]:
-            cv = getattr(self.keyboard_config, x)
-            nv = props.get(x)
-            if cv!=nv:
-                setattr(self.keyboard_config, x, nv)
-                modded.append(x)
-        keylog("assign_keymap_options(..) modified %s", modded)
-        return len(modded)>0
-
     def keys_changed(self):
         if self.keyboard_config:
             self.keyboard_config.compute_modifier_map()
@@ -890,6 +868,13 @@ class ServerSource(object):
         if self.keyboard_config and self.keyboard_config.enabled:
             self.keyboard_config.make_keymask_match(modifier_list, ignored_modifier_keycode, ignored_modifier_keynames)
 
+    def set_default_keymap(self):
+        keylog("set_default_keymap() keyboard_config=%s", self.keyboard_config)
+        if self.keyboard_config:
+            self.keyboard_config.set_default_keymap()
+        return self.keyboard_config
+
+
     def set_keymap(self, current_keyboard_config, keys_pressed, force):
         keylog("set_keymap%s", (current_keyboard_config, keys_pressed, force))
         if self.keyboard_config and self.keyboard_config.enabled:
@@ -900,26 +885,19 @@ class ServerSource(object):
             keylog("current keyboard id=%s, new keyboard id=%s", current_id, keymap_id)
             if force or current_id is None or keymap_id!=current_id:
                 self.keyboard_config.keys_pressed = keys_pressed
-                self.keyboard_config.set_keymap(self.client_platform)
+                self.keyboard_config.set_keymap()
                 current_keyboard_config = self.keyboard_config
             else:
                 keylog.info("keyboard mapping already configured (skipped)")
                 self.keyboard_config = current_keyboard_config
         return current_keyboard_config
 
+
     def get_keycode(self, client_keycode, keyname, modifiers):
-        if self.keyboard_config is None or not self.keyboard_config.enabled:
-            keylog.info("ignoring keycode since keyboard is turned off")
+        if self.keyboard_config is None:
+            keylog.info("ignoring client key %s / %s since keyboard is not configured", client_keycode, keyname)
             return -1
-        server_keycode = self.keyboard_config.keycode_translation.get((client_keycode, keyname))
-        if server_keycode is None:
-            if self.keyboard_config.is_native_keymap:
-                #native: assume no translation for this key
-                server_keycode = client_keycode
-            else:
-                #non-native: try harder to find matching keysym
-                server_keycode = self.keyboard_config.keycode_translation.get(keyname, client_keycode)
-        return server_keycode
+        return self.keyboard_config.get_keycode(client_keycode, keyname, modifiers)
 
 
 #
@@ -1180,7 +1158,7 @@ class ServerSource(object):
         self.cursor_sizes = sizes
         if not self.send_cursor_pending:
             self.send_cursor_pending = True
-            delay = max(10, int(self.global_batch_config.delay*4))
+            delay = max(10, int(self.global_batch_config.delay/4))
             self.timeout_add(delay, self.do_send_cursor)
 
     def do_send_cursor(self):
@@ -1539,13 +1517,13 @@ class ServerSource(object):
 #
 # Methods used by WindowSource:
 #
-    def queue_damage(self, encode_and_send_cb):
+    def queue_damage(self, *fn_and_args):
         """
             This is used by WindowSource to queue damage processing to be done in the 'data_to_packet' thread.
             The 'encode_and_send_cb' will then add the resulting packet to the 'damage_packet_queue' via 'queue_packet'.
         """
         self.statistics.damage_data_qsizes.append((time.time(), self.damage_data_queue.qsize()))
-        self.damage_data_queue.put(encode_and_send_cb)
+        self.damage_data_queue.put(fn_and_args)
 
     def queue_packet(self, packet, wid, pixels, start_send_cb, end_send_cb):
         """
@@ -1571,11 +1549,11 @@ class ServerSource(object):
             to ensure all the queued items get called.
         """
         while True:
-            encode_and_queue = self.damage_data_queue.get(True)
-            if encode_and_queue is None:
+            fn_and_args = self.damage_data_queue.get(True)
+            if fn_and_args is None:
                 return              #empty marker
             try:
-                encode_and_queue()
+                fn_and_args[0](*fn_and_args[1:])
             except Exception, e:
                 log.error("error processing damage data: %s", e, exc_info=True)
             NOYIELD or time.sleep(0)
