@@ -7,19 +7,20 @@
 import os
 import time
 import math
+import operator
 from threading import Lock
 
 from xpra.util import AtomicInteger
 from xpra.net.protocol import Compressed
 from xpra.codecs.codec_constants import get_avutil_enum_from_colorspace, get_subsampling_divs, get_default_csc_modes, \
-                                        TransientCodecException, RGB_FORMATS, PIXEL_SUBSAMPLING
-from xpra.server.window_source import WindowSource, AUTO_SWITCH_TO_RGB, MAX_PIXELS_PREFER_RGB
+                                        TransientCodecException, RGB_FORMATS, PIXEL_SUBSAMPLING, LOSSY_PIXEL_FORMATS
+from xpra.server.window_source import WindowSource, MAX_PIXELS_PREFER_RGB, STRICT_MODE
 from xpra.gtk_common.region import rectangle, merge_all
 from xpra.codecs.loader import PREFERED_ENCODING_ORDER
 from xpra.log import Logger
 
 log = Logger("video", "encoding")
-scorelog = Logger("stats")
+scorelog = Logger("score")
 sublog = Logger("subregion")
 
 
@@ -78,8 +79,8 @@ class WindowVideoSource(WindowSource):
                 self._encoders[x] = self.video_encode
         #these are used for non-video areas, ensure "jpeg" is used if available
         #as we may be dealing with large areas still, and we want speed:
-        common = [x for x in self.server_core_encodings if x in self.core_encodings]
-        self.non_video_encodings = [x for x in PREFERED_ENCODING_ORDER if (x in common and x not in self.video_encodings)]
+        nv_common = (set(self.server_core_encodings) & set(self.core_encodings)) - set(self.video_encodings)
+        self.non_video_encodings = [x for x in PREFERED_ENCODING_ORDER if x in nv_common]
 
         self._csc_encoder = None
         self._video_encoder = None
@@ -221,8 +222,8 @@ class WindowVideoSource(WindowSource):
         self.uses_swscale = properties.get("encoding.uses_swscale", self.uses_swscale)
         WindowSource.set_client_properties(self, properties)
         #encodings may have changed, so redo this:
-        common = [x for x in self.server_core_encodings if x in self.core_encodings]
-        self.non_video_encodings = [x for x in PREFERED_ENCODING_ORDER if (x in common and x not in self.video_encodings)]
+        nv_common = (set(self.server_core_encodings) & set(self.core_encodings)) - set(self.video_encodings)
+        self.non_video_encodings = [x for x in PREFERED_ENCODING_ORDER if x in nv_common]
         log("set_client_properties(%s) csc_modes=%s, full_csc_modes=%s, video_scaling=%s, video_subregion=%s, uses_swscale=%s, non_video_encodings=%s", properties, self.csc_modes, self.full_csc_modes, self.supports_video_scaling, self.supports_video_subregion, self.uses_swscale, self.non_video_encodings)
 
     def unmap(self):
@@ -251,7 +252,7 @@ class WindowVideoSource(WindowSource):
             sublog("identify video: not using a video mode! (%s)", self.encoding)
             self.video_subregion = None
             return
-        if self.full_frames_only:
+        if self.full_frames_only or STRICT_MODE:
             sublog("identify video: full frames only!")
             self.video_subregion = None
             return
@@ -410,45 +411,28 @@ class WindowVideoSource(WindowSource):
         self.video_subregion = None
 
 
-    def send_delayed_regions(self, damage_time, window, regions, coding, options):
+    def do_send_delayed_regions(self, damage_time, window, regions, coding, options):
         """
             Overriden here so we can try to intercept the video_subregion if one exists.
         """
         #overrides the default method for finding the encoding of a region
         #so we can ensure we don't use the video encoder when we don't want to:
-        def get_non_video_encoding(batching, window, pixel_count, ww, wh, current_encoding):
-            def get_fallback():
-                if len(self.non_video_encodings)==0:
-                    return None
-                return self.non_video_encodings[0]
-            if window.has_alpha() and self.supports_transparency:
-                return self.get_transparent_encoding(current_encoding) or get_fallback()
-            if window.is_tray():
-                return self.find_common_lossless_encoder(window.has_alpha(), coding, ww*wh) or get_fallback()
-            if AUTO_SWITCH_TO_RGB and pixel_count<MAX_PIXELS_PREFER_RGB:
-                return self.pick_encoding(["rgb24"]) or get_fallback()
-            if pixel_count>=256*256:
-                #jpeg for large areas, we're lossy already since we use video
-                return self.pick_encoding(["jpeg"]) or get_fallback()
-            return get_fallback()
 
-        if self.is_cancelled():
-            return
+        def nonvideo(regions=regions, encoding=coding, exclude_region=None):
+            WindowSource.do_send_delayed_regions(self, damage_time, window, regions, encoding, options, exclude_region=exclude_region, fallback=self.non_video_encodings)
 
-        if window.is_tray():
-            sublog("tray - don't use video region!")
-            WindowSource.send_delayed_regions(self, damage_time, window, regions, None, options, get_region_encoding=get_non_video_encoding)
-            return
+        if self.is_tray:
+            sublog("BUG? video for tray - don't use video region!")
+            return nonvideo(encoding=None)
 
-        if self.encoding not in self.video_encodings:
+        if coding not in self.video_encodings:
             sublog("not a video encoding")
-            WindowSource.send_delayed_regions(self, damage_time, window, regions, coding, options)
-            return
+            return nonvideo()
 
         vr = self.video_subregion
         if vr is None:
             sublog("no video region, we may use the video encoder for something else")
-            WindowSource.send_delayed_regions(self, damage_time, window, regions, coding, options)
+            WindowSource.do_send_delayed_regions(self, damage_time, window, regions, coding, options)
             return
         assert not self.full_frames_only
 
@@ -494,8 +478,7 @@ class WindowVideoSource(WindowSource):
 
         if actual_vr is None:
             sublog("send_delayed_regions: video region %s not found in: %s (using non video encoding)", vr, regions)
-            WindowSource.send_delayed_regions(self, damage_time, window, regions, None, options, get_region_encoding=get_non_video_encoding)
-            return
+            return nonvideo(encoding=None)
 
         #found the video region:
         #send this straight away using the video encoder:
@@ -526,8 +509,7 @@ class WindowVideoSource(WindowSource):
                 delay = self.video_subregion_non_max_wait-elapsed
                 self.expire_timer = self.timeout_add(int(delay), self.expire_delayed_region, delay)
                 return
-
-        WindowSource.send_delayed_regions(self, damage_time, window, trimmed, None, options, exclude_region=actual_vr, get_region_encoding=get_non_video_encoding)
+        nonvideo(regions=trimmed, encoding=None, exclude_region=actual_vr)
 
 
     def process_damage_region(self, damage_time, window, x, y, w, h, coding, options):
@@ -536,84 +518,87 @@ class WindowVideoSource(WindowSource):
         dw = w - (w & self.width_mask)
         dh = h - (h & self.height_mask)
         if coding in self.video_encodings and (dw>0 or dh>0):
+            #no point in using get_best_encoding here, rgb24 wins
+            #(as long as the mask is small - and it is)
             if dw>0:
-                lossless = self.find_common_lossless_encoder(window.has_alpha(), coding, dw*h)
-                WindowSource.process_damage_region(self, damage_time, window, x+w-dw, y, dw, h, lossless, options)
+                WindowSource.process_damage_region(self, damage_time, window, x+w-dw, y, dw, h, "rgb24", options)
             if dh>0:
-                lossless = self.find_common_lossless_encoder(window.has_alpha(), coding, w*dh)
-                WindowSource.process_damage_region(self, damage_time, window, x, y+h-dh, x+w, dh, lossless, options)
+                WindowSource.process_damage_region(self, damage_time, window, x, y+h-dh, x+w, dh, "rgb24", options)
 
 
     def must_encode_full_frame(self, window, encoding):
         return WindowSource.must_encode_full_frame(self, window, encoding) or (encoding in self.video_encodings)
 
-    def do_get_best_encoding(self, batching, has_alpha, is_tray, is_OR, pixel_count, ww, wh, current_encoding):
+
+    def get_encoding_options(self, batching, pixel_count, ww, wh, speed, quality, current_encoding):
         """
-            decide whether we send a full window update
-            using the video encoder or if a small lossless region(s) is a better choice
+            decide whether we send a full window update using the video encoder,
+            or if a separate small region(s) is a better choice
         """
-        encoding = WindowSource.do_get_best_encoding(self, batching, has_alpha, is_tray, is_OR, pixel_count, ww, wh, current_encoding)
-        if encoding is not None:
-            #superclass knows best (usually a tray or transparent window):
-            return encoding
+        def nonvideo(s=speed, q=quality):
+            s = max(0, min(100, s))
+            q = max(0, min(100, q))
+            return WindowSource.get_encoding_options(self, batching, pixel_count, ww, wh, s, q, current_encoding)
+
         if current_encoding not in self.video_encodings:
-            return None
+            #not doing video, bail out:
+            return nonvideo()
+
+        if ww*wh<=MAX_NONVIDEO_PIXELS:
+            #window is too small!
+            return nonvideo()
+
         if ww<self.min_w or ww>self.max_w or wh<self.min_h or wh>self.max_h:
             #video encoder cannot handle this size!
             #(maybe this should be an 'assert' statement here?)
-            return None
-        if self.video_subregion and (self.video_subregion.width!=ww or self.video_subregion.height!=wh):
-            #we have a video region, and this is not it, so don't use video:
-            return None
-
-        def switch_to_fast():
-            return self.pick_encoding(["jpeg", "rgb24"])
+            return nonvideo()
 
         if time.time()-self.statistics.last_resized<0.150:
-            #window has just been resized
-            log("resized not long ago")
-            return switch_to_fast()
+            #window has just been resized, may still resize
+            return nonvideo(q=quality-30)
 
-        def switch_to_lossless(reason):
-            coding = self.find_common_lossless_encoder(has_alpha, current_encoding, pixel_count)
-            log("do_get_best_encoding(..) temporarily switching to %s encoder for %s pixels: %s", coding, pixel_count, reason)
-            return  coding
+        if self.get_current_quality()!=quality or self.get_current_speed()!=speed:
+            #quality or speed override, best not to force video encoder re-init
+            return nonvideo()
+
+        def lossless(reason):
+            log("get_encoding_options(..) temporarily switching to lossless mode for %8i pixels: %s", pixel_count, reason)
+            return nonvideo(q=100)
+
+        #if speed is high, assume we have bandwidth to spare
+        smult = max(1, (speed-75)/5.0)
+        if pixel_count<=MAX_PIXELS_PREFER_RGB * smult:
+            return lossless("low pixel count")
+
+        if self.video_subregion and (self.video_subregion.width!=ww or self.video_subregion.height!=wh):
+            #we have a video region, and this is not it, so don't use video
+            #raise the quality as the areas around video tend to not be graphics
+            return nonvideo(q=quality+30)
 
         #calculate the threshold for using video vs small regions:
-        max_nvp = MAX_NONVIDEO_PIXELS
-        if pixel_count<=max_nvp:
-            #small window!
-            return switch_to_lossless("small window: %sx%s" % (ww, wh))
-
-        s = self.get_current_speed()
-        if s>75:
-            #if speed is high, assume we have bandwidth to spare
-            #and prefer non-video:
-            max_nvp = int(max_nvp * (1.0+(s-75.0)/5.0))
-        if is_OR:
-            #OR windows tend to be static:
-            max_nvp *= 4
-        if self._sequence<=5:
-            #discount the first frames, the window may be temporary:
-            max_nvp *= 10-self._sequence
-        if not batching:
-            #if we're not batching, allow more pixels:
-            max_nvp *= 4
-        if self._video_encoder:
-            #if we have a video encoder already, make it more likely we'll use it:
-            max_nvp /= 2
-
+        factors = (smult,                                       #speed multiplier
+                   1 + int(self.is_OR)*2,                       #OR windows tend to be static
+                   max(1, 10-self._sequence),                   #gradual discount the first 9 frames, as the window may be temporary
+                   1 + int(not batching)*2,                     #if we're not batching, allow more pixels
+                   1.0 / (int(bool(self._video_encoder)) + 1)   #if we have a video encoder already, make it more likely we'll use it:
+                   )
+        max_nvp = int(reduce(operator.mul, factors, MAX_NONVIDEO_PIXELS))
         if pixel_count<=max_nvp:
             #below threshold
-            return switch_to_lossless("frame number %s: %s pixels (threshold=%s)" % (self._sequence, pixel_count, max_nvp))
+            return nonvideo()
 
         #ensure the dimensions we use for decision making are the ones actually used:
         ww = ww & self.width_mask
         wh = wh & self.height_mask
         if ww<self.min_w or ww>self.max_w or wh<self.min_h or wh>self.max_h:
             #failsafe:
-            return switch_to_lossless("window dimensions are unsuitable for this encoder/csc")
-        return self.get_core_encoding(has_alpha, current_encoding)
+            return nonvideo()
+        return [current_encoding]
+
+    def do_get_best_encoding(self, options, current_encoding, fallback):
+        #video encodings: always pick from the ordered list of options
+        #rather than sticking with the current encoding:
+        return self.pick_encoding(options, fallback)
 
 
     def reconfigure(self, force_reload=False):
@@ -649,8 +634,7 @@ class WindowVideoSource(WindowSource):
                 pixel_format = self._csc_encoder.get_src_format()
             else:
                 pixel_format = ve.get_src_format()
-            width = ve.get_width()
-            height = ve.get_height()
+            width, height = self.window_dimensions
             quality = self.get_current_quality()
             speed = self.get_current_speed()
 
@@ -706,19 +690,15 @@ class WindowVideoSource(WindowSource):
         def add_scores(info, csc_spec, enc_in_format):
             #find encoders that take 'enc_in_format' as input:
             colorspace_specs = encoder_specs.get(enc_in_format)
-            scorelog("add_scores(%s, %s, %s) colorspace_specs=%s", info, csc_spec, enc_in_format, colorspace_specs)
             if not colorspace_specs:
                 return
             #log("%s encoding from %s: %s", info, pixel_format, colorspace_specs)
             for encoder_spec in colorspace_specs:
                 #ensure that the output of the encoder can be processed by the client:
-                matches = [x for x in encoder_spec.output_colorspaces if x in supported_csc_modes]
+                matches = set(encoder_spec.output_colorspaces) & set(supported_csc_modes)
                 if len(matches)==0:
                     continue
-                score = self.get_score(enc_in_format,
-                                       csc_spec, encoder_spec,
-                                       width, height)
-                scorelog("add_scores: score(%s)=%s", (enc_in_format, csc_spec, encoder_spec, width, height), score)
+                score = self.get_score(enc_in_format, csc_spec, encoder_spec, width, height)
                 if score>=0:
                     item = score, csc_spec, enc_in_format, encoder_spec
                     scores.append(item)
@@ -777,8 +757,9 @@ class WindowVideoSource(WindowSource):
         #score based on speed:
         speed = encoder_spec.speed
         if csc_spec:
+            #average and add 0.25 for the extra cost
             speed += csc_spec.speed
-            speed /= 2.0
+            speed /= 2.25
         #the lower the current speed
         #the more we need a fast encoder/csc to cancel it out:
         sscore = max(0, (100.0-self.get_current_speed()) * speed/100.0)
@@ -811,6 +792,8 @@ class WindowVideoSource(WindowSource):
             #client does not support video decoder reinit,
             #so we cannot swap for another encoder of the same type
             #(which would generate a new stream)
+            scorelog("encoding (%s vs %s) or type (%s vs %s) mismatch, without support for reinit",
+                     self._video_encoder.get_encoding(), encoder_spec.encoding, self._video_encoder.get_type(), encoder_spec.codec_type)
             return -1
         def clamp(v):
             return max(0, min(100, v))
@@ -828,7 +811,12 @@ class WindowVideoSource(WindowSource):
             height_mask = csc_spec.height_mask & encoder_spec.height_mask
             csc_width = width & width_mask
             csc_height = height & height_mask
-            if self._csc_encoder is None or self._csc_encoder.get_dst_format()!=csc_format or \
+            if csc_format=="RGB":
+                #converting to "RGB" is often a waste of CPU
+                #(can only get selected because the csc step will do scaling,
+                # but even then, the YUV subsampling are better options)
+                ecsc_score = 1
+            elif self._csc_encoder is None or self._csc_encoder.get_dst_format()!=csc_format or \
                type(self._csc_encoder)!=csc_spec.codec_class or \
                self._csc_encoder.get_src_width()!=csc_width or self._csc_encoder.get_src_height()!=csc_height:
                 #if we have to change csc, account for new csc setup cost:
@@ -843,6 +831,10 @@ class WindowVideoSource(WindowSource):
                 #csc cannot take care of scaling, so encoder will have to:
                 encoder_scaling = scaling
                 scaling = (1, 1)
+            if scaling!=(1, 1):
+                #if we are (down)scaling, we should prefer lossy pixel formats:
+                v = LOSSY_PIXEL_FORMATS.get(csc_format, 1)
+                qscore *= (v/2)
             enc_width, enc_height = self.get_encoder_dimensions(csc_spec, encoder_spec, csc_width, csc_height, scaling)
         else:
             #not using csc at all!
@@ -855,6 +847,7 @@ class WindowVideoSource(WindowSource):
 
         if encoder_scaling!=(1,1) and not encoder_spec.can_scale:
             #we need the encoder to scale but it cannot do it, fail it:
+            scorelog("scaling not supported (%s)", encoder_scaling)
             return -1
 
         ee_score = 100
@@ -867,8 +860,9 @@ class WindowVideoSource(WindowSource):
         #edge resistance score: average of csc and encoder score:
         er_score = (ecsc_score + ee_score) / 2.0
         score = int((qscore+sscore+er_score)*runtime_score/100.0/3.0)
-        scorelog("get_score%s quality:%.1f, speed:%.1f, setup:%.1f runtime:%.1f score=%s", (csc_format, csc_spec, encoder_spec,
-                  width, height), qscore, sscore, er_score, runtime_score, score)
+        scorelog("get_score(%-7s, %-24r, %-24r, %5i, %5i) quality: %2i, speed: %2i, setup: %2i runtime: %2i scaling: %s / %s, score=%2i",
+                 csc_format, csc_spec, encoder_spec, width, height,
+                 qscore, sscore, er_score, runtime_score, scaling, encoder_scaling, score)
         return score
 
     def get_encoder_dimensions(self, csc_spec, encoder_spec, width, height, scaling=(1,1)):
@@ -896,33 +890,40 @@ class WindowVideoSource(WindowSource):
         elif SCALING_HARDCODED:
             actual_scaling = tuple(SCALING_HARDCODED)
             log("using hardcoded scaling: %s", actual_scaling)
-        elif actual_scaling is None:
+        elif actual_scaling is None and self.statistics.damage_events_count>50 and (time.time()-self.statistics.last_resized)>0.5:
             #no scaling window attribute defined, so use heuristics to enable:
             q = self.get_current_quality()
             s = self.get_current_speed()
-            qs = s>q and q<80
-            #full frames per second:
+            #full frames per second (measured in pixels vs window size):
             ffps = 0
-            lde = list(self.statistics.last_damage_events)
+            stime = time.time()-5           #only look at the last 5 seconds max
+            lde = [x for x in list(self.statistics.last_damage_events) if x[0]>stime]
             if len(lde)>10:
                 #the first event's first element is the oldest event time:
                 otime = lde[0][0]
                 pixels = sum(w*h for _,_,_,w,h in lde)
                 ffps = int(pixels/(width*height)/(time.time() - otime))
 
+            #edge resistance:
+            er = 0
+            if self.actual_scaling!=(1, 1):
+                #if we are currently downscaling, stick with it a bit longer:
+                er = 1
+            qs = s>(q-er*10) and q<(70+er*15)
+
             if width>max_w or height>max_h:
                 #most encoders can't deal with that!
                 d = 2
                 while width/d>max_w or height/d>max_h:
                     d += 1
                 actual_scaling = 1,d
-            elif self.fullscreen and (qs or ffps>=10):
+            elif self.fullscreen and (qs or ffps>=(10-er*3)):
                 actual_scaling = 1,3
-            elif self.maximized and (qs or ffps>=10):
+            elif self.maximized and (qs or ffps>=(10-er*3)):
                 actual_scaling = 1,2
-            elif width*height>=2048*1200 and (q<80 or ffps>=25):
+            elif width*height>=(2048-er*768)*1200 and (qs or ffps>=(25-er*5)):
                 actual_scaling = 1,3
-            elif width*height>=1024*1024 and (q<80 or ffps>=30):
+            elif width*height>=(1024-er*384)*1024 and (qs or ffps>=(30-er*10)):
                 actual_scaling = 2,3
         if actual_scaling is None:
             actual_scaling = 1, 1
@@ -1112,7 +1113,7 @@ class WindowVideoSource(WindowSource):
             self._lock.acquire()
             if not self.check_pipeline(encoding, w, h, src_format):
                 #find one that is not video:
-                fallback_encodings = [x for x in self._encoders.keys() if (x not in self.video_encodings and x!="mmap")]
+                fallback_encodings = set(self._encoders.keys()) - set(self.video_encodings) - set(["mmap"])
                 log.error("BUG: failed to setup a video pipeline for %s encoding with source format %s, will fallback to: %s", encoding, src_format, fallback_encodings)
                 assert len(fallback_encodings)>0
                 fallback_encoding = [x for x in PREFERED_ENCODING_ORDER if x in fallback_encodings][0]
