@@ -69,7 +69,7 @@ class AtomicInteger(object):
 
 class typedict(dict):
 
-    def capsget(self, key, default):
+    def capsget(self, key, default=None):
         v = self.get(strtobytes(key), default)
         if sys.version >= '3' and type(v)==bytes:
             v = bytestostr(v)
@@ -114,21 +114,26 @@ class typedict(dict):
         if v is None:
             return default_value
         assert type(v) in (list, tuple), "expected a list or tuple value for %s but got %s" % (k, type(v))
+        aslist = list(v)
         if item_type:
-            for x in v:
+            for i in range(len(aslist)):
+                x = aslist[i]
+                if sys.version > '3' and type(x)==bytes and item_type==str:
+                    x = bytestostr(x)
+                    aslist[i] = x
                 assert type(x)==item_type, "invalid item type for %s %s: expected %s but got %s" % (type(v), k, item_type, type(x))
         if max_items is not None:
             assert len(v)<=max_items, "too many items in %s %s: maximum %s allowed, but got %s" % (type(v), k, max_items, len(v))
-        return v
+        return aslist
 
 
 def log_screen_sizes(root_w, root_h, sizes):
     try:
         do_log_screen_sizes(root_w, root_h, sizes)
-    except Exception, e:
+    except Exception:
         from xpra.log import Logger
         log = Logger("util")
-        log.warn("failed to parse screen size information: %s", e)
+        log.warn("failed to parse screen size information: %s", sys.exc_info()[1])
 
 def prettify_plug_name(s, default=""):
     if not s:
@@ -179,41 +184,113 @@ def do_log_screen_sizes(root_w, root_h, sizes):
             log.info("    "+" ".join(info))
 
 
-def dump_references(log, instances):
+def dump_references(log, instances, exclude=[]):
     import gc
     import inspect
     gc.collect()
-    cf = inspect.currentframe()
-    for instance in instances:
-        referrers = [x for x in gc.get_referrers(instance) if x not in (instances, cf)]
-        log.info("referrers for %s: %s", instance, len(referrers))
-        for i in range(len(referrers)):
-            r = referrers[i]
-            log.info("[%s] in %s", i, type(r))
-            if inspect.isframe(r):
-                log.info("  frame info: %s", inspect.getframeinfo(r))
-            elif type(r)==list:
-                listref = gc.get_referrers(r)
-                log.info("  list: %s referrers: %s", r, listref)
-            elif type(r)==dict:
-                for k,v in r.items():
-                    if k is instance:
-                        log.info("  key with value=%s", v)
-                    elif v is instance:
-                        log.info("  for key=%s", k)
-            else:
-                log.info("     %s : %s", type(r), r)
+    frame = inspect.currentframe()
+    try:
+        exclude.append(instances)
+        exclude.append([frame])
+        for instance in instances:
+            referrers = [x for x in gc.get_referrers(instance) if (x not in exclude and len([y for y in exclude if x in y])==0)]
+            log.info("referrers for %s: %s", instance, len(referrers))
+            for i in range(len(referrers)):
+                r = referrers[i]
+                log.info("[%s] in %s", i, type(r))
+                if inspect.isframe(r):
+                    log.info("  frame info: %s", str(inspect.getframeinfo(r))[:1024])
+                elif type(r)==list:
+                    listref = gc.get_referrers(r)
+                    log.info("  list: %s..  %s referrers: %s", str(r[:32])[:1024], len(listref), str(listref[:32])[:1024])
+                elif type(r)==dict:
+                    if len(r)>64:
+                        log.info("  %s items: %s", len(r), str(r)[:1024])
+                        continue
+                    for k,v in r.items():
+                        if k is instance:
+                            log.info("  key with value=%s", v)
+                        elif v is instance:
+                            log.info("  for key=%s", k)
+                else:
+                    log.info("     %s : %s", type(r), r)
+    finally:
+        del frame
 
+def detect_leaks(log, detailed=[]):
+    import types
+    from collections import defaultdict
+    import gc
+    import inspect
+    global before, after
+    gc.enable()
+    gc.set_debug(gc.DEBUG_LEAK)
+    before = defaultdict(int)
+    after = defaultdict(int)
+    gc.collect()
+    detailed = []
+    ignore = (defaultdict, types.BuiltinFunctionType, types.BuiltinMethodType, types.FunctionType, types.MethodType)
+    for i in gc.get_objects():
+        if type(i) not in ignore:
+            before[type(i)] += 1
+
+    def print_leaks():
+        global before, after
+        gc.collect()
+        lobjs = gc.get_objects()
+        for i in lobjs:
+            if type(i) not in ignore:
+                after[type(i)] += 1
+        log.info("print_leaks:")
+        leaked = {}
+        for k in after:
+            delta = after[k]-before[k]
+            if delta>0:
+                leaked[delta] = k
+        before = after
+        after = defaultdict(int)
+        for delta in reversed(sorted(leaked.keys())):
+            ltype = leaked[delta]
+            matches = [x for x in lobjs if type(x)==ltype and ltype not in ignore]
+            if len(matches)<32:
+                minfo = [str(x)[:32] for x in matches]
+            else:
+                minfo = "%s matches" % len(matches)
+            log.info("%8i : %s : %s", delta, ltype, minfo)
+            if len(matches)<32 and ltype in detailed:
+                frame = inspect.currentframe()
+                exclude = [frame, matches, lobjs]
+                try:
+                    dump_references(log, matches, exclude=exclude)
+                finally:
+                    del frame
+                    del exclude
+            del matches
+            del minfo
+        del lobjs
+        return True
+    return print_leaks
+
+
+def pver(v):
+    #print for lists with version numbers, or CSV strings
+    if type(v) in (list, tuple):
+        types = list(set([type(x) for x in v]))
+        if len(types)==1 and types[0]==int:
+            return ".".join([str(x) for x in v])
+        if len(types)==1 and types[0]==str:
+            return ", ".join(v)
+    return v
 
 def std(_str, extras="-,./ "):
     def f(v):
         return str.isalnum(str(v)) or v in extras
-    return filter(f, _str)
+    return "".join(filter(f, _str))
 
 def alnum(_str):
     def f(v):
         return str.isalnum(str(v))
-    return filter(f, _str)
+    return "".join(filter(f, _str))
 
 def nn(x):
     if x is None:
