@@ -20,17 +20,19 @@ from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@Unresolved
 X11Keyboard = X11KeyboardBindings()
 from xpra.x11.bindings.core_bindings import X11CoreBindings #@UnresolvedImport
 X11Core = X11CoreBindings()
-from xpra.x11.gtk_x11.error import XError, trap
+from xpra.gtk_common.error import XError, trap
 from xpra.server.server_uuid import save_uuid, get_uuid
 
 from xpra.log import Logger
 log = Logger("x11", "server")
 keylog = Logger("x11", "server", "keyboard")
 mouselog = Logger("x11", "server", "mouse")
+grablog = Logger("server", "grab")
 
 from xpra.util import prettify_plug_name
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.x11.xkbhelper import clean_keyboard_state
+from xpra.x11.server_keyboard_config import KeyboardConfig
 
 MAX_CONCURRENT_CONNECTIONS = 20
 
@@ -116,6 +118,11 @@ class X11ServerBase(GTKServerBase):
         clean_keyboard_state()
 
 
+    def init_packet_handlers(self):
+        GTKServerBase.init_packet_handlers(self)
+        self._authenticated_ui_packet_handlers["force-ungrab"] = self._process_force_ungrab
+
+
     def get_uuid(self):
         return get_uuid()
 
@@ -138,7 +145,9 @@ class X11ServerBase(GTKServerBase):
 
     def make_hello(self):
         capabilities = GTKServerBase.make_hello(self)
+        capabilities["server_type"] = "Python/gtk/x11"
         capabilities["resize_screen"] = self.randr
+        capabilities["force_ungrab"] = True
         return capabilities
 
     def do_get_info(self, proto, server_sources, window_ids):
@@ -149,16 +158,40 @@ class X11ServerBase(GTKServerBase):
             info["server.XShm"] = CompositeHelper.XShmEnabled
         except:
             pass
+        #randr:
+        try:
+            sizes = RandR.get_screen_sizes()
+            if self.randr and len(sizes)>=0:
+                info["server.randr.options"] = list(reversed(sorted(sizes)))
+        except:
+            pass
+        try:
+            from xpra.scripts.server import find_fakeXinerama
+            fx = find_fakeXinerama()
+        except:
+            fx = None
+        info["server.fakeXinerama"] = self.fake_xinerama and bool(fx)
+        info["server.libfakeXinerama"] = fx or ""
+        #this is added here because the server keyboard config doesn't know about "keys_pressed"..
+        info["keyboard.state.keys_pressed"] = list(self.keys_pressed.keys())
         return info
 
     def get_window_info(self, window):
         info = GTKServerBase.get_window_info(self, window)
-        if "has-grab" in window.get_property_names():
-            info["has-grab"] = window.get_property("has-grab")
         info["XShm"] = window.uses_XShm()
         return info
 
 
+    def get_keyboard_config(self, props):
+        keyboard_config = KeyboardConfig()
+        keyboard_config.enabled = props.boolget("keyboard", True)
+        keyboard_config.parse_options(props)
+        keyboard_config.xkbmap_layout = props.strget("xkbmap_layout")
+        keyboard_config.xkbmap_variant = props.strget("xkbmap_variant")
+        keylog("get_keyboard_config(..)=%s", keyboard_config)
+        return keyboard_config
+
+
     def set_keymap(self, server_source, force=False):
         try:
             #prevent _keys_changed() from firing:
@@ -233,6 +266,28 @@ class X11ServerBase(GTKServerBase):
         root_w, root_h = gtk.gdk.get_default_root_window().get_size()
         if desired_w==root_w and desired_h==root_h and not self.fake_xinerama:
             return    root_w,root_h    #unlikely: perfect match already!
+        #find the "physical" screen dimensions, so we can calculate the required dpi
+        #(and do this before changing the resolution)
+        wmm, hmm = 0, 0
+        client_w, client_h = 0, 0
+        sss = self._server_sources.values()
+        for ss in sss:
+            for s in ss.screen_sizes:
+                if len(s)>=10:
+                    #display_name, width, height, width_mm, height_mm, monitors, work_x, work_y, work_width, work_height
+                    client_w = max(client_w, s[1])
+                    client_h = max(client_h, s[2])
+                    wmm = max(wmm, s[3])
+                    hmm = max(hmm, s[4])
+        xdpi = self.default_dpi or self.dpi or 96
+        ydpi = self.default_dpi or self.dpi or 96
+        if wmm>0 and hmm>0 and client_w>0 and client_h>0:
+            #calculate "real" dpi using integer calculations:
+            xdpi = client_w * 254 / wmm / 10
+            ydpi = client_h * 254 / hmm / 10
+        log("calculated DPI: %s x %s (from w: %s / %s, h: %s / %s)", xdpi, ydpi, client_w, wmm, client_h, hmm)
+        self.set_dpi(xdpi, ydpi)
+
         #try to find the best screen size to resize to:
         new_size = None
         for w,h in RandR.get_screen_sizes():
@@ -275,7 +330,7 @@ class X11ServerBase(GTKServerBase):
                     log.info("temporarily switching to %sx%s as a Xinerama workaround", tw, th)
                     RandR.set_screen_size(tw, th)
             log.debug("calling RandR.set_screen_size(%s, %s)", w, h)
-            RandR.set_screen_size(w, h)
+            trap.call_synced(RandR.set_screen_size, w, h)
             log.debug("calling RandR.get_screen_size()")
             root_w, root_h = RandR.get_screen_size()
             log.debug("RandR.get_screen_size()=%s,%s", root_w, root_h)
@@ -287,6 +342,19 @@ class X11ServerBase(GTKServerBase):
                 if desired_w!=root_w or desired_h!=root_h:
                     msg += " (best match for %sx%s)" % (desired_w, desired_h)
                 log.info(msg)
+            def show_dpi():
+                sizes_mm = RandR.get_screen_sizes_mm()      #ie: [(1280, 1024)]
+                assert len(sizes_mm)>0
+                wmm = sum([x[0] for x in sizes_mm]) / len(sizes_mm)
+                hmm = sum([x[1] for x in sizes_mm]) / len(sizes_mm)
+                actual_xdpi = int(root_w * 25.4 / wmm + 0.5)
+                actual_ydpi = int(root_h * 25.4 / hmm + 0.5)
+                if actual_xdpi==xdpi and actual_ydpi==ydpi:
+                    log.info("DPI set to %s x %s", xdpi, ydpi)
+                else:
+                    log.info("DPI set to %s x %s (wanted %s x %s)", actual_xdpi, actual_ydpi, xdpi, ydpi)
+            #show dpi via idle_add so server has time to change the screen size (mm)
+            self.idle_add(show_dpi)
         except Exception, e:
             log.error("ouch, failed to set new resolution: %s", e, exc_info=True)
         return  root_w, root_h
@@ -313,8 +381,9 @@ class X11ServerBase(GTKServerBase):
             return oldconf is not None
         if not self.fake_xinerama:
             return delfile(None)
-        if len(self._server_sources)!=1:
-            return delfile("fakeXinerama can only be enabled for a single client")
+        ui_client_count = len([True for s in self._server_sources.values() if s.send_windows])
+        if ui_client_count!=1:
+            return delfile("fakeXinerama can only be enabled for a single client (found %s)", ui_client_count)
         source = self._server_sources.values()[0]
         ss = source.screen_sizes
         if len(ss)==0:
@@ -370,12 +439,25 @@ class X11ServerBase(GTKServerBase):
         settings = packet[1]
         self.update_server_settings(settings)
 
-    def update_server_settings(self, settings):
+    def update_server_settings(self, settings, reset=False):
         #implemented in the X11 xpra server only for now
         #(does not make sense to update a shadow server)
         log("ignoring server settings update in %s", self)
 
 
+    def _process_force_ungrab(self, proto, packet):
+        #ignore the window id: wid = packet[1]
+        grablog("force ungrab from %s", proto)
+        self.X11_ungrab()
+
+    def X11_ungrab(self):
+        grablog("X11_ungrab")
+        def do_X11_ungrab():
+            X11Core.UngrabKeyboard()
+            X11Core.UngrabPointer()
+        trap.call_synced(do_X11_ungrab)
+
+
     def fake_key(self, keycode, press):
         keylog("fake_key(%s, %s)", keycode, press)
         trap.call_synced(X11Keyboard.xtest_fake_key, keycode, press)
